@@ -11,72 +11,70 @@
 
 // Interface headers
 #include "imgLoader.h"
-#include "nsIClassOfService.h"
-#include "nsIConsoleService.h"
-#include "nsIDocShell.h"
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/dom/BindContext.h"
 #include "mozilla/dom/Document.h"
+#include "nsError.h"
+#include "nsIAppShell.h"
+#include "nsIAsyncVerifyRedirectCallback.h"
+#include "nsIClassOfService.h"
+#include "nsIConsoleService.h"
+#include "nsIDocShell.h"
 #include "nsIExternalProtocolHandler.h"
-#include "nsIPermissionManager.h"
 #include "nsIHttpChannel.h"
 #include "nsINestedURI.h"
-#include "nsScriptSecurityManager.h"
-#include "nsIURILoader.h"
+#include "nsIPermissionManager.h"
 #include "nsIScriptChannel.h"
-#include "nsIAsyncVerifyRedirectCallback.h"
-#include "nsIAppShell.h"
 #include "nsIScriptError.h"
+#include "nsIURILoader.h"
+#include "nsScriptSecurityManager.h"
 #include "nsSubDocumentFrame.h"
 
-#include "nsError.h"
-
 // Util headers
 #include "mozilla/Logging.h"
-
+#include "mozilla/Preferences.h"
 #include "nsContentPolicyUtils.h"
 #include "nsContentUtils.h"
 #include "nsDocShellLoadState.h"
 #include "nsGkAtoms.h"
-#include "nsThreadUtils.h"
-#include "nsNetUtil.h"
 #include "nsMimeTypes.h"
-#include "nsStyleUtil.h"
-#include "mozilla/Preferences.h"
+#include "nsNetUtil.h"
 #include "nsQueryObject.h"
+#include "nsStyleUtil.h"
+#include "nsThreadUtils.h"
 
 // Concrete classes
-#include "nsFrameLoader.h"
-
-#include "nsObjectLoadingContent.h"
-
-#include "nsWidgetsCID.h"
+#include "ReferrerInfo.h"
+#include "mozilla/AsyncEventDispatcher.h"
 #include "mozilla/BasicEvents.h"
 #include "mozilla/Components.h"
-#include "mozilla/LoadInfo.h"
-#include "mozilla/dom/BindingUtils.h"
-#include "mozilla/dom/Element.h"
-#include "mozilla/dom/Event.h"
-#include "mozilla/dom/ScriptSettings.h"
-#include "mozilla/AsyncEventDispatcher.h"
 #include "mozilla/EventDispatcher.h"
 #include "mozilla/IMEStateManager.h"
-#include "mozilla/widget/IMEData.h"
+#include "mozilla/LoadInfo.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/StaticPrefs_browser.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/dom/BindingUtils.h"
 #include "mozilla/dom/ContentChild.h"
+#include "mozilla/dom/Element.h"
+#include "mozilla/dom/Event.h"
 #include "mozilla/dom/HTMLEmbedElement.h"
-#include "mozilla/dom/HTMLObjectElementBinding.h"
 #include "mozilla/dom/HTMLObjectElement.h"
+#include "mozilla/dom/HTMLObjectElementBinding.h"
+#include "mozilla/dom/PolicyContainer.h"
+#include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/dom/UserActivation.h"
 #include "mozilla/dom/nsCSPContext.h"
 #include "mozilla/net/DocumentChannel.h"
 #include "mozilla/net/UrlClassifierFeatureFactory.h"
-#include "mozilla/PresShell.h"
-#include "mozilla/ProfilerLabels.h"
-#include "mozilla/StaticPrefs_browser.h"
+#include "mozilla/widget/IMEData.h"
 #include "nsChannelClassifier.h"
 #include "nsFocusManager.h"
-#include "ReferrerInfo.h"
+#include "nsFrameLoader.h"
 #include "nsIEffectiveTLDService.h"
+#include "nsObjectLoadingContent.h"
+#include "nsWidgetsCID.h"
 
 #ifdef XP_WIN
 // Thanks so much, Microsoft! :(
@@ -539,8 +537,8 @@ void nsObjectLoadingContent::MaybeRewriteYoutubeEmbed(nsIURI* aURI,
   }
 
   // See if requester is planning on using the JS API.
-  nsAutoCString uri;
-  nsresult rv = aURI->GetSpec(uri);
+  nsAutoCString prePath;
+  nsresult rv = aURI->GetPrePath(prePath);
   if (NS_FAILED(rv)) {
     return;
   }
@@ -551,10 +549,10 @@ void nsObjectLoadingContent::MaybeRewriteYoutubeEmbed(nsIURI* aURI,
   // URLs, convert the parameters to query in order to make the video load
   // correctly as an iframe. In either case, warn about it in the
   // developer console.
-  int32_t ampIndex = uri.FindChar('&', 0);
+  int32_t ampIndex = path.FindChar('&', 0);
   bool replaceQuery = false;
   if (ampIndex != -1) {
-    int32_t qmIndex = uri.FindChar('?', 0);
+    int32_t qmIndex = path.FindChar('?', 0);
     if (qmIndex == -1 || qmIndex > ampIndex) {
       replaceQuery = true;
     }
@@ -570,19 +568,21 @@ void nsObjectLoadingContent::MaybeRewriteYoutubeEmbed(nsIURI* aURI,
     return;
   }
 
-  nsAutoString utf16OldURI = NS_ConvertUTF8toUTF16(uri);
+  NS_ConvertUTF8toUTF16 utf16OldURI(prePath);
+  AppendUTF8toUTF16(path, utf16OldURI);
   // If we need to convert the URL, it means an ampersand comes first.
   // Use the index we found earlier.
   if (replaceQuery) {
     // Replace question marks with ampersands.
-    uri.ReplaceChar('?', '&');
+    path.ReplaceChar('?', '&');
     // Replace the first ampersand with a question mark.
-    uri.SetCharAt('?', ampIndex);
+    path.SetCharAt('?', ampIndex);
   }
   // Switch out video access url formats, which should possibly allow HTML5
   // video loading.
-  uri.ReplaceSubstring("/v/"_ns, "/embed/"_ns);
-  nsAutoString utf16URI = NS_ConvertUTF8toUTF16(uri);
+  path.ReplaceSubstring("/v/"_ns, "/embed/"_ns);
+  NS_ConvertUTF8toUTF16 utf16URI(prePath);
+  AppendUTF8toUTF16(path, utf16URI);
   rv = nsContentUtils::NewURIWithDocumentCharset(aRewrittenURI, utf16URI, doc,
                                                  aBaseURI);
   if (NS_FAILED(rv)) {
@@ -613,11 +613,15 @@ bool nsObjectLoadingContent::CheckLoadPolicy(int16_t* aContentPolicy) {
 
   nsContentPolicyType contentPolicyType = GetContentPolicyType();
 
-  nsCOMPtr<nsILoadInfo> secCheckLoadInfo =
-      new LoadInfo(doc->NodePrincipal(),  // loading principal
-                   doc->NodePrincipal(),  // triggering principal
-                   el, nsILoadInfo::SEC_ONLY_FOR_EXPLICIT_CONTENTSEC_CHECK,
-                   contentPolicyType);
+  Result<RefPtr<LoadInfo>, nsresult> maybeLoadInfo =
+      LoadInfo::Create(doc->NodePrincipal(),  // loading principal
+                       doc->NodePrincipal(),  // triggering principal
+                       el, nsILoadInfo::SEC_ONLY_FOR_EXPLICIT_CONTENTSEC_CHECK,
+                       contentPolicyType);
+  if (NS_WARN_IF(maybeLoadInfo.isErr())) {
+    return false;
+  }
+  RefPtr<LoadInfo> secCheckLoadInfo = maybeLoadInfo.unwrap();
 
   *aContentPolicy = nsIContentPolicy::ACCEPT;
   nsresult rv =
@@ -653,10 +657,14 @@ bool nsObjectLoadingContent::CheckProcessPolicy(int16_t* aContentPolicy) {
       return false;
   }
 
-  nsCOMPtr<nsILoadInfo> secCheckLoadInfo = new LoadInfo(
+  Result<RefPtr<LoadInfo>, nsresult> maybeLoadInfo = LoadInfo::Create(
       doc->NodePrincipal(),  // loading principal
       doc->NodePrincipal(),  // triggering principal
       el, nsILoadInfo::SEC_ONLY_FOR_EXPLICIT_CONTENTSEC_CHECK, objectType);
+  if (NS_WARN_IF(maybeLoadInfo.isErr())) {
+    return false;
+  }
+  RefPtr<LoadInfo> secCheckLoadInfo = maybeLoadInfo.unwrap();
 
   *aContentPolicy = nsIContentPolicy::ACCEPT;
   nsresult rv = NS_CheckContentProcessPolicy(
@@ -690,7 +698,6 @@ nsObjectLoadingContent::UpdateObjectParameters() {
 
   nsresult rv;
   nsAutoCString newMime;
-  nsAutoString typeAttr;
   nsCOMPtr<nsIURI> newURI;
   nsCOMPtr<nsIURI> newBaseURI;
   ObjectType newType;
@@ -718,7 +725,6 @@ nsObjectLoadingContent::UpdateObjectParameters() {
       el->HasNonEmptyAttr(nsGkAtoms::classid)) {
     // We don't support class ID plugin references, so we should always treat
     // having class Ids as attributes as invalid, and fallback accordingly.
-    newMime.Truncate();
     stateInvalid = true;
   }
 
@@ -726,11 +732,12 @@ nsObjectLoadingContent::UpdateObjectParameters() {
   /// Codebase
   ///
 
-  nsAutoString codebaseStr;
   nsIURI* docBaseURI = el->GetBaseURI();
-  el->GetAttr(nsGkAtoms::codebase, codebaseStr);
 
-  if (!codebaseStr.IsEmpty()) {
+  nsAutoString codebaseStr;
+  el->GetAttr(nsGkAtoms::codebase, codebaseStr);
+  if (StaticPrefs::dom_object_embed_codebase_enabled() &&
+      !codebaseStr.IsEmpty()) {
     rv = nsContentUtils::NewURIWithDocumentCharset(
         getter_AddRefs(newBaseURI), codebaseStr, el->OwnerDoc(), docBaseURI);
     if (NS_FAILED(rv)) {
@@ -747,16 +754,6 @@ nsObjectLoadingContent::UpdateObjectParameters() {
     newBaseURI = docBaseURI;
   }
 
-  nsAutoString rawTypeAttr;
-  el->GetAttr(nsGkAtoms::type, rawTypeAttr);
-  if (!rawTypeAttr.IsEmpty()) {
-    typeAttr = rawTypeAttr;
-    nsAutoString params;
-    nsAutoString mime;
-    nsContentUtils::SplitMimeType(rawTypeAttr, mime, params);
-    CopyUTF16toUTF8(mime, newMime);
-  }
-
   ///
   /// URI
   ///
@@ -791,6 +788,41 @@ nsObjectLoadingContent::UpdateObjectParameters() {
     }
   }
 
+  ///
+  /// type
+  ///
+  nsAutoString rawTypeAttr;
+  el->GetAttr(nsGkAtoms::type, rawTypeAttr);
+  // YouTube embeds might be using type="application/x-shockwave-flash"
+  // which needs to be allowed, but must not override the text/html MIME set
+  // above.
+  if (!mRewrittenYoutubeEmbed && !rawTypeAttr.IsEmpty()) {
+    nsAutoString params;
+    nsAutoString mime;
+    nsContentUtils::SplitMimeType(rawTypeAttr, mime, params);
+
+    if (!StaticPrefs::dom_object_embed_type_hint_enabled()) {
+      NS_ConvertUTF16toUTF8 mimeUTF8(mime);
+      if (imgLoader::SupportImageWithMimeType(mimeUTF8)) {
+        // Normally the type attribute should not be used as a hint, but for
+        // images it does seem to happen in Chrome and Safari. Images generally
+        // don't lead to code execution and we don't use
+        // AcceptedMimeTypes::IMAGES_AND_DOCUMENTS above.
+        newMime = mimeUTF8;
+      } else if (GetTypeOfContent(mimeUTF8) != ObjectType::Document) {
+        LOG(
+            ("OBJLC [%p]: MIME '%s' from type attribute is not supported, "
+             "forcing fallback.",
+             this, mimeUTF8.get()));
+        stateInvalid = true;
+      }
+
+      // Don't use the type attribute as a Content-Type hint in other cases.
+    } else {
+      CopyUTF16toUTF8(mime, newMime);
+    }
+  }
+
   ///
   /// Check if the original (pre-channel) content-type or URI changed, and
   /// record mOriginal{ContentType,URI}
@@ -1166,27 +1198,28 @@ nsresult nsObjectLoadingContent::LoadObject(bool aNotify, bool aForceLoad,
     }
   }
 
-  // Don't allow view-source scheme.
-  // view-source is the only scheme to which this applies at the moment due to
-  // potential timing attacks to read data from cross-origin documents. If this
-  // widens we should add a protocol flag for whether the scheme is only allowed
-  // in top and use something like nsNetUtil::NS_URIChainHasFlags.
-  if (mType != ObjectType::Fallback) {
-    nsCOMPtr<nsIURI> tempURI = mURI;
-    nsCOMPtr<nsINestedURI> nestedURI = do_QueryInterface(tempURI);
-    while (nestedURI) {
-      // view-source should always be an nsINestedURI, loop and check the
-      // scheme on this and all inner URIs that are also nested URIs.
-      if (tempURI->SchemeIs("view-source")) {
-        LOG(("OBJLC [%p]: Blocking as effective URI has view-source scheme",
-             this));
-        mType = ObjectType::Fallback;
+  // https://html.spec.whatwg.org/multipage/iframe-embed-object.html#the-object-element
+  // requires that `embed` and `object` go through `Fetch` with mode=navigate,
+  // see 1.3.5. This will in https://fetch.spec.whatwg.org/#fetching plumb us
+  // through to https://fetch.spec.whatwg.org/#concept-main-fetch where in step
+  // 12 a switch is performed. Since `object` and `embed` have mode=navigate the
+  // result of https://fetch.spec.whatwg.org/#concept-scheme-fetch will decide
+  // if main fetch proceeds. We short-circuit that scheme-fetch here, inspecting
+  // if the scheme of `mURI` is one that would return a network error. The
+  // following schemes are allowed through in scheme fetch:
+  // "about", "blob", "data", "file", "http", "https".
+  //
+  // Some accessibility tests use our internal "chrome" scheme.
+  if (mType != ObjectType::Fallback && mURI) {
+    ObjectType type = ObjectType::Fallback;
+    for (const auto& candidate :
+         {"about", "blob", "chrome", "data", "file", "http", "https"}) {
+      if (mURI->SchemeIs(candidate)) {
+        type = mType;
         break;
       }
-
-      nestedURI->GetInnerURI(getter_AddRefs(tempURI));
-      nestedURI = do_QueryInterface(tempURI);
     }
+    mType = type;
   }
 
   // Items resolved as Image/Document are not candidates for content blocking,
@@ -1402,7 +1435,7 @@ nsresult nsObjectLoadingContent::OpenChannel() {
       true,                 // aInheritForAboutBlank
       false);               // aForceInherit
 
-  bool inheritPrincipal = inheritAttrs && !SchemeIsData(mURI);
+  bool inheritPrincipal = inheritAttrs && !mURI->SchemeIs("data");
 
   nsSecurityFlags securityFlags =
       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL;
@@ -1419,21 +1452,23 @@ nsresult nsObjectLoadingContent::OpenChannel() {
                           nsIRequest::LOAD_HTML_OBJECT_DATA;
   uint32_t sandboxFlags = doc->GetSandboxFlags();
 
-  // For object loads we store the CSP that potentially needs to
+  // For object loads we store the policyContainer that potentially needs to
   // be inherited, e.g. in case we are loading an opaque origin
   // like a data: URI. The actual inheritance check happens within
-  // Document::InitCSP(). Please create an actual copy of the CSP
-  // (do not share the same reference) otherwise a Meta CSP of an
-  // opaque origin will incorrectly be propagated to the embedding
-  // document.
-  RefPtr<nsCSPContext> cspToInherit;
-  if (nsCOMPtr<nsIContentSecurityPolicy> csp = doc->GetCsp()) {
-    cspToInherit = new nsCSPContext();
-    cspToInherit->InitFromOther(static_cast<nsCSPContext*>(csp.get()));
+  // Document::InitPolicyContainer(). Please create an actual copy of the
+  // policyContainer (do not share the same reference) otherwise modifications
+  // done (such as the meta CSP of the new doc) in an opaque origin will
+  // incorrectly be propagated to the embedding document.
+  RefPtr<PolicyContainer> policyContainerToInherit;
+  if (nsCOMPtr<nsIPolicyContainer> policyContainer =
+          doc->GetPolicyContainer()) {
+    policyContainerToInherit = new PolicyContainer();
+    policyContainerToInherit->InitFromOther(
+        PolicyContainer::Cast(policyContainer.get()));
   }
 
   // --- Create LoadInfo
-  RefPtr<LoadInfo> loadInfo = new LoadInfo(
+  RefPtr<LoadInfo> loadInfo = MOZ_TRY(LoadInfo::Create(
       /*aLoadingPrincipal = aLoadingContext->NodePrincipal() */ nullptr,
       /*aTriggeringPrincipal = aLoadingPrincipal */ nullptr,
       /*aLoadingContext = */ el,
@@ -1441,14 +1476,21 @@ nsresult nsObjectLoadingContent::OpenChannel() {
       /*aContentPolicyType = */ contentPolicyType,
       /*aLoadingClientInfo = */ Nothing(),
       /*aController = */ Nothing(),
-      /*aSandboxFlags = */ sandboxFlags);
+      /*aSandboxFlags = */ sandboxFlags));
 
   if (inheritAttrs) {
     loadInfo->SetPrincipalToInherit(el->NodePrincipal());
   }
 
-  if (cspToInherit) {
-    loadInfo->SetCSPToInherit(cspToInherit);
+  // For object loads we store the policyContainer that potentially needs to
+  // be inherited, e.g. in case we are loading an opaque origin
+  // like a data: URI. The actual inheritance check happens within
+  // Document::InitPolicyContainer(). Please create an actual copy of the
+  // policyContainer (do not share the same reference) otherwise modifications
+  // done (such as the meta CSP of the new doc) in an opaque origin will
+  // incorrectly be propagated to the embedding document.
+  if (policyContainerToInherit) {
+    loadInfo->SetPolicyContainerToInherit(policyContainerToInherit);
   }
 
   if (DocumentChannel::CanUseDocumentChannel(mURI) &&
@@ -1458,8 +1500,8 @@ nsresult nsObjectLoadingContent::OpenChannel() {
     RefPtr<nsDocShellLoadState> loadState = new nsDocShellLoadState(mURI);
     loadState->SetPrincipalToInherit(el->NodePrincipal());
     loadState->SetTriggeringPrincipal(loadInfo->TriggeringPrincipal());
-    if (cspToInherit) {
-      loadState->SetCsp(cspToInherit);
+    if (policyContainerToInherit) {
+      loadState->SetPolicyContainer(policyContainerToInherit);
     }
     loadState->SetTriggeringSandboxFlags(sandboxFlags);
 
@@ -1497,36 +1539,17 @@ nsresult nsObjectLoadingContent::OpenChannel() {
                                loadFlags,             // aLoadFlags
                                nullptr);              // aIoService
     NS_ENSURE_SUCCESS(rv, rv);
-
-    if (inheritAttrs) {
-      nsCOMPtr<nsILoadInfo> loadinfo = chan->LoadInfo();
-      loadinfo->SetPrincipalToInherit(el->NodePrincipal());
-    }
-
-    // For object loads we store the CSP that potentially needs to
-    // be inherited, e.g. in case we are loading an opaque origin
-    // like a data: URI. The actual inheritance check happens within
-    // Document::InitCSP(). Please create an actual copy of the CSP
-    // (do not share the same reference) otherwise a Meta CSP of an
-    // opaque origin will incorrectly be propagated to the embedding
-    // document.
-    if (cspToInherit) {
-      nsCOMPtr<nsILoadInfo> loadinfo = chan->LoadInfo();
-      static_cast<LoadInfo*>(loadinfo.get())->SetCSPToInherit(cspToInherit);
-    }
   };
 
   // Referrer
-  nsCOMPtr<nsIHttpChannel> httpChan(do_QueryInterface(chan));
-  if (httpChan) {
+  if (nsCOMPtr<nsIHttpChannel> httpChan = do_QueryInterface(chan)) {
     auto referrerInfo = MakeRefPtr<ReferrerInfo>(*doc);
 
     rv = httpChan->SetReferrerInfoWithoutClone(referrerInfo);
     MOZ_ASSERT(NS_SUCCEEDED(rv));
 
     // Set the initiator type
-    nsCOMPtr<nsITimedChannel> timedChannel(do_QueryInterface(httpChan));
-    if (timedChannel) {
+    if (nsCOMPtr<nsITimedChannel> timedChannel = do_QueryInterface(httpChan)) {
       timedChannel->SetInitiatorType(el->LocalName());
     }
 
@@ -1536,12 +1559,6 @@ nsresult nsObjectLoadingContent::OpenChannel() {
     }
   }
 
-  nsCOMPtr<nsIScriptChannel> scriptChannel = do_QueryInterface(chan);
-  if (scriptChannel) {
-    // Allow execution against our context if the principals match
-    scriptChannel->SetExecutionPolicy(nsIScriptChannel::EXECUTE_NORMAL);
-  }
-
   // AsyncOpen can fail if a file does not exist.
   rv = chan->AsyncOpen(shim);
   NS_ENSURE_SUCCESS(rv, rv);
