@@ -21,9 +21,11 @@
 #include "mozilla/net/CookieCommons.h"
 #include "mozilla/net/CookiePrefixes.h"
 #include "mozilla/net/NeckoChannelParams.h"
+#include "nsGlobalWindowInner.h"
 #include "nsICookie.h"
 #include "nsIGlobalObject.h"
 #include "nsIPrincipal.h"
+#include "nsIURL.h"
 #include "nsReadableUtils.h"
 #include "nsSandboxFlags.h"
 
@@ -183,15 +185,42 @@ bool ValidateCookieDomain(nsIPrincipal* aPrincipal, const nsAString& aName,
   return true;
 }
 
-bool ValidateCookiePath(const nsAString& aPath, Promise* aPromise) {
+bool ValidateCookiePath(nsIURI* aURI, const nsAString& aPath,
+                        nsAString& aRetPath, Promise* aPromise) {
+  MOZ_ASSERT(aURI);
   MOZ_ASSERT(aPromise);
 
-  if (!aPath.IsEmpty() && aPath[0] != '/') {
+  aRetPath = aPath;
+
+  if (aRetPath.IsEmpty()) {
+    nsCOMPtr<nsIURL> url = do_QueryInterface(aURI);
+
+    if (!url) {
+      aPromise->MaybeRejectWithNotAllowedError("Permission denied");
+      return false;
+    }
+
+    nsAutoCString directory;
+    nsresult rv = url->GetDirectory(directory);
+
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      aPromise->MaybeRejectWithNotAllowedError("Permission denied");
+      return false;
+    }
+
+    if (!directory.IsEmpty() && directory.Last() == '/') {
+      directory.Truncate(directory.Length() - 1);
+    }
+
+    CopyUTF8toUTF16(directory, aRetPath);
+  }
+
+  if (aRetPath[0] != '/') {
     aPromise->MaybeRejectWithTypeError("Cookie path must start with '/'");
     return false;
   }
 
-  if (aPath.Length() > 1024) {
+  if (aRetPath.Length() > 1024) {
     aPromise->MaybeRejectWithTypeError(
         "Cookie domain size cannot be greater than 1024 bytes");
     return false;
@@ -417,6 +446,8 @@ already_AddRefed<Promise> CookieStore::Set(const CookieInit& aOptions,
   NS_DispatchToCurrentThread(NS_NewRunnableFunction(
       __func__, [self = RefPtr(this), promise = RefPtr(promise), aOptions,
                  cookiePrincipal = RefPtr(cookiePrincipal.get())]() {
+        nsCOMPtr<nsIURI> cookieURI = cookiePrincipal->GetURI();
+
         nsString name(TrimTabAndSpace(aOptions.mName));
         nsString value(TrimTabAndSpace(aOptions.mValue));
 
@@ -430,12 +461,12 @@ already_AddRefed<Promise> CookieStore::Set(const CookieInit& aOptions,
           return;
         }
 
-        if (!ValidateCookiePath(aOptions.mPath, promise)) {
+        nsString path;
+        if (!ValidateCookiePath(cookieURI, aOptions.mPath, path, promise)) {
           return;
         }
 
-        if (!ValidateCookieNamePrefix(name, value, domain, aOptions.mPath,
-                                      promise)) {
+        if (!ValidateCookieNamePrefix(name, value, domain, path, promise)) {
           return;
         }
 
@@ -470,7 +501,6 @@ already_AddRefed<Promise> CookieStore::Set(const CookieInit& aOptions,
         self->mNotificationWatcher->ResolvePromiseWhenNotified(operationID,
                                                                promise);
 
-        nsCOMPtr<nsIURI> cookieURI = cookiePrincipal->GetURI();
         RefPtr<CookieStoreChild::SetRequestPromise> ipcPromise =
             self->mActor->SendSetRequest(
                 mozilla::WrapNotNull(cookieURI.get()),
@@ -479,7 +509,7 @@ already_AddRefed<Promise> CookieStore::Set(const CookieInit& aOptions,
                 name, value,
                 // If expires is not set, it's a session cookie.
                 aOptions.mExpires.IsNull(), ComputeExpiry(aOptions), domain,
-                aOptions.mPath, SameSiteToConst(aOptions.mSameSite),
+                path, SameSiteToConst(aOptions.mSameSite),
                 aOptions.mPartitioned, operationID);
         if (NS_WARN_IF(!ipcPromise)) {
           promise->MaybeResolveWithUndefined();
@@ -555,18 +585,21 @@ already_AddRefed<Promise> CookieStore::Delete(
   NS_DispatchToCurrentThread(NS_NewRunnableFunction(
       __func__, [self = RefPtr(this), promise = RefPtr(promise), aOptions,
                  cookiePrincipal = RefPtr(cookiePrincipal.get())]() {
+        nsCOMPtr<nsIURI> cookieURI = cookiePrincipal->GetURI();
+        nsString name(TrimTabAndSpace(aOptions.mName));
+
         nsString domain;
-        if (!ValidateCookieDomain(cookiePrincipal, aOptions.mName,
-                                  aOptions.mDomain, domain, promise)) {
+        if (!ValidateCookieDomain(cookiePrincipal, name, aOptions.mDomain,
+                                  domain, promise)) {
           return;
         }
 
-        if (!ValidateCookiePath(aOptions.mPath, promise)) {
+        nsString path;
+        if (!ValidateCookiePath(cookieURI, aOptions.mPath, path, promise)) {
           return;
         }
 
-        if (!ValidateCookieNamePrefix(aOptions.mName, u""_ns, domain,
-                                      aOptions.mPath, promise)) {
+        if (!ValidateCookieNamePrefix(name, u""_ns, domain, path, promise)) {
           return;
         }
 
@@ -600,14 +633,12 @@ already_AddRefed<Promise> CookieStore::Delete(
 
         self->mNotificationWatcher->ResolvePromiseWhenNotified(operationID,
                                                                promise);
-        nsCOMPtr<nsIURI> cookieURI = cookiePrincipal->GetURI();
         RefPtr<CookieStoreChild::DeleteRequestPromise> ipcPromise =
             self->mActor->SendDeleteRequest(
                 mozilla::WrapNotNull(cookieURI.get()),
                 cookiePrincipal->OriginAttributesRef(), thirdPartyContext,
                 partitionForeign, usingStorageAccess, isOn3PCBExceptionList,
-                aOptions.mName, domain, aOptions.mPath, aOptions.mPartitioned,
-                operationID);
+                name, domain, path, aOptions.mPartitioned, operationID);
         if (NS_WARN_IF(!ipcPromise)) {
           promise->MaybeResolveWithUndefined();
           return;
@@ -699,7 +730,7 @@ already_AddRefed<Promise> CookieStore::GetInternal(
        aOnlyTheFirstMatch]() {
         nsAutoString name;
         if (aOptions.mName.WasPassed()) {
-          name = aOptions.mName.Value();
+          name = TrimTabAndSpace(aOptions.mName.Value());
         }
 
         nsAutoCString path;
