@@ -8,6 +8,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/dom/HTMLMediaElement.h"
 #include "mozilla/net/OpaqueResponseUtils.h"
+#include "nsHttp.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsICachingChannel.h"
 #include "nsIClassOfService.h"
@@ -15,7 +16,6 @@
 #include "nsIInputStream.h"
 #include "nsIThreadRetargetableRequest.h"
 #include "nsITimedChannel.h"
-#include "nsHttp.h"
 #include "nsNetUtil.h"
 
 static const uint32_t HTTP_PARTIAL_RESPONSE_CODE = 206;
@@ -58,14 +58,16 @@ NS_IMPL_ISUPPORTS(ChannelMediaResource::Listener, nsIRequestObserver,
                   nsIThreadRetargetableStreamListener)
 
 nsresult ChannelMediaResource::Listener::OnStartRequest(nsIRequest* aRequest) {
-  mMutex.AssertOnWritingThread();  // Writing thread is MainThread
+  AssertIsOnMainThread();
+  mLock.NoteOnMainThread();
   if (!mResource) return NS_OK;
   return mResource->OnStartRequest(aRequest, mOffset);
 }
 
 nsresult ChannelMediaResource::Listener::OnStopRequest(nsIRequest* aRequest,
                                                        nsresult aStatus) {
-  mMutex.AssertOnWritingThread();
+  AssertIsOnMainThread();
+  mLock.NoteOnMainThread();
   if (!mResource) return NS_OK;
   return mResource->OnStopRequest(aRequest, aStatus);
 }
@@ -76,7 +78,8 @@ nsresult ChannelMediaResource::Listener::OnDataAvailable(
   // This might happen off the main thread.
   RefPtr<ChannelMediaResource> res;
   {
-    MutexSingleWriterAutoLock lock(mMutex);
+    MutexAutoLock lock(mLock.Lock());
+    mLock.NoteLockHeld();
     res = mResource;
   }
   // Note Rekove() might happen at the same time to reset mResource. We check
@@ -87,7 +90,8 @@ nsresult ChannelMediaResource::Listener::OnDataAvailable(
 nsresult ChannelMediaResource::Listener::AsyncOnChannelRedirect(
     nsIChannel* aOld, nsIChannel* aNew, uint32_t aFlags,
     nsIAsyncVerifyRedirectCallback* cb) {
-  mMutex.AssertOnWritingThread();
+  AssertIsOnMainThread();
+  mLock.NoteOnMainThread();
 
   nsresult rv = NS_OK;
   if (mResource) {
@@ -113,8 +117,10 @@ nsresult ChannelMediaResource::Listener::GetInterface(const nsIID& aIID,
 }
 
 void ChannelMediaResource::Listener::Revoke() {
-  MOZ_ASSERT(NS_IsMainThread());
-  MutexSingleWriterAutoLock lock(mMutex);
+  AssertIsOnMainThread();
+  MutexAutoLock lock(mLock.Lock());
+  mLock.NoteExclusiveAccess();
+
   mResource = nullptr;
 }
 
@@ -351,6 +357,9 @@ nsresult ChannelMediaResource::OnStopRequest(nsIRequest* aRequest,
   // But don't reopen if we need to seek and we don't think we can... that would
   // cause us to just re-read the stream, which would be really bad.
   /*
+   * The conditions below were added in bug 522114 (offset 0 or seekable check)
+   * and bug 1373618 (offset != length check).
+   *
    * | length |    offset |   reopen |
    * +--------+-----------+----------+
    * |     -1 |         0 |      yes |
@@ -365,7 +374,12 @@ nsresult ChannelMediaResource::OnStopRequest(nsIRequest* aRequest,
    * +--------+-----------+----------+
    * |    > 0 | == length |       no |
    */
-  if (aStatus != NS_ERROR_PARSED_DATA_CACHED && aStatus != NS_BINDING_ABORTED) {
+  // Seek() below calls into OpenChannel(), which would fail in
+  // SetupChannelHeaders() with non-http channels.
+  nsCOMPtr<nsIHttpChannel> hc = do_QueryInterface(mChannel);
+  if (aStatus != NS_ERROR_PARSED_DATA_CACHED && aStatus != NS_BINDING_ABORTED &&
+      hc) {
+    // TODO: Move this logic to NotifyDataEnded, see bug 1464045.
     auto lengthAndOffset = mCacheStream.GetLengthAndOffset();
     int64_t length = lengthAndOffset.mLength;
     int64_t offset = lengthAndOffset.mOffset;
