@@ -24,6 +24,7 @@
 #include "mozilla/dom/MediaKeySystemAccessBinding.h"
 #include "mozilla/dom/MediaKeySystemAccessManager.h"
 #include "mozilla/dom/MediaSource.h"
+#include "mozilla/dom/PContent.h"
 #include "nsDOMString.h"
 #include "nsIObserverService.h"
 #include "nsMimeTypes.h"
@@ -41,6 +42,12 @@
 
 namespace mozilla::dom {
 
+#ifdef MOZ_WMF_CDM
+#  include "nsIWindowsMediaFoundationCDMOriginsListService.h"
+
+MOZ_RUNINIT static nsTArray<IPCOriginStatusEntry> sOriginStatusEntries;
+#endif
+
 #define LOG(msg, ...) \
   EME_LOG("MediaKeySystemAccess::%s " msg, __func__, ##__VA_ARGS__)
 
@@ -230,17 +237,39 @@ static KeySystemConfig::EMECodecString ToEMEAPICodecString(
 }
 
 #ifdef MOZ_WMF_CDM
-// TODO : we implement a temporary workaround to explicitly allow/block
-// domains for MFCDM capabilities. This workaround and should be removed after
-// fixing the bug 1901334, which could result in showing black frame for MFCDM
-// playback when specific CSS effects are applied on video, which requires
-// altering pixel's content.
+/* static */
+void MediaKeySystemAccess::UpdateMFCDMOriginEntries(
+    const nsTArray<IPCOriginStatusEntry>& aEntries) {
+  MOZ_ASSERT(NS_IsMainThread());
+  static bool sXPCOMShutdown = false;
+  if (sXPCOMShutdown) {
+    EME_LOG("XPCOM shutdown detected; entry update aborted");
+    return;
+  }
+  sOriginStatusEntries.Clear();
+  sOriginStatusEntries.AppendElements(aEntries);
+  EME_LOG("UpdateMFCDMOriginEntries");
+  for (const auto& entry : sOriginStatusEntries) {
+    EME_LOG("-- Origin: %s, Status: %d\n", entry.origin().get(),
+            entry.status());
+  }
+  RunOnShutdown([&] {
+    sOriginStatusEntries.Clear();
+    sXPCOMShutdown = true;
+  });
+}
+
 static bool IsMFCDMAllowedByOrigin(const Maybe<nsCString>& aOrigin) {
-  // 0 : disabled, 1 : enabled allowed list, 2 : enabled blocked list
+  // TODO: Remove hardcoded allowed and blocked lists once the Remote Settings
+  // lists are verified to be reliable in bug 1964811.
+  // 0 : disabled, 1 : enabled allowed list, 2 : enabled blocked list,
+  // 3 : enabled list status control via Remote Setting
   enum Filer : uint32_t {
     eDisable = 0,
     eAllowedListEnabled = 1,
     eBlockedListEnabled = 2,
+    eAllowedByDefaultRemoteSettings = 3,
+    eBlockedByDefaultRemoteSettings = 4,
   };
   const auto prefValue = StaticPrefs::media_eme_mfcdm_origin_filter_enabled();
   if (prefValue == Filer::eDisable || !aOrigin ||
@@ -271,25 +300,47 @@ static bool IsMFCDMAllowedByOrigin(const Maybe<nsCString>& aOrigin) {
   }
 
   // Check if the origin is blocked to use MFCDM.
-  MOZ_ASSERT(prefValue == Filer::eBlockedListEnabled);
-  static nsTArray<nsCString> kBlockedOrigins({
-      "https://on.orf.at"_ns,
-      "https://www.hulu.com"_ns,
-  });
-  for (const auto& blockedOrigin : kBlockedOrigins) {
-    if (FindInReadable(blockedOrigin, *aOrigin)) {
-      EME_LOG(
-          "MediaKeySystemAccess::IsMFCDMAllowedByOrigin, origin (%s) "
-          "is BLOCKED to use MFCDM",
-          aOrigin->get());
-      return false;
+  if (prefValue == Filer::eBlockedListEnabled) {
+    static nsTArray<nsCString> kBlockedOrigins({
+        "https://on.orf.at"_ns,
+        "https://www.hulu.com"_ns,
+    });
+    for (const auto& blockedOrigin : kBlockedOrigins) {
+      if (FindInReadable(blockedOrigin, *aOrigin)) {
+        EME_LOG(
+            "MediaKeySystemAccess::IsMFCDMAllowedByOrigin, origin (%s) "
+            "is BLOCKED to use MFCDM",
+            aOrigin->get());
+        return false;
+      }
+    }
+    EME_LOG(
+        "MediaKeySystemAccess::IsMFCDMAllowedByOrigin, origin (%s) "
+        "is allowed to use MFCDM",
+        aOrigin->get());
+    return true;
+  }
+
+  // List from Remote Settings. No duplicated origins; suborigins follow the
+  // main origin's result.
+  bool isAllowed = prefValue == Filer::eAllowedByDefaultRemoteSettings;
+  bool isFound = false;
+  for (const auto& entry : sOriginStatusEntries) {
+    // Check if the given origin matches the entry, or if it's a suborigin.
+    if (FindInReadable(entry.origin(), *aOrigin)) {
+      isAllowed =
+          entry.status() ==
+          nsIWindowsMediaFoundationCDMOriginsListService::ORIGIN_ALLOWED;
+      isFound = true;
+      break;
     }
   }
   EME_LOG(
       "MediaKeySystemAccess::IsMFCDMAllowedByOrigin, origin (%s) "
-      "is allowed to use MFCDM",
-      aOrigin->get());
-  return true;
+      "is %s to use MFCDM %s(Remote)",
+      aOrigin->get(), isAllowed ? "ALLOWED" : "BLOCKED",
+      isFound ? "" : "by default ");
+  return isAllowed;
 }
 #endif
 
