@@ -388,6 +388,8 @@ void CamerasChild::RemoveCallback(const int capture_id) {
 
 int CamerasChild::StartCapture(CaptureEngine aCapEngine, const int capture_id,
                                const webrtc::VideoCaptureCapability& webrtcCaps,
+                               const NormalizedConstraints& constraints,
+                               const dom::VideoResizeModeEnum& resize_mode,
                                FrameRelay* cb) {
   LOG(("%s", __PRETTY_FUNCTION__));
   AddCallback(capture_id, cb);
@@ -395,9 +397,12 @@ int CamerasChild::StartCapture(CaptureEngine aCapEngine, const int capture_id,
       webrtcCaps.width, webrtcCaps.height, webrtcCaps.maxFPS,
       static_cast<int>(webrtcCaps.videoType), webrtcCaps.interlaced);
   nsCOMPtr<nsIRunnable> runnable =
-      mozilla::NewRunnableMethod<CaptureEngine, int, VideoCaptureCapability>(
+      mozilla::NewRunnableMethod<CaptureEngine, int, VideoCaptureCapability,
+                                 NormalizedConstraints,
+                                 dom::VideoResizeModeEnum>(
           "camera::PCamerasChild::SendStartCapture", this,
-          &CamerasChild::SendStartCapture, aCapEngine, capture_id, capCap);
+          &CamerasChild::SendStartCapture, aCapEngine, capture_id, capCap,
+          constraints, resize_mode);
   LockAndDispatch<> dispatcher(this, __func__, runnable, -1, mZero);
   return dispatcher.ReturnValue();
 }
