@@ -4,17 +4,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Components.h"
+#include "nsHTTPSOnlyUtils.h"
+
 #include "mozilla/ClearOnShutdown.h"
-#include "mozilla/TimeStamp.h"
-#include "mozilla/glean/DomSecurityMetrics.h"
+#include "mozilla/Components.h"
 #include "mozilla/NullPrincipal.h"
 #include "mozilla/OriginAttributes.h"
 #include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/TimeStamp.h"
+#include "mozilla/glean/DomSecurityMetrics.h"
 #include "mozilla/net/DNS.h"
 #include "nsContentUtils.h"
 #include "nsDNSPrefetch.h"
-#include "nsHTTPSOnlyUtils.h"
 #include "nsIEffectiveTLDService.h"
 #include "nsIHttpChannel.h"
 #include "nsIHttpChannelInternal.h"
@@ -29,40 +30,34 @@
 #include "prnetdb.h"
 
 /* static */
-bool nsHTTPSOnlyUtils::IsHttpsOnlyModeEnabled(bool aFromPrivateWindow) {
-  // if the general pref is set to true, then we always return
-  if (mozilla::StaticPrefs::dom_security_https_only_mode()) {
-    return true;
+nsHTTPSOnlyUtils::UpgradeMode nsHTTPSOnlyUtils::GetUpgradeMode(
+    bool aFromPrivateWindow,
+    nsILoadInfo::SchemelessInputType aSchemelessInputType) {
+  if (mozilla::StaticPrefs::dom_security_https_only_mode() ||
+      (aFromPrivateWindow &&
+       mozilla::StaticPrefs::dom_security_https_only_mode_pbm())) {
+    return nsHTTPSOnlyUtils::HTTPS_ONLY_MODE;
   }
 
-  // otherwise we check if executing in private browsing mode and return true
-  // if the PBM pref for HTTPS-Only is set.
-  if (aFromPrivateWindow &&
-      mozilla::StaticPrefs::dom_security_https_only_mode_pbm()) {
-    return true;
+  if (mozilla::StaticPrefs::dom_security_https_first() ||
+      (aFromPrivateWindow &&
+       mozilla::StaticPrefs::dom_security_https_first_pbm())) {
+    return nsHTTPSOnlyUtils::HTTPS_FIRST_MODE;
   }
-  return false;
+
+  if (mozilla::StaticPrefs::dom_security_https_first_schemeless() &&
+      aSchemelessInputType == nsILoadInfo::SchemelessInputTypeSchemeless) {
+    return nsHTTPSOnlyUtils::SCHEMELESS_HTTPS_FIRST_MODE;
+  }
+
+  return NO_UPGRADE_MODE;
 }
 
 /* static */
-bool nsHTTPSOnlyUtils::IsHttpsFirstModeEnabled(bool aFromPrivateWindow) {
-  // HTTPS-Only takes priority over HTTPS-First
-  if (IsHttpsOnlyModeEnabled(aFromPrivateWindow)) {
-    return false;
-  }
-
-  // if the general pref is set to true, then we always return
-  if (mozilla::StaticPrefs::dom_security_https_first()) {
-    return true;
-  }
-
-  // otherwise we check if executing in private browsing mode and return true
-  // if the PBM pref for HTTPS-First is set.
-  if (aFromPrivateWindow &&
-      mozilla::StaticPrefs::dom_security_https_first_pbm()) {
-    return true;
-  }
-  return false;
+nsHTTPSOnlyUtils::UpgradeMode nsHTTPSOnlyUtils::GetUpgradeMode(
+    nsILoadInfo* aLoadInfo) {
+  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
+  return GetUpgradeMode(isPrivateWin, aLoadInfo->GetSchemelessInput());
 }
 
 /* static */
@@ -81,15 +76,11 @@ void nsHTTPSOnlyUtils::PotentiallyFireHttpRequestToShortenTimout(
   }
 
   nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
-  bool isPrivateWin = loadInfo->GetOriginAttributes().IsPrivateBrowsing();
+  UpgradeMode upgradeMode = GetUpgradeMode(loadInfo);
 
   // if neither HTTPS-Only nor HTTPS-First mode is enabled, then there is
   // nothing to do here.
-  if ((!IsHttpsOnlyModeEnabled(isPrivateWin) &&
-       !IsHttpsFirstModeEnabled(isPrivateWin)) &&
-      !(loadInfo->GetSchemelessInput() ==
-            nsILoadInfo::SchemelessInputTypeSchemeless &&
-        mozilla::StaticPrefs::dom_security_https_first_schemeless())) {
+  if (upgradeMode == NO_UPGRADE_MODE) {
     return;
   }
 
@@ -132,10 +123,8 @@ void nsHTTPSOnlyUtils::PotentiallyFireHttpRequestToShortenTimout(
   // HTTPS-First is enabled, but HTTPS-Only is not enabled, we might return
   // early if attempting to send a background request to a non standard port.
   if (!mozilla::StaticPrefs::dom_security_https_first_for_custom_ports() &&
-      (IsHttpsFirstModeEnabled(isPrivateWin) ||
-       (loadInfo->GetSchemelessInput() ==
-            nsILoadInfo::SchemelessInputTypeSchemeless &&
-        mozilla::StaticPrefs::dom_security_https_first_schemeless()))) {
+      (upgradeMode == HTTPS_FIRST_MODE ||
+       upgradeMode == SCHEMELESS_HTTPS_FIRST_MODE)) {
     int32_t port = 0;
     nsresult rv = channelURI->GetPort(&port);
     int defaultPortforScheme = NS_GetDefaultPort("http");
@@ -158,8 +147,7 @@ void nsHTTPSOnlyUtils::PotentiallyFireHttpRequestToShortenTimout(
 bool nsHTTPSOnlyUtils::ShouldUpgradeRequest(nsIURI* aURI,
                                             nsILoadInfo* aLoadInfo) {
   // 1. Check if the HTTPS-Only Mode is even enabled, before we do anything else
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  if (!IsHttpsOnlyModeEnabled(isPrivateWin)) {
+  if (GetUpgradeMode(aLoadInfo) != HTTPS_ONLY_MODE) {
     return false;
   }
 
@@ -183,7 +171,8 @@ bool nsHTTPSOnlyUtils::ShouldUpgradeRequest(nsIURI* aURI,
   ExtContentPolicyType contentType = aLoadInfo->GetExternalContentPolicyType();
   if (contentType != ExtContentPolicy::TYPE_DOCUMENT) {
     if (!aLoadInfo->TriggeringPrincipal()->IsSystemPrincipal() &&
-        TestIfPrincipalIsExempt(aLoadInfo->TriggeringPrincipal())) {
+        TestIfPrincipalIsExempt(aLoadInfo->TriggeringPrincipal(),
+                                HTTPS_ONLY_MODE)) {
       return false;
     }
   }
@@ -226,8 +215,7 @@ bool nsHTTPSOnlyUtils::ShouldUpgradeRequest(nsIURI* aURI,
 bool nsHTTPSOnlyUtils::ShouldUpgradeWebSocket(nsIURI* aURI,
                                               nsILoadInfo* aLoadInfo) {
   // 1. Check if the HTTPS-Only Mode is even enabled, before we do anything else
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  if (!IsHttpsOnlyModeEnabled(isPrivateWin)) {
+  if (GetUpgradeMode(aLoadInfo) != HTTPS_ONLY_MODE) {
     return false;
   }
 
@@ -250,7 +238,8 @@ bool nsHTTPSOnlyUtils::ShouldUpgradeWebSocket(nsIURI* aURI,
 
   // All subresources of an exempt triggering principal are also exempt.
   if (!aLoadInfo->TriggeringPrincipal()->IsSystemPrincipal() &&
-      TestIfPrincipalIsExempt(aLoadInfo->TriggeringPrincipal())) {
+      TestIfPrincipalIsExempt(aLoadInfo->TriggeringPrincipal(),
+                              HTTPS_ONLY_MODE)) {
     return false;
   }
 
@@ -275,13 +264,13 @@ bool nsHTTPSOnlyUtils::IsUpgradeDowngradeEndlessLoop(
     const mozilla::EnumSet<UpgradeDowngradeEndlessLoopOptions>& aOptions) {
   // 1. Check if the HTTPS-Only/HTTPS-First is even enabled, before doing
   // anything else
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
+  UpgradeMode upgradeMode = GetUpgradeMode(aLoadInfo);
   bool enforceForHTTPSOnlyMode =
-      IsHttpsOnlyModeEnabled(isPrivateWin) &&
+      upgradeMode == HTTPS_ONLY_MODE &&
       aOptions.contains(
           UpgradeDowngradeEndlessLoopOptions::EnforceForHTTPSOnlyMode);
   bool enforceForHTTPSFirstMode =
-      IsHttpsFirstModeEnabled(isPrivateWin) &&
+      upgradeMode == HTTPS_FIRST_MODE &&
       aOptions.contains(
           UpgradeDowngradeEndlessLoopOptions::EnforceForHTTPSFirstMode);
   bool enforceForHTTPSRR =
@@ -357,11 +346,9 @@ bool nsHTTPSOnlyUtils::ShouldUpgradeHttpsFirstRequest(nsIURI* aURI,
   MOZ_ASSERT(aURI->SchemeIs("http"), "how come the request is not 'http'?");
 
   // 1. Check if HTTPS-First Mode is enabled
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  if (!IsHttpsFirstModeEnabled(isPrivateWin) &&
-      !(aLoadInfo->GetSchemelessInput() ==
-            nsILoadInfo::SchemelessInputTypeSchemeless &&
-        mozilla::StaticPrefs::dom_security_https_first_schemeless())) {
+  UpgradeMode upgradeMode = GetUpgradeMode(aLoadInfo);
+  if (upgradeMode != HTTPS_FIRST_MODE &&
+      upgradeMode != SCHEMELESS_HTTPS_FIRST_MODE) {
     return false;
   }
   // 2. HTTPS-First only upgrades top-level loads (and speculative connections)
@@ -416,9 +403,7 @@ bool nsHTTPSOnlyUtils::ShouldUpgradeHttpsFirstRequest(nsIURI* aURI,
 
   // We can upgrade the request - let's log to the console and set the status
   // so we know that we upgraded the request.
-  if (aLoadInfo->GetSchemelessInput() ==
-          nsILoadInfo::SchemelessInputTypeSchemeless &&
-      !IsHttpsFirstModeEnabled(isPrivateWin)) {
+  if (upgradeMode == SCHEMELESS_HTTPS_FIRST_MODE) {
     nsAutoCString urlCString;
     aURI->GetSpec(urlCString);
     NS_ConvertUTF8toUTF16 urlString(urlCString);
@@ -510,12 +495,26 @@ nsHTTPSOnlyUtils::PotentiallyDowngradeHttpsFirstRequest(
 
   // We're only downgrading if it's possible that the error was
   // caused by the upgrade.
-  if (HttpsUpgradeUnrelatedErrorCode(status)) {
+  nsCOMPtr<nsIHttpChannelInternal> httpChannelInternal(
+      do_QueryInterface(channel));
+  if (!httpChannelInternal) {
+    return nullptr;
+  }
+  bool proxyUsed = false;
+  nsresult rv = httpChannelInternal->GetIsProxyUsed(&proxyUsed);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  if (!(proxyUsed && status == nsresult::NS_ERROR_UNKNOWN_HOST)
+      // When a proxy returns an error code it is converted by
+      // HttpProxyResponseToErrorCode. We do want to downgrade in
+      // that case. If the host is actually unreachable this will
+      // show the same error page, but technically for the HTTP
+      // site not the HTTPS site.
+      && HttpsUpgradeUnrelatedErrorCode(status)) {
     return nullptr;
   }
 
   nsCOMPtr<nsIURI> uri;
-  nsresult rv = channel->GetURI(getter_AddRefs(uri));
+  rv = channel->GetURI(getter_AddRefs(uri));
   NS_ENSURE_SUCCESS(rv, nullptr);
 
   nsAutoCString spec;
@@ -604,19 +603,14 @@ void nsHTTPSOnlyUtils::UpdateLoadStateAfterHTTPSFirstDowngrade(
       mozilla::TimeDuration duration =
           mozilla::TimeStamp::Now() - navigationStart;
 
-      bool isPrivateWin = loadInfo->GetOriginAttributes().IsPrivateBrowsing();
-      bool isSchemeless =
-          loadInfo->GetSchemelessInput() ==
-              nsILoadInfo::SchemelessInputTypeSchemeless &&
-          !nsHTTPSOnlyUtils::IsHttpsFirstModeEnabled(isPrivateWin);
-
       nsresult channelStatus;
       channel->GetStatus(&channelStatus);
 
       RefPtr downgradeData = mozilla::MakeRefPtr<HTTPSFirstDowngradeData>();
       downgradeData->downgradeTime = duration;
       downgradeData->isOnTimer = channelStatus == NS_ERROR_NET_TIMEOUT_EXTERNAL;
-      downgradeData->isSchemeless = isSchemeless;
+      downgradeData->isSchemeless =
+          GetUpgradeMode(loadInfo) == SCHEMELESS_HTTPS_FIRST_MODE;
       aLoadState->SetHttpsFirstDowngradeData(downgradeData);
     }
   }
@@ -649,8 +643,7 @@ void nsHTTPSOnlyUtils::SubmitHTTPSFirstTelemetry(
              nsILoadInfo::HTTPS_ONLY_UPGRADED_HTTPS_FIRST) {
     // Successfully upgraded load
 
-    if (aLoadInfo->GetSchemelessInput() ==
-        nsILoadInfo::SchemelessInputTypeSchemeless) {
+    if (GetUpgradeMode(aLoadInfo) == SCHEMELESS_HTTPS_FIRST_MODE) {
       upgraded_schemeless.Add();
     } else {
       upgraded.Add();
@@ -668,8 +661,7 @@ bool nsHTTPSOnlyUtils::CouldBeHttpsOnlyError(nsIChannel* aChannel,
 
   // If HTTPS-Only Mode is not enabled, then there is nothing to do here.
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
-  bool isPrivateWin = loadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  if (!IsHttpsOnlyModeEnabled(isPrivateWin)) {
+  if (GetUpgradeMode(loadInfo) != HTTPS_ONLY_MODE) {
     return false;
   }
 
@@ -688,7 +680,7 @@ bool nsHTTPSOnlyUtils::CouldBeHttpsOnlyError(nsIChannel* aChannel,
 
 /* static */
 bool nsHTTPSOnlyUtils::TestIfPrincipalIsExempt(nsIPrincipal* aPrincipal,
-                                               bool aCheckForHTTPSFirst) {
+                                               UpgradeMode aUpgradeMode) {
   static nsCOMPtr<nsIPermissionManager> sPermMgr;
   if (!sPermMgr) {
     sPermMgr = mozilla::components::PermissionManager::Service();
@@ -701,9 +693,12 @@ bool nsHTTPSOnlyUtils::TestIfPrincipalIsExempt(nsIPrincipal* aPrincipal,
       aPrincipal, "https-only-load-insecure"_ns, &perm);
   NS_ENSURE_SUCCESS(rv, false);
 
+  bool checkForHTTPSFirst = aUpgradeMode == HTTPS_FIRST_MODE ||
+                            aUpgradeMode == SCHEMELESS_HTTPS_FIRST_MODE;
+
   return perm == nsIHttpsOnlyModePermission::LOAD_INSECURE_ALLOW ||
          perm == nsIHttpsOnlyModePermission::LOAD_INSECURE_ALLOW_SESSION ||
-         (aCheckForHTTPSFirst &&
+         (checkForHTTPSFirst &&
           perm == nsIHttpsOnlyModePermission::HTTPSFIRST_LOAD_INSECURE_ALLOW);
 }
 
@@ -715,15 +710,9 @@ void nsHTTPSOnlyUtils::TestSitePermissionAndPotentiallyAddExemption(
   // If HTTPS-Only or HTTPS-First Mode is not enabled, then there is nothing to
   // do here.
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
-  bool isPrivateWin = loadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  bool isHttpsOnly = IsHttpsOnlyModeEnabled(isPrivateWin);
-  bool isHttpsFirst = IsHttpsFirstModeEnabled(isPrivateWin);
-  bool isSchemelessHttpsFirst =
-      (loadInfo->GetSchemelessInput() ==
-       nsILoadInfo::SchemelessInputTypeSchemeless) &&
-      mozilla::StaticPrefs::dom_security_https_first_schemeless() &&
-      !isHttpsOnly && !isHttpsFirst;
-  if (!isHttpsOnly && !isHttpsFirst && !isSchemelessHttpsFirst) {
+  UpgradeMode upgradeMode = GetUpgradeMode(loadInfo);
+
+  if (upgradeMode == NO_UPGRADE_MODE) {
     return;
   }
 
@@ -745,8 +734,7 @@ void nsHTTPSOnlyUtils::TestSitePermissionAndPotentiallyAddExemption(
   NS_ENSURE_SUCCESS_VOID(rv);
 
   uint32_t httpsOnlyStatus = loadInfo->GetHttpsOnlyStatus();
-  bool isPrincipalExempt = TestIfPrincipalIsExempt(
-      principal, isHttpsFirst || isSchemelessHttpsFirst);
+  bool isPrincipalExempt = TestIfPrincipalIsExempt(principal, upgradeMode);
   if (isPrincipalExempt) {
     httpsOnlyStatus |= nsILoadInfo::HTTPS_ONLY_EXEMPT;
   }
@@ -776,8 +764,7 @@ bool nsHTTPSOnlyUtils::IsSafeToAcceptCORSOrMixedContent(
     return false;
   }
   // Check if HTTPS-Only Mode is enabled for this request
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  return nsHTTPSOnlyUtils::IsHttpsOnlyModeEnabled(isPrivateWin);
+  return GetUpgradeMode(aLoadInfo) == HTTPS_ONLY_MODE;
 }
 
 /* static */
@@ -894,29 +881,6 @@ bool nsHTTPSOnlyUtils::UnknownPublicSuffixException(nsIURI* aURI) {
   return !hasKnownPublicSuffix;
 }
 
-/* static */
-bool nsHTTPSOnlyUtils::ShouldUpgradeConnection(nsILoadInfo* aLoadInfo) {
-  // Check if one of parameters is null then webpage can't be loaded yet
-  // and no further inspections are needed
-  if (!aLoadInfo) {
-    return false;
-  }
-
-  // Check if the HTTPS-Only Mode is even enabled, before we do anything else
-  bool isPrivateWin = aLoadInfo->GetOriginAttributes().IsPrivateBrowsing();
-  if (!IsHttpsOnlyModeEnabled(isPrivateWin) &&
-      !IsHttpsFirstModeEnabled(isPrivateWin)) {
-    return false;
-  }
-
-  // If the load is exempt, then don't upgrade
-  uint32_t httpsOnlyStatus = aLoadInfo->GetHttpsOnlyStatus();
-  if (httpsOnlyStatus & nsILoadInfo::HTTPS_ONLY_EXEMPT) {
-    return false;
-  }
-  return true;
-}
-
 /* static */
 bool nsHTTPSOnlyUtils::IsHttpDowngrade(nsIURI* aFromURI, nsIURI* aToURI) {
   MOZ_ASSERT(aFromURI);
@@ -927,12 +891,12 @@ bool nsHTTPSOnlyUtils::IsHttpDowngrade(nsIURI* aFromURI, nsIURI* aToURI) {
   }
 
   // 2. If the target URI is not http, then it's not a http downgrade
-  if (!mozilla::net::SchemeIsHTTP(aToURI)) {
+  if (!aToURI->SchemeIs("http")) {
     return false;
   }
 
   // 3. If the origin URI isn't https, then it's not a http downgrade either.
-  if (!mozilla::net::SchemeIsHTTPS(aFromURI)) {
+  if (!aFromURI->SchemeIs("https")) {
     return false;
   }
 
