@@ -6,38 +6,36 @@
 
 #include "mozilla/dom/TrustedTypeUtils.h"
 
+#include "js/RootingAPI.h"
+#include "mozilla/ErrorResult.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/StaticPrefs_dom.h"
-
-#include "js/RootingAPI.h"
-
-#include "mozilla/ErrorResult.h"
+#include "mozilla/dom/CSPViolationData.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/dom/ElementBinding.h"
+#include "mozilla/dom/HTMLScriptElementBinding.h"
+#include "mozilla/dom/PolicyContainer.h"
 #include "mozilla/dom/TrustedHTML.h"
 #include "mozilla/dom/TrustedScript.h"
 #include "mozilla/dom/TrustedScriptURL.h"
 #include "mozilla/dom/TrustedTypePolicy.h"
 #include "mozilla/dom/TrustedTypePolicyFactory.h"
 #include "mozilla/dom/TrustedTypesConstants.h"
+#include "mozilla/dom/UnionTypes.h"
+#include "mozilla/dom/WindowOrWorkerGlobalScopeBinding.h"
 #include "mozilla/dom/WorkerCommon.h"
 #include "mozilla/dom/WorkerPrivate.h"
 #include "mozilla/dom/WorkerRunnable.h"
 #include "mozilla/dom/WorkerScope.h"
+#include "mozilla/dom/nsCSPUtils.h"
+#include "mozilla/extensions/WebExtensionPolicy.h"
+#include "nsContentUtils.h"
 #include "nsGlobalWindowInner.h"
+#include "nsIContentSecurityPolicy.h"
 #include "nsLiteralString.h"
 #include "nsTArray.h"
 #include "xpcpublic.h"
 
-#include "mozilla/dom/CSPViolationData.h"
-#include "mozilla/dom/ElementBinding.h"
-#include "mozilla/dom/HTMLScriptElementBinding.h"
-#include "mozilla/dom/UnionTypes.h"
-#include "mozilla/dom/WindowOrWorkerGlobalScopeBinding.h"
-#include "mozilla/dom/nsCSPUtils.h"
-
-#include "nsContentUtils.h"
-#include "nsIContentSecurityPolicy.h"
-
 namespace mozilla::dom::TrustedTypeUtils {
 
 nsString GetTrustedTypeName(TrustedType aTrustedType) {
@@ -71,11 +69,13 @@ static constexpr nsLiteralString kAsyncFunctionStarAnonymousPrefix =
 
 // Implement reporting of sink type mismatch violations.
 // https://w3c.github.io/trusted-types/dist/spec/#abstract-opdef-should-sink-type-mismatch-violation-be-blocked-by-content-security-policy
-static void ReportSinkTypeMismatchViolations(
-    nsIContentSecurityPolicy* aCSP, nsICSPEventListener* aCSPEventListener,
-    const nsCString& aFileName, uint32_t aLine, uint32_t aColumn,
-    const nsAString& aSink, const nsAString& aSinkGroup,
-    const nsAString& aSource) {
+void ReportSinkTypeMismatchViolations(nsIContentSecurityPolicy* aCSP,
+                                      nsICSPEventListener* aCSPEventListener,
+                                      const nsCString& aFileName,
+                                      uint32_t aLine, uint32_t aColumn,
+                                      const nsAString& aSink,
+                                      const nsAString& aSinkGroup,
+                                      const nsAString& aSource) {
   MOZ_ASSERT(aSinkGroup == kTrustedTypesOnlySinkGroup);
   MOZ_ASSERT(aCSP);
   MOZ_ASSERT(aCSP->GetRequireTrustedTypesForDirectiveState() !=
@@ -412,7 +412,8 @@ template <typename ExpectedType, typename TrustedTypeOrString,
 MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
     const TrustedTypeOrString& aInput, const nsAString& aSink,
     const nsAString& aSinkGroup, NodeOrGlobalObject& aNodeOrGlobalObject,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   MOZ_ASSERT(aSinkGroup == kTrustedTypesOnlySinkGroup);
   if (!StaticPrefs::dom_security_trusted_types_enabled()) {
     // A trusted type might've been created before the pref was set to `false`,
@@ -424,6 +425,14 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
     return GetAsTrustedType(aInput);
   }
 
+  // Exempt web extension content scripts from trusted types policies defined by
+  // the page in which they are running.
+  if (auto* principal = BasePrincipal::Cast(aPrincipalOrNull)) {
+    if (principal->ContentScriptAddonPolicyCore()) {
+      return GetAsString(aInput);
+    }
+  }
+
   // Below, we use fast paths when there are no require-trusted-types-for
   // directives. Note that the global object's CSP may differ from the
   // owner-document's one. E.g. when aDocument was created by
@@ -434,6 +443,9 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   nsIGlobalObject* globalObject = nullptr;
   nsPIDOMWindowInner* piDOMWindowInner = nullptr;
   if constexpr (std::is_same_v<NodeOrGlobalObjectArg, nsINode>) {
+    if (aNodeOrGlobalObject.HasBeenInUAWidget()) {
+      return GetAsString(aInput);
+    }
     Document* ownerDoc = aNodeOrGlobalObject.OwnerDoc();
     const bool ownerDocLoadedAsData = ownerDoc->IsLoadedAsData();
     if (!ownerDoc->HasPolicyWithRequireTrustedTypesForDirective() &&
@@ -481,7 +493,7 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   RequireTrustedTypesForDirectiveState requireTrustedTypesForDirectiveState =
       RequireTrustedTypesForDirectiveState::NONE;
   if (piDOMWindowInner) {
-    csp = piDOMWindowInner->GetCsp();
+    csp = PolicyContainer::GetCSP(piDOMWindowInner->GetPolicyContainer());
     if (!csp) {
       return GetAsString(aInput);
     }
@@ -557,9 +569,11 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   const nsAString* GetTrustedTypesCompliantString(                        \
       const _trustedTypeOrString& aInput, const nsAString& aSink,         \
       const nsAString& aSinkGroup, _nodeOrGlobalObject& aNodeOrGlobal,    \
-      Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {          \
+      nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder, \
+      ErrorResult& aError) {                                              \
     return GetTrustedTypesCompliantString<_expectedType>(                 \
-        aInput, aSink, aSinkGroup, aNodeOrGlobal, aResultHolder, aError); \
+        aInput, aSink, aSinkGroup, aNodeOrGlobal, aPrincipalOrNull,       \
+        aResultHolder, aError);                                           \
   }
 
 IMPL_GET_TRUSTED_TYPES_COMPLIANT_STRING(TrustedHTMLOrString, TrustedHTML,
@@ -578,6 +592,8 @@ IMPL_GET_TRUSTED_TYPES_COMPLIANT_STRING(TrustedScriptURLOrString,
                                         TrustedScriptURL, const nsINode);
 IMPL_GET_TRUSTED_TYPES_COMPLIANT_STRING(TrustedScriptURLOrUSVString,
                                         TrustedScriptURL, nsIGlobalObject);
+IMPL_GET_TRUSTED_TYPES_COMPLIANT_STRING(TrustedScriptURLOrUSVString,
+                                        TrustedScriptURL, const nsINode);
 IMPL_GET_TRUSTED_TYPES_COMPLIANT_STRING(OwningTrustedScriptURLOrString,
                                         TrustedScriptURL, nsIGlobalObject);
 
@@ -586,19 +602,34 @@ GetTrustedTypesCompliantStringForTrustedHTML(const nsAString& aInput,
                                              const nsAString& aSink,
                                              const nsAString& aSinkGroup,
                                              const nsINode& aNode,
+                                             nsIPrincipal* aPrincipalOrNull,
                                              Maybe<nsAutoString>& aResultHolder,
                                              ErrorResult& aError) {
-  return GetTrustedTypesCompliantString<TrustedHTML>(
-      &aInput, aSink, aSinkGroup, aNode, aResultHolder, aError);
+  return GetTrustedTypesCompliantString<TrustedHTML>(&aInput, aSink, aSinkGroup,
+                                                     aNode, aPrincipalOrNull,
+                                                     aResultHolder, aError);
 }
 
 MOZ_CAN_RUN_SCRIPT const nsAString*
 GetTrustedTypesCompliantStringForTrustedScript(
     const nsAString& aInput, const nsAString& aSink,
     const nsAString& aSinkGroup, nsIGlobalObject& aGlobalObject,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   return GetTrustedTypesCompliantString<TrustedScript>(
-      &aInput, aSink, aSinkGroup, aGlobalObject, aResultHolder, aError);
+      &aInput, aSink, aSinkGroup, aGlobalObject, aPrincipalOrNull,
+      aResultHolder, aError);
+}
+
+MOZ_CAN_RUN_SCRIPT const nsAString*
+GetTrustedTypesCompliantStringForTrustedScript(
+    const nsAString& aInput, const nsAString& aSink,
+    const nsAString& aSinkGroup, const nsINode& aNode,
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
+  return GetTrustedTypesCompliantString<TrustedScript>(
+      &aInput, aSink, aSinkGroup, aNode, aPrincipalOrNull, aResultHolder,
+      aError);
 }
 
 bool GetTrustedTypeDataForAttribute(const nsAtom* aElementName,
@@ -662,7 +693,8 @@ template <typename TrustedTypeOrStringArg>
 MOZ_CAN_RUN_SCRIPT const nsAString* GetTrustedTypesCompliantAttributeValue(
     const nsINode& aElement, nsAtom* aAttributeName,
     int32_t aAttributeNamespaceID, const TrustedTypeOrStringArg& aNewValue,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   if (!StaticPrefs::dom_security_trusted_types_enabled()) {
     // A trusted type might've been created before the pref was set to `false`,
     // so we cannot assume aNewValue.IsString().
@@ -706,16 +738,16 @@ MOZ_CAN_RUN_SCRIPT const nsAString* GetTrustedTypesCompliantAttributeValue(
   switch (expectedType) {
     case TrustedType::TrustedHTML:
       return GetTrustedTypesCompliantString<TrustedHTML>(
-          input, sink, kTrustedTypesOnlySinkGroup, aElement, aResultHolder,
-          aError);
+          input, sink, kTrustedTypesOnlySinkGroup, aElement, aPrincipalOrNull,
+          aResultHolder, aError);
     case TrustedType::TrustedScript:
       return GetTrustedTypesCompliantString<TrustedScript>(
-          input, sink, kTrustedTypesOnlySinkGroup, aElement, aResultHolder,
-          aError);
+          input, sink, kTrustedTypesOnlySinkGroup, aElement, aPrincipalOrNull,
+          aResultHolder, aError);
     case TrustedType::TrustedScriptURL:
       return GetTrustedTypesCompliantString<TrustedScriptURL>(
-          input, sink, kTrustedTypesOnlySinkGroup, aElement, aResultHolder,
-          aError);
+          input, sink, kTrustedTypesOnlySinkGroup, aElement, aPrincipalOrNull,
+          aResultHolder, aError);
   }
   MOZ_ASSERT_UNREACHABLE();
   return nullptr;
@@ -725,20 +757,22 @@ MOZ_CAN_RUN_SCRIPT const nsAString* GetTrustedTypesCompliantAttributeValue(
     const nsINode& aElement, nsAtom* aAttributeName,
     int32_t aAttributeNamespaceID,
     const TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString& aNewValue,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   return GetTrustedTypesCompliantAttributeValue<
       TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString>(
-      aElement, aAttributeName, aAttributeNamespaceID, aNewValue, aResultHolder,
-      aError);
+      aElement, aAttributeName, aAttributeNamespaceID, aNewValue,
+      aPrincipalOrNull, aResultHolder, aError);
 }
 
 MOZ_CAN_RUN_SCRIPT const nsAString* GetTrustedTypesCompliantAttributeValue(
     const nsINode& aElement, nsAtom* aAttributeName,
     int32_t aAttributeNamespaceID, const nsAString& aNewValue,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   return GetTrustedTypesCompliantAttributeValue<const nsAString*>(
       aElement, aAttributeName, aAttributeNamespaceID, &aNewValue,
-      aResultHolder, aError);
+      aPrincipalOrNull, aResultHolder, aError);
 }
 
 bool HostGetCodeForEval(JSContext* aCx, JS::Handle<JSObject*> aCode,
@@ -763,7 +797,8 @@ bool AreArgumentsTrustedForEnsureCSPDoesNotBlockStringCompilation(
     JS::Handle<JS::StackGCVector<JSString*>> aParameterStrings,
     JS::Handle<JSString*> aBodyString,
     JS::Handle<JS::StackGCVector<JS::Value>> aParameterArgs,
-    JS::Handle<JS::Value> aBodyArg, ErrorResult& aError) {
+    JS::Handle<JS::Value> aBodyArg, nsIPrincipal* aPrincipalOrNull,
+    ErrorResult& aError) {
   // EnsureCSPDoesNotBlockStringCompilation is essentially HTML's implementation
   // of HostEnsureCanCompileStrings, so we only consider the cases described in
   // the Dynamic Code Brand Checks spec. The algorithm is also supposed to be
@@ -894,8 +929,8 @@ bool AreArgumentsTrustedForEnsureCSPDoesNotBlockStringCompilation(
           codeString,
           aCompilationType == JS::CompilationType::Function ? functionSink
                                                             : evalSink,
-          kTrustedTypesOnlySinkGroup, *pinnedGlobal, compliantStringHolder,
-          aError);
+          kTrustedTypesOnlySinkGroup, *pinnedGlobal, aPrincipalOrNull,
+          compliantStringHolder, aError);
 
   // Step 2.7-2.8.
   // Callers will take care of throwing an EvalError when we return false.
@@ -906,4 +941,29 @@ bool AreArgumentsTrustedForEnsureCSPDoesNotBlockStringCompilation(
   return compliantString->Equals(codeString);
 }
 
+MOZ_CAN_RUN_SCRIPT const nsAString*
+GetConvertedScriptSourceForPreNavigationCheck(
+    nsIGlobalObject& aGlobalObject, const nsAString& aEncodedScriptSource,
+    const nsAString& aSink, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
+  RefPtr<TrustedScript> convertedScriptSource;
+  nsCOMPtr<nsIGlobalObject> pinnedGlobalObject = &aGlobalObject;
+  ProcessValueWithADefaultPolicy<TrustedScript>(
+      *pinnedGlobalObject, aEncodedScriptSource, aSink,
+      getter_AddRefs(convertedScriptSource), aError);
+
+  // If that algorithm threw an error or convertedScriptSource is not a
+  // TrustedScript object, return "Blocked" and abort further steps.
+  if (aError.Failed()) {
+    return nullptr;
+  }
+  if (!convertedScriptSource) {
+    aError.ThrowTypeError("Pre-Navigation check Blocked"_ns);
+    return nullptr;
+  }
+
+  aResultHolder = Some(convertedScriptSource->mData);
+  return aResultHolder.ptr();
+}
+
 }  // namespace mozilla::dom::TrustedTypeUtils
