@@ -6,26 +6,26 @@
 
 #include "SVGAnimatedLength.h"
 
+#include "DOMSVGAnimatedLength.h"
+#include "DOMSVGLength.h"
+#include "LayoutLogging.h"
+#include "SVGAttrTearoffTable.h"
+#include "SVGGeometryProperty.h"
+#include "SVGLengthSMILType.h"
 #include "mozAutoDocUpdate.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/GeckoBindings.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/SMILValue.h"
-#include "mozilla/StaticPresData.h"
-#include "nsPresContextInlines.h"
 #include "mozilla/SVGIntegrationUtils.h"
+#include "mozilla/StaticPresData.h"
 #include "mozilla/dom/SVGViewportElement.h"
-#include "DOMSVGAnimatedLength.h"
-#include "DOMSVGLength.h"
-#include "LayoutLogging.h"
 #include "nsContentUtils.h"
 #include "nsIFrame.h"
 #include "nsLayoutUtils.h"
+#include "nsPresContextInlines.h"
 #include "nsTextFormatter.h"
-#include "SVGLengthSMILType.h"
-#include "SVGAttrTearoffTable.h"
-#include "SVGGeometryProperty.h"
 
 using namespace mozilla::dom;
 
@@ -45,15 +45,13 @@ class MOZ_RAII AutoChangeLengthNotifier {
 
     if (mDoSetAttr) {
       mUpdateBatch.emplace(aSVGElement->GetComposedDoc(), true);
-      mEmptyOrOldValue =
-          mSVGElement->WillChangeLength(mLength->mAttrEnum, mUpdateBatch.ref());
+      mSVGElement->WillChangeLength(mLength->mAttrEnum, mUpdateBatch.ref());
     }
   }
 
   ~AutoChangeLengthNotifier() {
     if (mDoSetAttr) {
-      mSVGElement->DidChangeLength(mLength->mAttrEnum, mEmptyOrOldValue,
-                                   mUpdateBatch.ref());
+      mSVGElement->DidChangeLength(mLength->mAttrEnum, mUpdateBatch.ref());
     }
     if (mLength->mIsAnimated) {
       mSVGElement->AnimationNeedsResample();
@@ -64,7 +62,6 @@ class MOZ_RAII AutoChangeLengthNotifier {
   SVGAnimatedLength* const mLength;
   SVGElement* const mSVGElement;
   Maybe<mozAutoDocUpdate> mUpdateBatch;
-  nsAttrValue mEmptyOrOldValue;
   bool mDoSetAttr;
 };
 
@@ -133,8 +130,9 @@ GeckoFontMetrics UserSpaceMetrics::GetFontMetrics(const Element* aElement) {
           metrics = Gecko_GetFontMetrics(
               presContext, WritingMode(style).IsVertical(), style->StyleFont(),
               style->StyleFont()->mFont.size,
-              /* aUseUserFontSet = */ true,
-              /* aRetrieveMathScales */ false);
+              StyleQueryFontMetricsFlags::USE_USER_FONT_SET |
+                  StyleQueryFontMetricsFlags::NEEDS_CH |
+                  StyleQueryFontMetricsFlags::NEEDS_IC);
         });
   }
   return metrics;
@@ -290,8 +288,9 @@ GeckoFontMetrics NonSVGFrameUserSpaceMetrics::GetFontMetricsForType(
       return Gecko_GetFontMetrics(
           mFrame->PresContext(), mFrame->GetWritingMode().IsVertical(),
           mFrame->StyleFont(), mFrame->StyleFont()->mFont.size,
-          /* aUseUserFontSet = */ true,
-          /* aRetrieveMathScales */ false);
+          StyleQueryFontMetricsFlags::USE_USER_FONT_SET |
+              StyleQueryFontMetricsFlags::NEEDS_CH |
+              StyleQueryFontMetricsFlags::NEEDS_IC);
     case Type::Root:
       return GetFontMetrics(
           mFrame->PresContext()->Document()->GetRootElement());
