@@ -5,30 +5,27 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "CacheLoadHandler.h"
+
 #include "ScriptResponseHeaderProcessor.h"  // ScriptResponseHeaderProcessor
 #include "WorkerLoadContext.h"              // WorkerLoadContext
-
-#include "nsIPrincipal.h"
-
-#include "nsIThreadRetargetableRequest.h"
-#include "nsIXPConnect.h"
-
 #include "jsapi.h"
-#include "nsNetUtil.h"
-
 #include "mozilla/Assertions.h"
 #include "mozilla/Encoding.h"
-#include "mozilla/dom/CacheBinding.h"
-#include "mozilla/dom/cache/CacheTypes.h"
-#include "mozilla/dom/Response.h"
-#include "mozilla/dom/ServiceWorkerBinding.h"  // ServiceWorkerState
 #include "mozilla/Result.h"
 #include "mozilla/TaskQueue.h"
 #include "mozilla/UniquePtr.h"
+#include "mozilla/dom/CacheBinding.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/dom/PolicyContainer.h"
+#include "mozilla/dom/Response.h"
+#include "mozilla/dom/ServiceWorkerBinding.h"  // ServiceWorkerState
 #include "mozilla/dom/WorkerScope.h"
-
+#include "mozilla/dom/cache/CacheTypes.h"
 #include "mozilla/dom/workerinternals/ScriptLoader.h"  // WorkerScriptLoader
+#include "nsIPrincipal.h"
+#include "nsIThreadRetargetableRequest.h"
+#include "nsIXPConnect.h"
+#include "nsNetUtil.h"
 
 namespace mozilla {
 namespace dom {
@@ -579,7 +576,7 @@ nsresult CacheLoadHandler::DataReceivedFromCache(
 
     nsCOMPtr<nsIContentSecurityPolicy> csp;
     if (parentDoc) {
-      csp = parentDoc->GetCsp();
+      csp = PolicyContainer::GetCSP(parentDoc->GetPolicyContainer());
     }
     MOZ_DIAGNOSTIC_ASSERT(!csp);
 #endif
@@ -610,13 +607,13 @@ nsresult CacheLoadHandler::DataReceivedFromCache(
   }
 
   if (NS_SUCCEEDED(rv)) {
-    DataReceived();
+    return DataReceived();
   }
 
   return rv;
 }
 
-void CacheLoadHandler::DataReceived() {
+nsresult CacheLoadHandler::DataReceived() {
   MOZ_ASSERT(!mRequestHandle->IsEmpty());
   WorkerLoadContext* loadContext = mRequestHandle->GetContext();
 
@@ -627,12 +624,13 @@ void CacheLoadHandler::DataReceived() {
       // XHR Params Allowed
       mWorkerRef->Private()->SetXHRParamsAllowed(parent->XHRParamsAllowed());
 
-      // Set Eval and ContentSecurityPolicy
-      mWorkerRef->Private()->SetCsp(parent->GetCsp());
-      mWorkerRef->Private()->SetEvalAllowed(parent->IsEvalAllowed());
-      mWorkerRef->Private()->SetWasmEvalAllowed(parent->IsWasmEvalAllowed());
+      // Set ContentSecurityPolicy
+      nsresult rv = mWorkerRef->Private()->SetCsp(parent->GetCsp());
+      NS_ENSURE_SUCCESS(rv, rv);
     }
   }
+
+  return NS_OK;
 }
 
 }  // namespace workerinternals::loader
