@@ -8,6 +8,7 @@
 #  include "WMF.h"
 #  include "WMFDecoderModule.h"
 #endif
+#include "FFVPXRuntimeLinker.h"
 #include "GLContextProvider.h"
 #include "GPUParent.h"
 #include "GPUProcessHost.h"
@@ -22,6 +23,7 @@
 #include "gfxConfig.h"
 #include "gfxCrashReporterUtils.h"
 #include "gfxPlatform.h"
+#include "MediaCodecsSupport.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Components.h"
 #include "mozilla/FOGIPC.h"
@@ -29,8 +31,8 @@
 #include "mozilla/PerfStats.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/ProcessPriorityManager.h"
-#include "mozilla/RemoteDecoderManagerChild.h"
-#include "mozilla/RemoteDecoderManagerParent.h"
+#include "mozilla/RemoteMediaManagerChild.h"
+#include "mozilla/RemoteMediaManagerParent.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StaticPrefs_media.h"
@@ -65,6 +67,9 @@
 #include "nscore.h"
 #include "prenv.h"
 #include "skia/include/core/SkGraphics.h"
+#if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
+#  include "mozilla/SandboxSettings.h"
+#endif
 #if defined(XP_WIN)
 #  include <dwrite.h>
 #  include <process.h>
@@ -74,11 +79,10 @@
 #  include "gfxWindowsPlatform.h"
 #  include "mozilla/WindowsVersion.h"
 #  include "mozilla/gfx/DeviceManagerDx.h"
+#  include "mozilla/layers/CompositeProcessD3D11FencesHolderMap.h"
 #  include "mozilla/layers/GpuProcessD3D11TextureMap.h"
-#  include "mozilla/layers/GpuProcessD3D11QueryMap.h"
 #  include "mozilla/layers/TextureD3D11.h"
 #  include "mozilla/widget/WinCompositorWindowThread.h"
-#  include "MediaCodecsSupport.h"
 #  include "WMFDecoderModule.h"
 #else
 #  include <unistd.h>
@@ -104,25 +108,6 @@ namespace mozilla::gfx {
 using namespace ipc;
 using namespace layers;
 
-static media::MediaCodecsSupported GetFullMediaCodecSupport(
-    bool aForceRefresh = false) {
-#if defined(XP_WIN)
-  // Re-initializing WMFPDM if forcing a refresh is required or hardware
-  // decoding is supported in order to get HEVC result properly. We will disable
-  // it later if the pref is OFF.
-  if (aForceRefresh || (gfx::gfxVars::IsInitialized() &&
-                        gfx::gfxVars::CanUseHardwareVideoDecoding())) {
-    WMFDecoderModule::Init(WMFDecoderModule::Config::ForceEnableHEVC);
-  }
-  auto disableHEVCIfNeeded = MakeScopeExit([]() {
-    if (!StaticPrefs::media_hevc_enabled()) {
-      WMFDecoderModule::DisableForceEnableHEVC();
-    }
-  });
-#endif
-  return PDMFactory::Supported(aForceRefresh);
-}
-
 static GPUParent* sGPUParent;
 
 GPUParent::GPUParent() : mLaunchTime(TimeStamp::Now()) { sGPUParent = this; }
@@ -215,8 +200,8 @@ bool GPUParent::Init(mozilla::ipc::UntypedEndpoint&& aEndpoint,
 #if defined(XP_WIN)
   gfxWindowsPlatform::InitMemoryReportersForGPUProcess();
   DeviceManagerDx::Init();
+  CompositeProcessD3D11FencesHolderMap::Init();
   GpuProcessD3D11TextureMap::Init();
-  GpuProcessD3D11QueryMap::Init();
   auto rv = wmf::MediaFoundationInitializer::HasInitialized();
   if (!rv) {
     NS_WARNING("Failed to init Media Foundation in the GPU process");
@@ -229,7 +214,21 @@ bool GPUParent::Init(mozilla::ipc::UntypedEndpoint&& aEndpoint,
   apz::InitializeGlobalState();
   LayerTreeOwnerTracker::Initialize();
   CompositorBridgeParent::InitializeStatics();
+
+#if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
+  // On macOS, we pass the empty string for the process name because
+  // the bundle name (CFBundleName) is the complete name already.
+  // If the sandbox is enabled, setting the executable name will fail
+  // so don't attempt it. The executable name will be shown in
+  // Activity Monitor as the process name.
+  if (!IsGPUSandboxEnabled()) {
+    mozilla::ipc::SetThisProcessName("");
+  }
+#elif defined(XP_MACOSX)
+  mozilla::ipc::SetThisProcessName("");
+#else
   mozilla::ipc::SetThisProcessName("GPU Process");
+#endif  // XP_MACOSX && MOZ_SANDBOX
 
   return true;
 }
@@ -297,9 +296,7 @@ mozilla::ipc::IPCResult GPUParent::RecvInit(
     nsTArray<GfxVarUpdate>&& vars, const DevicePrefs& devicePrefs,
     nsTArray<LayerTreeIdMapping>&& aMappings,
     nsTArray<GfxInfoFeatureStatus>&& aFeatures, uint32_t aWrNamespace) {
-  for (const auto& var : vars) {
-    gfxVars::ApplyUpdate(var);
-  }
+  gfxVars::ApplyUpdate(vars);
 
   // Inherit device preferences.
   gfxConfig::Inherit(Feature::HW_COMPOSITING, devicePrefs.hwCompositing());
@@ -431,7 +428,7 @@ mozilla::ipc::IPCResult GPUParent::RecvInit(
           []() {
             NS_DispatchToMainThread(NS_NewRunnableFunction(
                 "GPUParent::UpdateMediaCodecsSupported",
-                [supported = GetFullMediaCodecSupport()]() {
+                [supported = media::MCSInfo::GetSupportFromFactory()]() {
                   Unused << GPUParent::GetSingleton()
                                 ->SendUpdateMediaCodecsSupported(supported);
                 }));
@@ -516,29 +513,29 @@ mozilla::ipc::IPCResult GPUParent::RecvInitProfiler(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult GPUParent::RecvUpdateVar(const GfxVarUpdate& aUpdate) {
-#if defined(XP_WIN)
-  auto scopeExit = MakeScopeExit(
-      [couldUseHWDecoder = gfx::gfxVars::CanUseHardwareVideoDecoding()] {
-        if (couldUseHWDecoder != gfx::gfxVars::CanUseHardwareVideoDecoding()) {
-          MOZ_ALWAYS_SUCCEEDS(NS_DispatchBackgroundTask(
-              NS_NewRunnableFunction(
-                  "GPUParent::RecvUpdateVar",
-                  []() {
-                    NS_DispatchToMainThread(NS_NewRunnableFunction(
-                        "GPUParent::UpdateMediaCodecsSupported",
-                        [supported = GetFullMediaCodecSupport(
-                             true /* force refresh */)]() {
-                          Unused << GPUParent::GetSingleton()
-                                        ->SendUpdateMediaCodecsSupported(
-                                            supported);
-                        }));
-                  }),
-              nsIEventTarget::DISPATCH_NORMAL));
-        }
-      });
-#endif
+mozilla::ipc::IPCResult GPUParent::RecvUpdateVar(
+    const nsTArray<GfxVarUpdate>& aUpdate) {
   gfxVars::ApplyUpdate(aUpdate);
+  MOZ_ALWAYS_SUCCEEDS(NS_DispatchBackgroundTask(
+      NS_NewRunnableFunction(
+          "GPUParent::RecvUpdateVar",
+          []() {
+#ifdef XP_WIN
+            WMFDecoderModule::Init();
+#endif
+            if (StaticPrefs::media_ffvpx_hw_enabled()) {
+              FFVPXRuntimeLinker::Init();
+            }
+            NS_DispatchToMainThread(NS_NewRunnableFunction(
+                "GPUParent::UpdateMediaCodecsSupported",
+                [supported = media::MCSInfo::GetSupportFromFactory(
+                     true /* force refresh */)]() {
+                  if (auto* gpu = GPUParent::GetSingleton()) {
+                    Unused << gpu->SendUpdateMediaCodecsSupported(supported);
+                  }
+                }));
+          }),
+      nsIEventTarget::DISPATCH_NORMAL));
   return IPC_OK();
 }
 
@@ -632,11 +629,11 @@ mozilla::ipc::IPCResult GPUParent::RecvNewContentVRManager(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult GPUParent::RecvNewContentRemoteDecoderManager(
-    Endpoint<PRemoteDecoderManagerParent>&& aEndpoint,
+mozilla::ipc::IPCResult GPUParent::RecvNewContentRemoteMediaManager(
+    Endpoint<PRemoteMediaManagerParent>&& aEndpoint,
     const ContentParentId& aChildId) {
-  if (!RemoteDecoderManagerParent::CreateForContent(std::move(aEndpoint),
-                                                    aChildId)) {
+  if (!RemoteMediaManagerParent::CreateForContent(std::move(aEndpoint),
+                                                  aChildId)) {
     return IPC_FAIL_NO_REASON(this);
   }
   return IPC_OK();
@@ -749,7 +746,7 @@ void GPUParent::ActorDestroy(ActorDestroyReason aWhy) {
   ProcessChild::QuickExit();
 #endif
 
-  // Wait until all RemoteDecoderManagerParent have closed.
+  // Wait until all RemoteMediaManagerParent have closed.
   mShutdownBlockers.WaitUntilClear(10 * 1000 /* 10s timeout*/)
       ->Then(GetCurrentSerialEventTarget(), __func__, [self = RefPtr{this}]() {
         if (self->mProfilerController) {
@@ -805,8 +802,8 @@ void GPUParent::ActorDestroy(ActorDestroyReason aWhy) {
 #endif
 
 #if defined(XP_WIN)
-        GpuProcessD3D11QueryMap::Shutdown();
         GpuProcessD3D11TextureMap::Shutdown();
+        CompositeProcessD3D11FencesHolderMap::Shutdown();
         DeviceManagerDx::Shutdown();
 #endif
         LayerTreeOwnerTracker::Shutdown();
