@@ -50,8 +50,6 @@
 #ifdef MOZ_WIDGET_GTK
 #  include <gtk/gtkx.h>
 #  include "gfxPlatformGtk.h"
-#  include "mozilla/layers/DMABUFTextureClientOGL.h"
-#  include "mozilla/widget/DMABufLibWrapper.h"
 #endif
 #ifdef MOZ_WAYLAND
 #  include "mozilla/widget/nsWaylandDisplay.h"
@@ -301,15 +299,6 @@ static TextureType ChooseTextureType(gfx::SurfaceFormat aFormat,
   }
 #endif
 
-#ifdef MOZ_WIDGET_GTK
-  if ((layersBackend == LayersBackend::LAYERS_WR &&
-       !aKnowsCompositor->UsingSoftwareWebRender()) &&
-      widget::DMABufDevice::IsDMABufTexturesEnabled() &&
-      aFormat != SurfaceFormat::A8) {
-    return TextureType::DMABUF;
-  }
-#endif
-
 #ifdef XP_MACOSX
   if (StaticPrefs::gfx_use_iosurface_textures_AtStartup() &&
       !aKnowsCompositor->UsingSoftwareWebRender()) {
@@ -345,11 +334,6 @@ TextureData* TextureData::Create(TextureType aTextureType,
       return D3D11TextureData::Create(aSize, aFormat, aAllocFlags);
 #endif
 
-#ifdef MOZ_WIDGET_GTK
-    case TextureType::DMABUF:
-      return DMABUFTextureData::Create(aSize, aFormat, aBackendType);
-#endif
-
 #ifdef XP_MACOSX
     case TextureType::MacIOSurface:
       return MacIOSurfaceTextureData::Create(aSize, aFormat, aBackendType);
@@ -808,18 +792,11 @@ void TextureClient::Unlock() {
   }
 
   if (mBorrowedDrawTarget) {
-    if (!(mOpenMode & OpenMode::OPEN_ASYNC)) {
-      if (mOpenMode & OpenMode::OPEN_WRITE) {
-        mBorrowedDrawTarget->Flush();
-      }
-
-      mBorrowedDrawTarget->DetachAllSnapshots();
-      // If this assertion is hit, it means something is holding a strong
-      // reference to our DrawTarget externally, which is not allowed.
-      MOZ_ASSERT(mBorrowedDrawTarget->refCount() <= mExpectedDtRefs);
+    if (mOpenMode & OpenMode::OPEN_WRITE) {
+      mBorrowedDrawTarget->Flush();
     }
 
-    mBorrowedDrawTarget = nullptr;
+    mData->ReturnDrawTarget(mBorrowedDrawTarget.forget());
   }
   mBorrowedSnapshot = false;
 
@@ -993,9 +970,6 @@ gfx::DrawTarget* TextureClient::BorrowDrawTarget() {
 
   if (!mBorrowedDrawTarget) {
     mBorrowedDrawTarget = mData->BorrowDrawTarget();
-#ifdef DEBUG
-    mExpectedDtRefs = mBorrowedDrawTarget ? mBorrowedDrawTarget->refCount() : 0;
-#endif
   }
 
   return mBorrowedDrawTarget;
@@ -1008,14 +982,17 @@ void TextureClient::EndDraw() {
   // end of a transaction, we need to Flush and DetachAllSnapshots to ensure any
   // dependents are updated.
   mBorrowedDrawTarget->Flush();
-  mBorrowedDrawTarget->DetachAllSnapshots();
-  MOZ_ASSERT(mBorrowedDrawTarget->refCount() <= mExpectedDtRefs);
+  mData->ReturnDrawTarget(mBorrowedDrawTarget.forget());
 
-  mBorrowedDrawTarget = nullptr;
   mBorrowedSnapshot = false;
   mData->EndDraw();
 }
 
+void TextureData::ReturnDrawTarget(already_AddRefed<gfx::DrawTarget> aDT) {
+  RefPtr<gfx::DrawTarget> dt(aDT);
+  dt->DetachAllSnapshots();
+}
+
 already_AddRefed<gfx::SourceSurface> TextureClient::BorrowSnapshot() {
   MOZ_ASSERT(mIsLocked);
 
@@ -1033,6 +1010,11 @@ already_AddRefed<gfx::SourceSurface> TextureClient::BorrowSnapshot() {
   return surface.forget();
 }
 
+void TextureData::ReturnSnapshot(
+    already_AddRefed<gfx::SourceSurface> aSnapshot) {
+  RefPtr<gfx::SourceSurface> snapshot(aSnapshot);
+}
+
 void TextureClient::ReturnSnapshot(
     already_AddRefed<gfx::SourceSurface> aSnapshot) {
   RefPtr<gfx::SourceSurface> snapshot = aSnapshot;
@@ -1541,12 +1523,7 @@ TextureClient::TextureClient(TextureData* aData, TextureFlags aFlags,
       mActor(nullptr),
       mData(aData),
       mFlags(aFlags),
-      mOpenMode(OpenMode::OPEN_NONE)
-#ifdef DEBUG
-      ,
-      mExpectedDtRefs(0)
-#endif
-      ,
+      mOpenMode(OpenMode::OPEN_NONE),
       mIsLocked(false),
       mIsReadLocked(false),
       mUpdated(false),
