@@ -358,22 +358,29 @@ void CanvasTranslator::GetDataSurface(uint64_t aSurfaceRef) {
   MOZ_ASSERT(IsInTaskQueue());
 
   ReferencePtr surfaceRef = reinterpret_cast<void*>(aSurfaceRef);
-  gfx::SourceSurface* surface = LookupSourceSurface(surfaceRef);
-  if (!surface) {
+  RefPtr<gfx::DataSourceSurface> dataSurface = LookupDataSurface(surfaceRef);
+  if (!dataSurface) {
+    gfx::SourceSurface* surface = LookupSourceSurface(surfaceRef);
+    if (!surface) {
+      return;
+    }
+    dataSurface = surface->GetDataSurface();
+    if (!dataSurface) {
+      return;
+    }
+  }
+  gfx::DataSourceSurface::ScopedMap map(dataSurface,
+                                        gfx::DataSourceSurface::READ);
+  if (!map.IsMapped()) {
     return;
   }
 
-  UniquePtr<gfx::DataSourceSurface::ScopedMap> map = GetPreparedMap(surfaceRef);
-  if (!map) {
-    return;
-  }
-
-  auto dstSize = surface->GetSize();
-  auto srcSize = map->GetSurface()->GetSize();
-  gfx::SurfaceFormat format = surface->GetFormat();
+  auto dstSize = dataSurface->GetSize();
+  auto srcSize = map.GetSurface()->GetSize();
+  gfx::SurfaceFormat format = dataSurface->GetFormat();
   int32_t bpp = BytesPerPixel(format);
   int32_t dataFormatWidth = dstSize.width * bpp;
-  int32_t srcStride = map->GetStride();
+  int32_t srcStride = map.GetStride();
   if (dataFormatWidth > srcStride || srcSize != dstSize) {
     return;
   }
@@ -387,7 +394,7 @@ void CanvasTranslator::GetDataSurface(uint64_t aSurfaceRef) {
   }
 
   uint8_t* dst = mDataSurfaceShmem.DataAs<uint8_t>();
-  const uint8_t* src = map->GetData();
+  const uint8_t* src = map.GetData();
   const uint8_t* endSrc = src + (srcSize.height * srcStride);
   while (src < endSrc) {
     memcpy(dst, src, dataFormatWidth);
@@ -545,11 +552,16 @@ bool CanvasTranslator::TryDrawTargetWebglFallback(
   NotifyRequiresRefresh(aTextureOwnerId);
 
   const auto& info = mTextureInfo[aTextureOwnerId];
+  if (info.mTextureData) {
+    return true;
+  }
   if (RefPtr<gfx::DrawTarget> dt =
           CreateFallbackDrawTarget(info.mRefPtr, aTextureOwnerId,
                                    aWebgl->GetSize(), aWebgl->GetFormat())) {
     bool success = aWebgl->CopyToFallback(dt);
-    AddDrawTarget(info.mRefPtr, dt);
+    if (info.mRefPtr) {
+      AddDrawTarget(info.mRefPtr, dt);
+    }
     return success;
   }
   return false;
@@ -945,6 +957,9 @@ void CanvasTranslator::DeviceResetAcknowledged() { DeviceChangeAcknowledged(); }
 
 bool CanvasTranslator::CreateReferenceTexture() {
   if (mReferenceTextureData) {
+    if (mBaseDT) {
+      mReferenceTextureData->ReturnDrawTarget(mBaseDT.forget());
+    }
     mReferenceTextureData->Unlock();
   }
 
@@ -1097,16 +1112,18 @@ void CanvasTranslator::CacheSnapshotShmem(
   if (gfx::DrawTargetWebgl* webgl = GetDrawTargetWebgl(aTextureOwnerId)) {
     if (auto shmemHandle = webgl->TakeShmemHandle()) {
       // Lock the DT so that it doesn't get removed while shmem is in transit.
-      mTextureInfo[aTextureOwnerId].mLocked++;
+      AddTextureKeepAlive(aTextureOwnerId);
       nsCOMPtr<nsIThread> thread =
           gfx::CanvasRenderThread::GetCanvasRenderThread();
       RefPtr<CanvasTranslator> translator = this;
       SendSnapshotShmem(aTextureOwnerId, std::move(shmemHandle))
           ->Then(
               thread, __func__,
-              [=](bool) { translator->RemoveTexture(aTextureOwnerId); },
+              [=](bool) {
+                translator->RemoveTextureKeepAlive(aTextureOwnerId);
+              },
               [=](ipc::ResponseRejectReason) {
-                translator->RemoveTexture(aTextureOwnerId);
+                translator->RemoveTextureKeepAlive(aTextureOwnerId);
               });
     }
   }
@@ -1116,14 +1133,12 @@ void CanvasTranslator::PrepareShmem(
     const RemoteTextureOwnerId aTextureOwnerId) {
   if (gfx::DrawTargetWebgl* webgl =
           GetDrawTargetWebgl(aTextureOwnerId, false)) {
-    if (const auto& fallback = mTextureInfo[aTextureOwnerId].mTextureData) {
+    if (RefPtr<gfx::DrawTarget> dt =
+            mTextureInfo[aTextureOwnerId].mFallbackDrawTarget) {
       // If there was a fallback, copy the fallback to the software framebuffer
       // shmem for reading.
-      if (RefPtr<gfx::DrawTarget> dt = fallback->BorrowDrawTarget()) {
-        if (RefPtr<gfx::SourceSurface> snapshot = dt->Snapshot()) {
-          webgl->CopySurface(snapshot, snapshot->GetRect(),
-                             gfx::IntPoint(0, 0));
-        }
+      if (RefPtr<gfx::SourceSurface> snapshot = dt->Snapshot()) {
+        webgl->CopySurface(snapshot, snapshot->GetRect(), gfx::IntPoint(0, 0));
       }
     } else {
       // Otherwise, just ensure the software framebuffer is up to date.
@@ -1226,6 +1241,7 @@ already_AddRefed<gfx::DrawTarget> CanvasTranslator::CreateFallbackDrawTarget(
 
     TextureInfo& info = mTextureInfo[aTextureOwnerId];
     info.mRefPtr = aRefPtr;
+    info.mFallbackDrawTarget = dt;
     info.mTextureData = std::move(textureData);
     info.mTextureLockMode = kInitMode;
   } while (!dt && CheckForFreshCanvasDevice(__LINE__));
@@ -1242,6 +1258,19 @@ already_AddRefed<gfx::DrawTarget> CanvasTranslator::CreateDrawTarget(
     return nullptr;
   }
 
+  {
+    auto result = mTextureInfo.find(aTextureOwnerId);
+    if (result != mTextureInfo.end()) {
+      const TextureInfo& info = result->second;
+      if (info.mTextureData || info.mDrawTarget) {
+#ifndef FUZZING_SNAPSHOT
+        MOZ_DIAGNOSTIC_CRASH("DrawTarget already exists");
+#endif
+        return nullptr;
+      }
+    }
+  }
+
   RefPtr<gfx::DrawTarget> dt;
   if (gfx::gfxVars::UseAcceleratedCanvas2D()) {
     if (EnsureSharedContextWebgl()) {
@@ -1268,7 +1297,9 @@ already_AddRefed<gfx::DrawTarget> CanvasTranslator::CreateDrawTarget(
     dt = CreateFallbackDrawTarget(aRefPtr, aTextureOwnerId, aSize, aFormat);
   }
 
-  AddDrawTarget(aRefPtr, dt);
+  if (dt && aRefPtr) {
+    AddDrawTarget(aRefPtr, dt);
+  }
   return dt.forget();
 }
 
@@ -1291,9 +1322,23 @@ void CanvasTranslator::NotifyTextureDestruction(
   Unused << SendNotifyTextureDestruction(aTextureOwnerId);
 }
 
+void CanvasTranslator::AddTextureKeepAlive(const RemoteTextureOwnerId& aId) {
+  auto result = mTextureInfo.find(aId);
+  if (result == mTextureInfo.end()) {
+    return;
+  }
+  auto& info = result->second;
+  ++info.mKeepAlive;
+}
+
+void CanvasTranslator::RemoveTextureKeepAlive(const RemoteTextureOwnerId& aId) {
+  RemoveTexture(aId, 0, 0, false);
+}
+
 void CanvasTranslator::RemoveTexture(const RemoteTextureOwnerId aTextureOwnerId,
                                      RemoteTextureTxnType aTxnType,
-                                     RemoteTextureTxnId aTxnId) {
+                                     RemoteTextureTxnId aTxnId,
+                                     bool aFinalize) {
   // Don't erase the texture if still in use
   auto result = mTextureInfo.find(aTextureOwnerId);
   if (result == mTextureInfo.end()) {
@@ -1303,10 +1348,21 @@ void CanvasTranslator::RemoveTexture(const RemoteTextureOwnerId aTextureOwnerId,
   if (mRemoteTextureOwner && aTxnType && aTxnId) {
     mRemoteTextureOwner->WaitForTxn(aTextureOwnerId, aTxnType, aTxnId);
   }
-  if (--info.mLocked > 0) {
+  // Remove the DrawTarget only if this is being called from a recorded event
+  // or if there are no remaining keepalives. If this is being called only to
+  // remove a keepalive without forcing removal, then the DrawTarget is still
+  // being used by the recording.
+  if ((aFinalize || info.mKeepAlive <= 1) && info.mRefPtr) {
+    RemoveDrawTarget(info.mRefPtr);
+    info.mRefPtr = ReferencePtr();
+  }
+  if (--info.mKeepAlive > 0) {
     return;
   }
   if (info.mTextureData) {
+    if (info.mFallbackDrawTarget) {
+      info.mTextureData->ReturnDrawTarget(info.mFallbackDrawTarget.forget());
+    }
     info.mTextureData->Unlock();
   }
   if (mRemoteTextureOwner) {
@@ -1474,9 +1530,13 @@ void CanvasTranslator::ClearTextureInfo() {
   mUsedWrapperForSurfaceDescriptor = nullptr;
   mUsedSurfaceDescriptorForSurfaceDescriptor = Nothing();
 
-  for (auto const& entry : mTextureInfo) {
-    if (entry.second.mTextureData) {
-      entry.second.mTextureData->Unlock();
+  for (auto& entry : mTextureInfo) {
+    auto& info = entry.second;
+    if (info.mTextureData) {
+      if (info.mFallbackDrawTarget) {
+        info.mTextureData->ReturnDrawTarget(info.mFallbackDrawTarget.forget());
+      }
+      info.mTextureData->Unlock();
     }
   }
   mTextureInfo.clear();
@@ -1489,8 +1549,10 @@ void CanvasTranslator::ClearTextureInfo() {
   if (sSharedContext && sSharedContext->hasOneRef()) {
     sSharedContext->ClearCaches();
   }
-  mBaseDT = nullptr;
   if (mReferenceTextureData) {
+    if (mBaseDT) {
+      mReferenceTextureData->ReturnDrawTarget(mBaseDT.forget());
+    }
     mReferenceTextureData->Unlock();
   }
   if (mRemoteTextureOwner) {
@@ -1558,9 +1620,6 @@ CanvasTranslator::MaybeRecycleDataSurfaceForSurfaceDescriptor(
     return do_AddRef(usedWrapper);
   }
 
-  usedWrapper = nullptr;
-  usedDescriptor = Some(aSurfaceDescriptor);
-
   bool isYuvVideo = false;
   if (aTextureHost->AsMacIOSurfaceTextureHost()) {
     if (aTextureHost->GetFormat() == SurfaceFormat::NV12 ||
@@ -1571,24 +1630,22 @@ CanvasTranslator::MaybeRecycleDataSurfaceForSurfaceDescriptor(
     isYuvVideo = true;
   }
 
-  if (isYuvVideo && usedSurf && usedSurf->refCount() == 1 &&
-      usedSurf->GetFormat() == gfx::SurfaceFormat::B8G8R8X8 &&
-      aTextureHost->GetSize() == usedSurf->GetSize()) {
-    // Reuse previously used DataSourceSurface if it is not used and same
-    // size/format.
-    usedSurf = aTextureHost->GetAsSurface(usedSurf);
-    // Wrap DataSourceSurface with DataSourceSurfaceWrapper to force upload in
-    // DrawTargetWebgl::DrawSurface().
-    usedWrapper =
-        new gfx::DataSourceSurfaceWrapper(mUsedDataSurfaceForSurfaceDescriptor);
-    return do_AddRef(usedWrapper);
+  // Reuse previously used DataSourceSurface if it is not used and same
+  // size/format.
+  bool reuseSurface = isYuvVideo && usedSurf && usedSurf->refCount() == 1 &&
+                      usedSurf->GetFormat() == gfx::SurfaceFormat::B8G8R8X8 &&
+                      aTextureHost->GetSize() == usedSurf->GetSize();
+  usedSurf =
+      aTextureHost->GetAsSurface(reuseSurface ? usedSurf.get() : nullptr);
+  if (NS_WARN_IF(!usedSurf)) {
+    usedWrapper = nullptr;
+    usedDescriptor = Nothing();
+    return nullptr;
   }
-
-  usedSurf = aTextureHost->GetAsSurface(nullptr);
   // Wrap DataSourceSurface with DataSourceSurfaceWrapper to force upload in
   // DrawTargetWebgl::DrawSurface().
-  usedWrapper =
-      new gfx::DataSourceSurfaceWrapper(mUsedDataSurfaceForSurfaceDescriptor);
+  usedDescriptor = Some(aSurfaceDescriptor);
+  usedWrapper = new gfx::DataSourceSurfaceWrapper(usedSurf);
   return do_AddRef(usedWrapper);
 }
 
@@ -1839,26 +1896,5 @@ void CanvasTranslator::RemoveDataSurface(gfx::ReferencePtr aRefPtr) {
   mDataSurfaces.Remove(aRefPtr);
 }
 
-void CanvasTranslator::SetPreparedMap(
-    gfx::ReferencePtr aSurface,
-    UniquePtr<gfx::DataSourceSurface::ScopedMap> aMap) {
-  mMappedSurface = aSurface;
-  mPreparedMap = std::move(aMap);
-}
-
-UniquePtr<gfx::DataSourceSurface::ScopedMap> CanvasTranslator::GetPreparedMap(
-    gfx::ReferencePtr aSurface) {
-  if (!mPreparedMap) {
-    // We might fail to set the map during, for example, device resets.
-    return nullptr;
-  }
-
-  MOZ_RELEASE_ASSERT(mMappedSurface == aSurface,
-                     "aSurface must match previously stored surface.");
-
-  mMappedSurface = nullptr;
-  return std::move(mPreparedMap);
-}
-
 }  // namespace layers
 }  // namespace mozilla
