@@ -1417,7 +1417,7 @@ void CompositorBridgeParent::PostInsertVsyncProfilerMarker(
   }
 }
 
-widget::PCompositorWidgetParent*
+already_AddRefed<widget::PCompositorWidgetParent>
 CompositorBridgeParent::AllocPCompositorWidgetParent(
     const CompositorWidgetInitData& aInitData) {
 #if defined(MOZ_WIDGET_SUPPORTS_OOP_COMPOSITING)
@@ -1426,13 +1426,12 @@ CompositorBridgeParent::AllocPCompositorWidgetParent(
     return nullptr;
   }
 
-  widget::CompositorWidgetParent* widget =
+  RefPtr<widget::CompositorWidgetParent> widget =
       new widget::CompositorWidgetParent(aInitData, mOptions);
-  widget->AddRef();
 
   // Sending the constructor acts as initialization as well.
   mWidget = widget;
-  return widget;
+  return widget.forget();
 #else
   return nullptr;
 #endif
@@ -1453,16 +1452,6 @@ CompositorBridgeParent::RecvPCompositorWidgetConstructor(
 }
 #endif
 
-bool CompositorBridgeParent::DeallocPCompositorWidgetParent(
-    PCompositorWidgetParent* aActor) {
-#if defined(MOZ_WIDGET_SUPPORTS_OOP_COMPOSITING)
-  static_cast<widget::CompositorWidgetParent*>(aActor)->Release();
-  return true;
-#else
-  return false;
-#endif
-}
-
 CompositorController*
 CompositorBridgeParent::LayerTreeState::GetCompositorController() const {
   return mParent;
