@@ -348,8 +348,8 @@ class WebRenderAPI final {
                wr::WebRenderAPI* aRootDocumentApi = nullptr);
 
   ~WebRenderAPI();
-  // Should be used only for shutdown handling
-  void WaitFlushed();
+
+  void WaitUntilPresentationFlushed();
 
   void UpdateDebugFlags(uint32_t aFlags);
   bool CheckIsRemoteTextureReady(layers::RemoteTextureInfoList* aList,
