@@ -28,6 +28,8 @@ use glyph_rasterizer::SharedFontResources;
 use crate::scene_builder_thread::{SceneBuilderRequest, SceneBuilderResult};
 use crate::intern::InterningMemoryReport;
 use crate::profiler::{self, TransactionProfile};
+#[cfg(feature = "debugger")]
+use crate::debugger::{DebugQuery, DebuggerClient};
 
 #[repr(C)]
 #[derive(Clone, Copy, Debug)]
@@ -952,10 +954,11 @@ pub struct CapturedDocument {
 }
 
 /// Update of the state of built-in debugging facilities.
-#[derive(Clone)]
 pub enum DebugCommand {
     /// Sets the provided debug flags.
     SetFlags(DebugFlags),
+    /// Get current debug flags
+    GetDebugFlags(Sender<DebugFlags>),
     /// Save a capture of all the documents state.
     SaveCapture(PathBuf, CaptureBits),
     /// Load a capture of all the documents state.
@@ -979,6 +982,14 @@ pub enum DebugCommand {
     SetPictureTileSize(Option<DeviceIntSize>),
     /// Set an override for max off-screen surface size
     SetMaximumSurfaceSize(Option<usize>),
+    /// Generate a frame to force a redraw / recomposite
+    GenerateFrame,
+    #[cfg(feature = "debugger")]
+    /// Query internal information about WR
+    Query(DebugQuery),
+    #[cfg(feature = "debugger")]
+    /// Add a new profiler consumer
+    AddDebugClient(DebuggerClient),
 }
 
 /// Message sent by the `RenderApi` to the render backend thread.
@@ -1419,6 +1430,14 @@ impl RenderApi {
         self.send_message(msg);
     }
 
+    /// Get the current debug flags
+    pub fn get_debug_flags(&self) -> DebugFlags {
+        let (tx, rx) = unbounded_channel();
+        let msg = ApiMsg::DebugCommand(DebugCommand::GetDebugFlags(tx));
+        self.send_message(msg);
+        rx.recv().unwrap()
+    }
+
     /// Update the state of builtin debugging facilities.
     pub fn send_debug_cmd(&self, cmd: DebugCommand) {
         let msg = ApiMsg::DebugCommand(cmd);
