@@ -35,6 +35,8 @@ use crate::renderer::{
     upload::UploadTexturePool,
     shade::{Shaders, SharedShaders},
 };
+#[cfg(feature = "debugger")]
+use crate::debugger::Debugger;
 
 use std::{
     mem,
@@ -202,6 +204,14 @@ pub struct WebRenderOptions {
     /// make the result look quite close to the high-quality zoom, except for glyphs.
     pub low_quality_pinch_zoom: bool,
     pub max_shared_surface_size: i32,
+    /// If true, open a debug socket to listen for remote debugger.
+    /// Relies on `debugger` cargo feature being enabled.
+    pub enable_debugger: bool,
+
+    /// Use a more precise method for sampling gradients.
+    pub precise_linear_gradients: bool,
+    pub precise_radial_gradients: bool,
+    pub precise_conic_gradients: bool,
 }
 
 impl WebRenderOptions {
@@ -274,6 +284,10 @@ impl Default for WebRenderOptions {
             reject_software_rasterizer: false,
             low_quality_pinch_zoom: false,
             max_shared_surface_size: 2048,
+            enable_debugger: true,
+            precise_linear_gradients: false,
+            precise_radial_gradients: false,
+            precise_conic_gradients: false,
         }
     }
 }
@@ -570,6 +584,9 @@ pub fn create_webrender_instance(
         low_quality_pinch_zoom: options.low_quality_pinch_zoom,
         max_shared_surface_size: options.max_shared_surface_size,
         enable_dithering: options.enable_dithering,
+        precise_linear_gradients: options.precise_linear_gradients,
+        precise_radial_gradients: options.precise_radial_gradients,
+        precise_conic_gradients: options.precise_conic_gradients,
     };
     info!("WR {:?}", config);
 
@@ -823,11 +840,14 @@ pub fn create_webrender_instance(
         target_frame_publish_id: None,
         pending_result_msg: None,
         layer_compositor_frame_state_in_prev_frame: None,
+        #[cfg(feature = "debugger")]
+        debugger: Debugger::new(),
     };
 
     // We initially set the flags to default and then now call set_debug_flags
     // to ensure any potential transition when enabling a flag is run.
     renderer.set_debug_flags(debug_flags);
+    renderer.profiler.set_ui("Default");
 
     let sender = RenderApiSender::new(
         api_tx,
@@ -836,5 +856,16 @@ pub fn create_webrender_instance(
         blob_image_handler,
         fonts,
     );
+
+    #[cfg(feature = "debugger")]
+    if options.enable_debugger {
+        let api = if namespace_alloc_by_client {
+            sender.create_api_by_client(IdNamespace::DEBUGGER)
+        } else {
+            sender.create_api()
+        };
+        crate::debugger::start(api);
+    }
+
     Ok((renderer, sender))
 }
