@@ -8,7 +8,7 @@
 #define js_loader_ModuleLoaderBase_h
 
 #include "LoadedScript.h"
-#include "ScriptLoadRequest.h"
+#include "ScriptLoadRequestList.h"
 
 #include "ImportMap.h"
 #include "js/ColumnNumber.h"  // JS::ColumnNumberOneOrigin
@@ -18,15 +18,17 @@
 #include "nsCOMArray.h"
 #include "nsCOMPtr.h"
 #include "nsILoadInfo.h"    // nsSecurityFlags
-#include "nsINode.h"        // nsIURI
 #include "nsThreadUtils.h"  // GetMainThreadSerialEventTarget
 #include "nsURIHashKey.h"
 #include "mozilla/Attributes.h"  // MOZ_RAII
 #include "mozilla/CORSMode.h"
 #include "mozilla/MaybeOneOf.h"
 #include "mozilla/UniquePtr.h"
+#include "mozilla/dom/ReferrerPolicyBinding.h"
+#include "mozilla/StaticPrefs_layout.h"
 #include "ResolveResult.h"
 
+class nsIConsoleReportCollector;
 class nsIURI;
 
 namespace mozilla {
@@ -34,6 +36,10 @@ namespace mozilla {
 class LazyLogModule;
 union Utf8Unit;
 
+namespace dom {
+class SRIMetadata;
+}  // namespace dom
+
 }  // namespace mozilla
 
 namespace JS {
@@ -59,14 +65,14 @@ class ModuleScript;
  *
  *     * Error Logging
  *     * Generating the compile options
- *     * Optional: Bytecode Encoding
+ *     * Optional: Caching
  *
  * ScriptLoaderInterface does not provide any implementations.
  * It enables the ModuleLoaderBase to reference back to the behavior implemented
  * by a given ScriptLoader.
  *
- * Not all methods will be used by all ModuleLoaders. For example, Bytecode
- * Encoding does not apply to workers, as we only work with source text there.
+ * Not all methods will be used by all ModuleLoaders. For example, caching
+ * does not apply to workers, as we only work with source text there.
  * Fully virtual methods are implemented by all.
  *
  */
@@ -104,15 +110,15 @@ class ScriptLoaderInterface : public nsISupports {
       JSContext* cx, ScriptLoadRequest* aRequest, CompileOptions* aOptions,
       MutableHandle<JSScript*> aIntroductionScript) = 0;
 
-  virtual void MaybePrepareModuleForBytecodeEncodingBeforeExecute(
+  virtual void MaybePrepareModuleForCacheBeforeExecute(
       JSContext* aCx, ModuleLoadRequest* aRequest) {}
 
-  virtual nsresult MaybePrepareModuleForBytecodeEncodingAfterExecute(
+  virtual nsresult MaybePrepareModuleForCacheAfterExecute(
       ModuleLoadRequest* aRequest, nsresult aRv) {
     return NS_OK;
   }
 
-  virtual void MaybeTriggerBytecodeEncoding() {}
+  virtual void MaybeUpdateCache() {}
 };
 
 class ModuleMapKey : public PLDHashEntryHdr {
@@ -305,16 +311,28 @@ class ModuleLoaderBase : public nsISupports {
   // internally by ModuleLoaderBase.
 
  private:
-  // Create a module load request for a static module import.
-  virtual already_AddRefed<ModuleLoadRequest> CreateStaticImport(
-      nsIURI* aURI, ModuleType aModuleType, ModuleScript* aReferrerScript,
-      const mozilla::dom::SRIMetadata& aSriMetadata,
-      LoadContextBase* aLoadContext, ModuleLoaderBase* aLoader) = 0;
+  // Determine the environment's referrer when the referrer script is empty.
+  //
+  // https://html.spec.whatwg.org/#hostloadimportedmodule
+  //   Step 5. Let fetchReferrer be "client".
+  //
+  // Then check the referrer policy specification for determining the referrer.
+  // https://w3c.github.io/webappsec-referrer-policy/#determine-requests-referrer
+  virtual nsIURI* GetClientReferrerURI() { return nullptr; }
 
-  // Called by HostImportModuleDynamically hook.
-  virtual already_AddRefed<ModuleLoadRequest> CreateDynamicImport(
-      JSContext* aCx, nsIURI* aURI, LoadedScript* aMaybeActiveScript,
-      Handle<JSObject*> aModuleRequestObj, Handle<JSObject*> aPromise) = 0;
+  // Create default ScriptFetchOptions if the referrer script is empty.
+  virtual already_AddRefed<JS::loader::ScriptFetchOptions>
+  CreateDefaultScriptFetchOptions() {
+    return nullptr;
+  }
+
+  // Create a ModuleLoadRequest for static/dynamic imports.
+  virtual already_AddRefed<ModuleLoadRequest> CreateRequest(
+      JSContext* aCx, nsIURI* aURI, Handle<JSObject*> aModuleRequest,
+      Handle<Value> aHostDefined, Handle<Value> aPayload, bool aIsDynamicImport,
+      ScriptFetchOptions* aOptions,
+      mozilla::dom::ReferrerPolicy aReferrerPolicy, nsIURI* aBaseURL,
+      const mozilla::dom::SRIMetadata& aSriMetadata) = 0;
 
   virtual bool IsDynamicImportSupported() { return true; }
 
@@ -386,7 +404,7 @@ class ModuleLoaderBase : public nsISupports {
   nsresult EvaluateModuleInContext(JSContext* aCx, ModuleLoadRequest* aRequest,
                                    ModuleErrorBehaviour errorBehaviour);
 
-  nsresult StartDynamicImport(ModuleLoadRequest* aRequest);
+  void AppendDynamicImport(ModuleLoadRequest* aRequest);
   void ProcessDynamicImport(ModuleLoadRequest* aRequest);
   void CancelAndClearDynamicImports();
 
@@ -404,6 +422,19 @@ class ModuleLoaderBase : public nsISupports {
   // https://html.spec.whatwg.org/multipage/webappapis.html#disallow-further-import-maps
   void DisallowImportMaps() { mImportMapsAllowed = false; }
 
+  virtual bool IsModuleTypeAllowed(ModuleType aModuleType) {
+    if (aModuleType == ModuleType::Unknown) {
+      return false;
+    }
+
+    if (aModuleType == ModuleType::CSS &&
+        !mozilla::StaticPrefs::layout_css_module_scripts_enabled()) {
+      return false;
+    }
+
+    return true;
+  }
+
   // Returns whether there has been an entry in the import map
   // for the given aURI.
   bool GetImportMapSRI(nsIURI* aURI, nsIURI* aSourceURI,
@@ -506,13 +537,6 @@ class ModuleLoaderBase : public nsISupports {
 
   void StartFetchingModuleDependencies(ModuleLoadRequest* aRequest);
 
-  void StartFetchingModuleAndDependencies(JSContext* aCx,
-                                          const ModuleMapKey& aRequestedModule,
-                                          Handle<JSScript*> aReferrer,
-                                          Handle<JSObject*> aModuleRequest,
-                                          Handle<Value> aHostDefined,
-                                          Handle<Value> aPayload);
-
   void InstantiateAndEvaluateDynamicImport(ModuleLoadRequest* aRequest);
 
   static bool OnLoadRequestedModulesResolved(JSContext* aCx, unsigned aArgc,
@@ -525,23 +549,10 @@ class ModuleLoaderBase : public nsISupports {
                                              Handle<Value> aHostDefined,
                                              Handle<Value> aError);
   static bool OnLoadRequestedModulesResolved(ModuleLoadRequest* aRequest);
-  static bool OnLoadRequestedModulesRejected(ModuleLoadRequest* aRequest,
+  static bool OnLoadRequestedModulesRejected(JSContext* aCx,
+                                             ModuleLoadRequest* aRequest,
                                              Handle<Value> aError);
 
-  /**
-   * Shorthand Wrapper for JSAPI FinishDynamicImport function for the reject
-   * case where we do not have `aEvaluationPromise`. As there is no evaluation
-   * Promise, JS::FinishDynamicImport will always reject.
-   *
-   * @param aRequest
-   *        The module load request for the dynamic module.
-   * @param aResult
-   *        The result of running ModuleEvaluate -- If this is successful, then
-   *        we can await the associated EvaluationPromise.
-   */
-  void FinishDynamicImportAndReject(ModuleLoadRequest* aRequest,
-                                    nsresult aResult);
-
   void RemoveDynamicImport(ModuleLoadRequest* aRequest);
 
   nsresult CreateModuleScript(ModuleLoadRequest* aRequest);
@@ -549,6 +560,7 @@ class ModuleLoaderBase : public nsISupports {
 
   void OnFetchSucceeded(ModuleLoadRequest* aRequest);
   void OnFetchFailed(ModuleLoadRequest* aRequest);
+  void Cancel(ModuleLoadRequest* aRequest);
 
   // The slot stored in ImportMetaResolve function.
   enum class ImportMetaSlots : uint32_t { ModulePrivateSlot = 0, SlotCount };
