@@ -37,9 +37,11 @@ namespace js {
 
 class BoundFunctionObject;
 class NativeObject;
+class ObjectFuse;
 class PropertyResult;
 class ProxyObject;
 enum class UnaryMathFunction : uint8_t;
+enum class SetSlotOptimizable;
 
 namespace jit {
 
@@ -56,6 +58,14 @@ enum class InlinableNative : uint16_t;
 void LoadShapeWrapperContents(MacroAssembler& masm, Register obj, Register dst,
                               Label* failure);
 
+enum class NativeGetPropKind {
+  None,
+  Missing,
+  Slot,
+  NativeGetter,
+  ScriptedGetter,
+};
+
 class MOZ_RAII IRGenerator {
  protected:
   CacheIRWriter writer;
@@ -99,6 +109,37 @@ class MOZ_RAII IRGenerator {
   void emitOptimisticClassGuard(ObjOperandId objId, JSObject* obj,
                                 GuardClassKind kind);
 
+  enum class AccessorKind { Getter, Setter };
+  void emitGuardGetterSetterSlot(NativeObject* holder, PropertyInfo prop,
+                                 ObjOperandId holderId, AccessorKind kind,
+                                 bool holderIsConstant = false);
+  void emitCallGetterResultNoGuards(NativeGetPropKind kind, NativeObject* obj,
+                                    NativeObject* holder, PropertyInfo prop,
+                                    ValOperandId receiverId);
+  void emitCallDOMGetterResultNoGuards(NativeObject* holder, PropertyInfo prop,
+                                       ObjOperandId objId);
+
+  void emitCallAccessorGuards(NativeObject* obj, NativeObject* holder,
+                              HandleId id, PropertyInfo prop,
+                              ObjOperandId objId, AccessorKind accessorKind);
+
+  bool canOptimizeConstantDataProperty(NativeObject* holder, PropertyInfo prop,
+                                       ObjectFuse** objFuse);
+  void emitConstantDataPropertyResult(NativeObject* holder,
+                                      ObjOperandId holderId, PropertyKey key,
+                                      PropertyInfo prop, ObjectFuse* objFuse);
+  void emitLoadDataPropertyResult(NativeObject* obj, NativeObject* holder,
+                                  PropertyKey key, PropertyInfo prop,
+                                  ObjOperandId objId);
+
+  bool canOptimizeConstantAccessorProperty(NativeObject* holder,
+                                           PropertyInfo prop,
+                                           ObjectFuse** objFuse);
+  void emitGuardConstantAccessorProperty(NativeObject* holder,
+                                         ObjOperandId holderId, PropertyKey key,
+                                         PropertyInfo prop,
+                                         ObjectFuse* objFuse);
+
   gc::AllocSite* maybeCreateAllocSite();
 
   friend class CacheIRSpewer;
@@ -233,6 +274,17 @@ class MOZ_RAII GetPropIRGenerator : public IRGenerator {
   // matches |id|.
   void maybeEmitIdGuard(jsid id);
 
+  void emitCallGetterResultGuards(NativeObject* obj, NativeObject* holder,
+                                  HandleId id, PropertyInfo prop,
+                                  ObjOperandId objId);
+  void emitCallGetterResult(NativeGetPropKind kind, NativeObject* obj,
+                            NativeObject* holder, HandleId id,
+                            PropertyInfo prop, ObjOperandId objId,
+                            ValOperandId receiverId);
+  void emitCallDOMGetterResult(NativeObject* obj, NativeObject* holder,
+                               HandleId id, PropertyInfo prop,
+                               ObjOperandId objId);
+
   void trackAttached(const char* name /* must be a C string literal */);
 
  public:
@@ -309,6 +361,9 @@ class MOZ_RAII SetPropIRGenerator : public IRGenerator {
   // matches |id|.
   void maybeEmitIdGuard(jsid id);
 
+  SetSlotOptimizable canAttachNativeSetSlot(JSObject* obj, PropertyKey id,
+                                            mozilla::Maybe<PropertyInfo>* prop);
+
   AttachDecision tryAttachNativeSetSlot(HandleObject obj, ObjOperandId objId,
                                         HandleId id, ValOperandId rhsId);
   AttachDecision tryAttachMegamorphicSetSlot(HandleObject obj,
@@ -364,6 +419,12 @@ class MOZ_RAII SetPropIRGenerator : public IRGenerator {
 
   bool canAttachAddSlotStub(HandleObject obj, HandleId id);
 
+  void emitCallSetterNoGuards(NativeObject* obj, NativeObject* holder,
+                              PropertyInfo prop, ObjOperandId receiverId,
+                              ValOperandId rhsId);
+  void emitCallDOMSetterNoGuards(NativeObject* holder, PropertyInfo prop,
+                                 ObjOperandId objId, ValOperandId rhsId);
+
  public:
   SetPropIRGenerator(JSContext* cx, HandleScript script, jsbytecode* pc,
                      CacheKind cacheKind, ICState state, HandleValue lhsVal,
@@ -692,10 +753,9 @@ class MOZ_RAII InlinableNativeIRGenerator {
   AttachDecision tryAttachIsConstructor();
   AttachDecision tryAttachIsCrossRealmArrayConstructor();
   AttachDecision tryAttachCanOptimizeArraySpecies();
+  AttachDecision tryAttachCanOptimizeStringProtoSymbolLookup();
   AttachDecision tryAttachGuardToClass(InlinableNative native);
   AttachDecision tryAttachGuardToClass(GuardClassKind kind);
-  AttachDecision tryAttachGuardToEitherClass(GuardClassKind kind1,
-                                             GuardClassKind kind2);
   AttachDecision tryAttachGuardToArrayBuffer();
   AttachDecision tryAttachGuardToSharedArrayBuffer();
   AttachDecision tryAttachHasClass(const JSClass* clasp,
@@ -703,13 +763,12 @@ class MOZ_RAII InlinableNativeIRGenerator {
   AttachDecision tryAttachRegExpMatcherSearcher(InlinableNative native);
   AttachDecision tryAttachRegExpSearcherLastLimit();
   AttachDecision tryAttachRegExpHasCaptureGroups();
-  AttachDecision tryAttachRegExpPrototypeOptimizable();
-  AttachDecision tryAttachRegExpInstanceOptimizable();
+  AttachDecision tryAttachIsRegExpPrototypeOptimizable();
+  AttachDecision tryAttachIsOptimizableRegExpObject();
   AttachDecision tryAttachIntrinsicRegExpBuiltinExec(InlinableNative native);
   AttachDecision tryAttachIntrinsicRegExpExec(InlinableNative native);
   AttachDecision tryAttachGetFirstDollarIndex();
   AttachDecision tryAttachSubstringKernel();
-  AttachDecision tryAttachObjectHasPrototype();
   AttachDecision tryAttachString();
   AttachDecision tryAttachStringConstructor();
   AttachDecision tryAttachStringToStringValueOf();
@@ -750,13 +809,12 @@ class MOZ_RAII InlinableNativeIRGenerator {
   AttachDecision tryAttachMathPow();
   AttachDecision tryAttachMathMinMax(bool isMax);
   AttachDecision tryAttachSpreadMathMinMax(bool isMax);
+  AttachDecision tryAttachTypedArrayFill();
+  AttachDecision tryAttachTypedArraySet();
+  AttachDecision tryAttachTypedArraySubarray();
   AttachDecision tryAttachIsTypedArray(bool isPossiblyWrapped);
   AttachDecision tryAttachIsTypedArrayConstructor();
-  AttachDecision tryAttachTypedArrayByteOffset();
-  AttachDecision tryAttachTypedArrayElementSize();
-  AttachDecision tryAttachTypedArrayLength(bool isPossiblyWrapped,
-                                           bool allowOutOfBounds);
-  AttachDecision tryAttachArrayBufferByteLength(bool isPossiblyWrapped);
+  AttachDecision tryAttachTypedArrayLength(bool isPossiblyWrapped);
   AttachDecision tryAttachIsConstructing();
   AttachDecision tryAttachGetNextMapSetEntryForIterator(bool isMap);
   AttachDecision tryAttachNewArrayIterator();
@@ -767,6 +825,9 @@ class MOZ_RAII InlinableNativeIRGenerator {
   AttachDecision tryAttachObjectConstructor();
   AttachDecision tryAttachArrayConstructor();
   AttachDecision tryAttachTypedArrayConstructor();
+  AttachDecision tryAttachTypedArrayConstructorFromLength();
+  AttachDecision tryAttachTypedArrayConstructorFromArrayBuffer();
+  AttachDecision tryAttachTypedArrayConstructorFromArray();
   AttachDecision tryAttachMapSetConstructor(InlinableNative native);
   AttachDecision tryAttachNumber();
   AttachDecision tryAttachNumberParseInt();
@@ -802,8 +863,11 @@ class MOZ_RAII InlinableNativeIRGenerator {
   AttachDecision tryAttachMapGet();
   AttachDecision tryAttachMapDelete();
   AttachDecision tryAttachMapSet();
-  AttachDecision tryAttachDateGetTime(InlinableNative native);
+  AttachDecision tryAttachDateGetTime();
   AttachDecision tryAttachDateGet(DateComponent component);
+  AttachDecision tryAttachWeakMapHas();
+  AttachDecision tryAttachWeakMapGet();
+  AttachDecision tryAttachWeakSetHas();
 #ifdef FUZZING_JS_FUZZILLI
   AttachDecision tryAttachFuzzilliHash();
 #endif
@@ -1022,7 +1086,7 @@ inline bool BytecodeCallOpCanHaveInlinableNative(JSOp op) {
 inline bool BytecodeOpCanHaveAllocSite(JSOp op) {
   return BytecodeCallOpCanHaveInlinableNative(op) || op == JSOp::NewArray ||
          op == JSOp::NewObject || op == JSOp::NewInit || op == JSOp::CallIter ||
-         op == JSOp::CallContentIter;
+         op == JSOp::CallContentIter || op == JSOp::Lambda;
 }
 
 class MOZ_RAII CloseIterIRGenerator : public IRGenerator {
