@@ -28,6 +28,7 @@ class JSONPrinter;
 
 class ArrayBufferViewObject;
 class AutoSetNewObjectMetadata;
+class FixedLengthTypedArrayObject;
 class WasmArrayRawBuffer;
 
 namespace wasm {
@@ -481,6 +482,9 @@ class ArrayBufferObject : public ArrayBufferObjectMaybeShared {
   static ArrayBufferObject* createForContents(JSContext* cx, size_t nbytes,
                                               BufferContents contents);
 
+  static ArrayBufferObject* createFromTypedArrayMallocedElements(
+      JSContext* cx, Handle<FixedLengthTypedArrayObject*> tarray);
+
  protected:
   template <class ArrayBufferType>
   static ArrayBufferType* copy(JSContext* cx, size_t newByteLength,
