@@ -28,6 +28,7 @@
 #include "js/AllocationRecording.h"
 #include "js/BuildId.h"  // JS::BuildIdOp
 #include "js/Context.h"
+#include "js/DOMEventDispatch.h"
 #include "js/experimental/CTypes.h"     // JS::CTypesActivityCallback
 #include "js/friend/StackLimits.h"      // js::ReportOverRecursed
 #include "js/friend/UsageStatistics.h"  // JSAccumulateTelemetryDataCallback
@@ -54,7 +55,8 @@
 #include "vm/JSScript.h"
 #include "vm/Logging.h"
 #include "vm/OffThreadPromiseRuntimeState.h"  // js::OffThreadPromiseRuntimeState
-#include "vm/SharedScriptDataTableHolder.h"   // js::SharedScriptDataTableHolder
+#include "vm/RuntimeFuses.h"
+#include "vm/SharedScriptDataTableHolder.h"  // js::SharedScriptDataTableHolder
 #include "vm/Stack.h"
 #include "wasm/WasmTypeDecls.h"
 
@@ -297,31 +299,6 @@ class Metrics {
 #undef DECLARE_METRIC_HELPER
 };
 
-class HasSeenObjectEmulateUndefinedFuse : public js::InvalidatingRuntimeFuse {
-  virtual const char* name() override {
-    return "HasSeenObjectEmulateUndefinedFuse";
-  }
-  virtual bool checkInvariant(JSContext* cx) override {
-    // Without traversing the GC heap I don't think it's possible to assert
-    // this invariant directly.
-    return true;
-  }
-
- public:
-  virtual void popFuse(JSContext* cx) override;
-};
-
-class HasSeenArrayExceedsInt32LengthFuse : public js::InvalidatingRuntimeFuse {
-  virtual const char* name() override {
-    return "HasSeenArrayExceedsInt32LengthFuse";
-  }
-
-  virtual bool checkInvariant(JSContext* cx) override { return true; }
-
- public:
-  virtual void popFuse(JSContext* cx) override;
-};
-
 }  // namespace js
 
 struct JSRuntime {
@@ -480,6 +457,9 @@ struct JSRuntime {
   js::MainThreadData<JSSizeOfIncludingThisCompartmentCallback>
       sizeOfIncludingThisCompartmentCallback;
 
+  /* DOM event dispatch callback for testing. */
+  js::MainThreadData<JS::DispatchDOMEventCallback> dispatchDOMEventCallback;
+
   /* Callback for creating ubi::Nodes representing DOM node objects. Set by
    * JS::ubi::SetConstructUbiNodeForDOMObjectCallback. Refer to
    * js/public/UbiNode.h.
@@ -593,6 +573,9 @@ struct JSRuntime {
   void setTrustedPrincipals(const JSPrincipals* p) { trustedPrincipals_ = p; }
   const JSPrincipals* trustedPrincipals() const { return trustedPrincipals_; }
 
+  void commitPendingWrapperPreservations();
+  void commitPendingWrapperPreservations(JS::Zone* zone);
+
   js::MainThreadData<const JSWrapObjectCallbacks*> wrapObjectCallbacks;
   js::MainThreadData<js::PreserveWrapperCallback> preserveWrapperCallback;
   js::MainThreadData<js::HasReleasedWrapperCallback> hasReleasedWrapperCallback;
@@ -1151,11 +1134,7 @@ struct JSRuntime {
   js::MainThreadData<JS::GlobalCreationCallback>
       shadowRealmGlobalCreationCallback;
 
-  js::MainThreadData<js::HasSeenObjectEmulateUndefinedFuse>
-      hasSeenObjectEmulateUndefinedFuse;
-
-  js::MainThreadData<js::HasSeenArrayExceedsInt32LengthFuse>
-      hasSeenArrayExceedsInt32LengthFuse;
+  js::MainThreadData<js::RuntimeFuses> runtimeFuses;
 };
 
 namespace js {
@@ -1164,53 +1143,6 @@ void Metrics::addTelemetry(JSMetric id, uint32_t sample) {
   rt_->addTelemetry(id, sample);
 }
 
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(Value* vec, size_t len) {
-  // Don't PodZero here because JS::Value is non-trivial.
-  for (size_t i = 0; i < len; i++) {
-    vec[i].setDouble(+0.0);
-  }
-}
-
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(Value* beg, Value* end) {
-  MakeRangeGCSafe(beg, end - beg);
-}
-
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(jsid* beg, jsid* end) {
-  std::fill(beg, end, PropertyKey::Int(0));
-}
-
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(jsid* vec, size_t len) {
-  MakeRangeGCSafe(vec, vec + len);
-}
-
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(Shape** beg, Shape** end) {
-  std::fill(beg, end, nullptr);
-}
-
-static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(Shape** vec, size_t len) {
-  MakeRangeGCSafe(vec, vec + len);
-}
-
-static MOZ_ALWAYS_INLINE void SetValueRangeToUndefined(Value* beg, Value* end) {
-  for (Value* v = beg; v != end; ++v) {
-    v->setUndefined();
-  }
-}
-
-static MOZ_ALWAYS_INLINE void SetValueRangeToUndefined(Value* vec, size_t len) {
-  SetValueRangeToUndefined(vec, vec + len);
-}
-
-static MOZ_ALWAYS_INLINE void SetValueRangeToNull(Value* beg, Value* end) {
-  for (Value* v = beg; v != end; ++v) {
-    v->setNull();
-  }
-}
-
-static MOZ_ALWAYS_INLINE void SetValueRangeToNull(Value* vec, size_t len) {
-  SetValueRangeToNull(vec, vec + len);
-}
-
 extern const JSSecurityCallbacks NullSecurityCallbacks;
 
 // This callback is set by JS::SetProcessLargeAllocationFailureCallback
