@@ -141,19 +141,41 @@ struct AccessCheck {
 
 // Encapsulate all the information about a function call.
 struct FunctionCall {
-  FunctionCall()
-      : restoreRegisterStateAndRealm(false),
-        usesSystemAbi(false),
+  FunctionCall(ABIKind abiKind, RestoreState restoreState)
+      : abi(abiKind),
+        restoreState(restoreState),
+        abiKind(abiKind),
 #ifdef JS_CODEGEN_ARM
         hardFP(true),
 #endif
         frameAlignAdjustment(0),
         stackArgAreaSize(0) {
+    // The system ABI preserves the instance register (as it's in a
+    // non-volatile register) and realm. We just need to reload the HeapReg in
+    // case the memory has been moved.
+    MOZ_ASSERT_IF(abiKind == ABIKind::System,
+                  restoreState == RestoreState::None ||
+                      restoreState == RestoreState::PinnedRegs);
+    // Our uses of the wasm ABI either preserves everything or nothing.
+    MOZ_ASSERT_IF(abiKind == ABIKind::Wasm,
+                  restoreState == RestoreState::None ||
+                      restoreState == RestoreState::All);
+    if (abiKind == ABIKind::System) {
+      // Builtin calls use the system hardFP setting on ARM32.
+#if defined(JS_CODEGEN_ARM)
+      hardFP = ARMFlags::UseHardFpABI();
+      abi.setUseHardFp(hardFP);
+#endif
+    } else {
+#if defined(JS_CODEGEN_ARM)
+      MOZ_ASSERT(hardFP, "The WASM ABI passes FP arguments in registers");
+#endif
+    }
   }
 
-  WasmABIArgGenerator abi;
-  bool restoreRegisterStateAndRealm;
-  bool usesSystemAbi;
+  ABIArgGenerator abi;
+  RestoreState restoreState;
+  ABIKind abiKind;
 #ifdef JS_CODEGEN_ARM
   bool hardFP;
 #endif
@@ -170,13 +192,18 @@ enum class PreBarrierKind {
 };
 
 enum class PostBarrierKind {
+  // No post barrier.
+  None,
+  // Add a store buffer entry if the new value requires it, but do not attempt
+  // to remove a pre-existing entry.
+  Imprecise,
   // Remove an existing store buffer entry if the new value does not require
   // one. This is required to preserve invariants with HeapPtr when used for
   // movable storage.
   Precise,
-  // Add a store buffer entry if the new value requires it, but do not attempt
-  // to remove a pre-existing entry.
-  Imprecise,
+  // Add a store buffer entry for the entire cell (e.g. the entire struct or
+  // array who now has a field pointing into the nursery).
+  WholeCell,
 };
 
 struct BranchIfRefSubtypeRegisters {
@@ -974,17 +1001,45 @@ struct BaseCompiler final {
   //
   // Calls.
 
-  void beginCall(FunctionCall& call, UseABI useABI,
-                 RestoreRegisterStateAndRealm restoreRegisterStateAndRealm);
+  void beginCall(FunctionCall& call);
   void endCall(FunctionCall& call, size_t stackSpace);
+
   void startCallArgs(size_t stackArgAreaSizeUnaligned, FunctionCall* call);
   ABIArg reservePointerArgument(FunctionCall* call);
   void passArg(ValType type, const Stk& arg, FunctionCall* call);
-  CodeOffset callDefinition(uint32_t funcIndex, const FunctionCall& call);
-  CodeOffset callSymbolic(SymbolicAddress callee, const FunctionCall& call);
+
+  // A flag passed to emitCallArgs, describing how the value stack is laid out.
+  enum class CalleeOnStack {
+    // After the arguments to the call, there is a callee pushed onto value
+    // stack.  This is only the case for callIndirect.  To get the arguments to
+    // the call, emitCallArgs has to reach one element deeper into the value
+    // stack, to skip the callee.
+    True,
+
+    // No callee on the stack.
+    False
+  };
+  // The typename T for emitCallArgs can be one of the following:
+  // NormalCallResults, TailCallResults, or NoCallResults.
+  template <typename T>
+  [[nodiscard]] bool emitCallArgs(const ValTypeVector& argTypes, T results,
+                                  FunctionCall* baselineCall,
+                                  CalleeOnStack calleeOnStack);
+
+  [[nodiscard]] bool pushStackResultsForWasmCall(const ResultType& type,
+                                                 RegPtr temp,
+                                                 StackResultsLoc* loc);
+  void popStackResultsAfterWasmCall(const StackResultsLoc& results,
+                                    uint32_t stackArgBytes);
+
+  void pushBuiltinCallResult(const FunctionCall& call, MIRType type);
+  [[nodiscard]] bool pushWasmCallResults(const FunctionCall& call,
+                                         ResultType type,
+                                         const StackResultsLoc& loc);
 
   // Precondition for the call*() methods: sync()
-
+  CodeOffset callDefinition(uint32_t funcIndex, const FunctionCall& call);
+  CodeOffset callSymbolic(SymbolicAddress callee, const FunctionCall& call);
   bool callIndirect(uint32_t funcTypeIndex, uint32_t tableIndex,
                     const Stk& indexVal, const FunctionCall& call,
                     bool tailCall, CodeOffset* fastCallOffset,
@@ -1000,8 +1055,6 @@ struct BaseCompiler final {
   CodeOffset builtinInstanceMethodCall(const SymbolicAddressSignature& builtin,
                                        const ABIArg& instanceArg,
                                        const FunctionCall& call);
-  [[nodiscard]] bool pushCallResults(const FunctionCall& call, ResultType type,
-                                     const StackResultsLoc& loc);
 
   // Helpers to pick up the returned value from the return register.
   inline RegI32 captureReturnedI32();
@@ -1218,10 +1271,8 @@ struct BaseCompiler final {
                    RegI32 temp);
   void load(MemoryAccessDesc* access, AccessCheck* check, RegPtr instance,
             RegPtr memoryBase, RegI32 ptr, AnyReg dest, RegI32 temp);
-#ifdef ENABLE_WASM_MEMORY64
   void load(MemoryAccessDesc* access, AccessCheck* check, RegPtr instance,
             RegPtr memoryBase, RegI64 ptr, AnyReg dest, RegI64 temp);
-#endif
 
   template <typename RegType>
   void doLoadCommon(MemoryAccessDesc* access, AccessCheck check, ValType type);
@@ -1235,10 +1286,8 @@ struct BaseCompiler final {
                     RegI32 temp);
   void store(MemoryAccessDesc* access, AccessCheck* check, RegPtr instance,
              RegPtr memoryBase, RegI32 ptr, AnyReg src, RegI32 temp);
-#ifdef ENABLE_WASM_MEMORY64
   void store(MemoryAccessDesc* access, AccessCheck* check, RegPtr instance,
              RegPtr memoryBase, RegI64 ptr, AnyReg src, RegI64 temp);
-#endif
 
   template <typename RegType>
   void doStoreCommon(MemoryAccessDesc* access, AccessCheck check,
@@ -1348,28 +1397,53 @@ struct BaseCompiler final {
   // update.  This function preserves that register.
   void emitPreBarrier(RegPtr valueAddr);
 
-  // This emits a GC post-write barrier. The post-barrier is needed when we
+  // These emit GC post-write barriers. The post-barrier is needed when we
   // replace a member field with a new value, the new value is in the nursery,
-  // and the containing object is a tenured object. The field must then be
-  // added to the store buffer so that the nursery can be correctly collected.
-  // The field might belong to an object or be a stack slot or a register or a
-  // heap allocated value.
+  // and the containing object is a tenured object. The field (or the entire
+  // containing object) must then be added to the store buffer so that the
+  // nursery can be correctly collected. The field might belong to an object or
+  // be a stack slot or a register or a heap allocated value.
   //
   // For the difference between 'precise' and 'imprecise', look at the
   // documentation on PostBarrierKind.
+
+  // Emits a post-write barrier that creates a whole-cell store buffer entry.
+  // See above for details.
   //
-  // `object` is a pointer to the object that contains the field. It is used, if
-  // present, to skip adding a store buffer entry when the containing object is
-  // in the nursery. This register is preserved by this function.
-  // `valueAddr` is the address of the location that we are writing to. This
-  // register is consumed by this function.
-  // `prevValue` is the value that existed in the field before `value` was
-  // stored. This register is consumed by this function.
-  // `value` is the value that was stored in the field. This register is
-  // preserved by this function.
-  [[nodiscard]] bool emitPostBarrierImprecise(
+  // - `object` is a pointer to the object that contains the field. This
+  //   register is preserved by this function.
+  // - `value` is the value that was stored in the field. This register is
+  //   preserved by this function.
+  // - `temp` is consumed by this function.
+  [[nodiscard]] bool emitPostBarrierWholeCell(RegRef object, RegRef value,
+                                              RegPtr temp);
+
+  // Emits a post-write barrier of type WasmAnyRefEdge, imprecisely. See above
+  // for details.
+  //
+  // - `object` is a pointer to the object that contains the field. It is used,
+  //   if present, to skip adding a store buffer entry when the containing
+  //   object is in the nursery. This register is preserved by this function.
+  // - `valueAddr` is the address of the location that we are writing to. This
+  //   register is consumed by this function.
+  // - `value` is the value that was stored in the field. This register is
+  //   preserved by this function.
+  [[nodiscard]] bool emitPostBarrierEdgeImprecise(
       const mozilla::Maybe<RegRef>& object, RegPtr valueAddr, RegRef value);
-  [[nodiscard]] bool emitPostBarrierPrecise(
+
+  // Emits a post-write barrier of type WasmAnyRefEdge, precisely. See above for
+  // details.
+  //
+  // - `object` is a pointer to the object that contains the field. It is used,
+  //   if present, to skip adding a store buffer entry when the containing
+  //   object is in the nursery. This register is preserved by this function.
+  // - `valueAddr` is the address of the location that we are writing to. This
+  //   register is consumed by this function.
+  // - `prevValue` is the value that existed in the field before `value` was
+  //   stored. This register is consumed by this function.
+  // - `value` is the value that was stored in the field. This register is
+  //   preserved by this function.
+  [[nodiscard]] bool emitPostBarrierEdgePrecise(
       const mozilla::Maybe<RegRef>& object, RegPtr valueAddr, RegRef prevValue,
       RegRef value);
 
@@ -1405,7 +1479,7 @@ struct BaseCompiler final {
   // Jump to the given branch, passing results, if the WasmGcObject, `object`,
   // is a subtype of `destType`.
   [[nodiscard]] bool jumpConditionalWithResults(BranchState* b, RegRef object,
-                                                RefType sourceType,
+                                                MaybeRefType sourceType,
                                                 RefType destType,
                                                 bool onSuccess);
   template <typename Cond>
@@ -1440,26 +1514,6 @@ struct BaseCompiler final {
   [[nodiscard]] bool emitBrTable();
   [[nodiscard]] bool emitDrop();
   [[nodiscard]] bool emitReturn();
-
-  // A flag passed to emitCallArgs, describing how the value stack is laid out.
-  enum class CalleeOnStack {
-    // After the arguments to the call, there is a callee pushed onto value
-    // stack.  This is only the case for callIndirect.  To get the arguments to
-    // the call, emitCallArgs has to reach one element deeper into the value
-    // stack, to skip the callee.
-    True,
-
-    // No callee on the stack.
-    False
-  };
-
-  // The typename T for emitCallArgs can be one of the following:
-  // NormalCallResults, TailCallResults, or NoCallResults.
-  template <typename T>
-  [[nodiscard]] bool emitCallArgs(const ValTypeVector& argTypes, T results,
-                                  FunctionCall* baselineCall,
-                                  CalleeOnStack calleeOnStack);
-
   [[nodiscard]] bool emitCall();
   [[nodiscard]] bool emitReturnCall();
   [[nodiscard]] bool emitCallIndirect();
@@ -1492,12 +1546,6 @@ struct BaseCompiler final {
   [[nodiscard]] bool endTryTable(ResultType type);
 
   void doReturn(ContinuationKind kind);
-  void pushReturnValueOfCall(const FunctionCall& call, MIRType type);
-
-  [[nodiscard]] bool pushStackResultsForCall(const ResultType& type,
-                                             RegPtr temp, StackResultsLoc* loc);
-  void popStackResultsAfterCall(const StackResultsLoc& results,
-                                uint32_t stackArgBytes);
 
   void emitCompareI32(Assembler::Condition compareOp, ValType compareType);
   void emitCompareI64(Assembler::Condition compareOp, ValType compareType);
@@ -1727,7 +1775,8 @@ struct BaseCompiler final {
   [[nodiscard]] bool emitBrOnCastCommon(bool onSuccess,
                                         uint32_t labelRelativeDepth,
                                         const ResultType& labelType,
-                                        RefType sourceType, RefType destType);
+                                        MaybeRefType sourceType,
+                                        RefType destType);
   [[nodiscard]] bool emitBrOnCast(bool onSuccess);
   [[nodiscard]] bool emitAnyConvertExtern();
   [[nodiscard]] bool emitExternConvertAny();
@@ -1799,7 +1848,8 @@ struct BaseCompiler final {
 
   [[nodiscard]] bool emitGcArraySet(RegRef object, RegPtr data, RegI32 index,
                                     const ArrayType& array, AnyReg value,
-                                    PreBarrierKind preBarrierKind);
+                                    PreBarrierKind preBarrierKind,
+                                    PostBarrierKind postBarrierKind);
 
 #ifdef ENABLE_WASM_SIMD
   void emitVectorAndNot();
