@@ -41,15 +41,19 @@ namespace js {
 class SharedArrayRawBuffer;
 class WasmBreakpointSite;
 
+class WasmGcObject;
 class WasmStructObject;
 class WasmArrayObject;
 
+struct AllocationMetadataBuilder;
+
 namespace gc {
 class StoreBuffer;
 }  // namespace gc
 
 namespace wasm {
 
+struct CodeTailMetadata;
 struct FuncDefInstanceData;
 class FuncImport;
 struct FuncImportInstanceData;
@@ -142,7 +146,7 @@ class alignas(16) Instance {
   // to assert that we can use compact offsets on x86(-64) for these fields.
   // We cannot have the assertion here, due to C++ 'offsetof' rules.
   static constexpr size_t offsetOfLastCommonJitField() {
-    return offsetof(Instance, addressOfNeedsIncrementalBarrier_);
+    return offsetof(Instance, allocSites_);
   }
 
   // The number of baseline scratch storage words available.
@@ -162,9 +166,6 @@ class alignas(16) Instance {
   // easily use a symbolic address.
   const JSClass* valueBoxClass_;
 
-  // Address of the JitRuntime's arguments rectifier trampoline
-  void* jsJitArgsRectifier_;
-
   // Address of the JitRuntime's exception handler trampoline
   void* jsJitExceptionHandler_;
 
@@ -212,11 +213,16 @@ class alignas(16) Instance {
   void* allocatedBase_;
 
   // Fields from the JS context for memory allocation, stashed on the instance
-  // so it can be accessed from JIT code.
+  // so it can be accessed from JIT code efficiently.
   const void* addressOfNurseryPosition_;
 #ifdef JS_GC_ZEAL
   const void* addressOfGCZealModeBits_;
 #endif
+  const js::AllocationMetadataBuilder* allocationMetadataBuilder_;
+
+  // A copy of the runtime's addressOfLastBufferedWholeCell, used for whole-cell
+  // store buffer entries.
+  const void* addressOfLastBufferedWholeCell_;
 
   // Pointer to a per-module builtin stub that will request tier-up for the
   // wasm function that calls it.
@@ -229,7 +235,7 @@ class alignas(16) Instance {
   // The data must be the last field.  Globals for the module start here
   // and are inline in this structure.  16-byte alignment is required for SIMD
   // data.
-  MOZ_ALIGNED_DECL(16, char data_);
+  alignas(16) char data_;
 
   // Internal helpers:
   FuncDefInstanceData* funcDefInstanceData(uint32_t funcIndex) const;
@@ -322,6 +328,12 @@ class alignas(16) Instance {
   static constexpr size_t offsetOfAllocSites() {
     return offsetof(Instance, allocSites_);
   }
+  static constexpr size_t offsetOfAllocationMetadataBuilder() {
+    return offsetof(Instance, allocationMetadataBuilder_);
+  }
+  static constexpr size_t offsetOfAddressOfLastBufferedWholeCell() {
+    return offsetof(Instance, addressOfLastBufferedWholeCell_);
+  }
   static constexpr size_t offsetOfAddressOfNeedsIncrementalBarrier() {
     return offsetof(Instance, addressOfNeedsIncrementalBarrier_);
   }
@@ -334,9 +346,6 @@ class alignas(16) Instance {
   static constexpr size_t sizeOfBaselineScratchWords() {
     return sizeof(baselineScratchWords_);
   }
-  static constexpr size_t offsetOfJSJitArgsRectifier() {
-    return offsetof(Instance, jsJitArgsRectifier_);
-  }
   static constexpr size_t offsetOfJSJitExceptionHandler() {
     return offsetof(Instance, jsJitExceptionHandler_);
   }
@@ -391,7 +400,13 @@ class alignas(16) Instance {
   void setTemporaryStackLimit(JS::NativeStackLimit limit);
   void resetTemporaryStackLimit(JSContext* cx);
 
-  int32_t computeInitialHotnessCounter(uint32_t funcIndex);
+  void setAllocationMetadataBuilder(
+      const js::AllocationMetadataBuilder* allocationMetadataBuilder) {
+    allocationMetadataBuilder_ = allocationMetadataBuilder;
+  }
+
+  int32_t computeInitialHotnessCounter(uint32_t funcIndex,
+                                       size_t codeSectionSize);
   void resetHotnessCounter(uint32_t funcIndex);
   int32_t readHotnessCounter(uint32_t funcIndex) const;
   void submitCallRefHints(uint32_t funcIndex);
@@ -401,6 +416,7 @@ class alignas(16) Instance {
 
   const Code& code() const { return *code_; }
   inline const CodeMetadata& codeMeta() const;
+  inline const CodeTailMetadata& codeTailMeta() const;
   inline const CodeMetadataForAsmJS* codeMetaForAsmJS() const;
   inline bool isAsmJS() const;
 
@@ -587,11 +603,10 @@ class alignas(16) Instance {
   static int32_t wake_m64(Instance* instance, uint64_t byteOffset,
                           int32_t count, uint32_t memoryIndex);
   static void* refFunc(Instance* instance, uint32_t funcIndex);
-  static void postBarrier(Instance* instance, void** location);
-  static void postBarrierPrecise(Instance* instance, void** location,
-                                 void* prev);
-  static void postBarrierPreciseWithOffset(Instance* instance, void** base,
-                                           uint32_t offset, void* prev);
+  static void postBarrierEdge(Instance* instance, AnyRef* location);
+  static void postBarrierEdgePrecise(Instance* instance, AnyRef* location,
+                                     void* prev);
+  static void postBarrierWholeCell(Instance* instance, gc::Cell* object);
   static void* exceptionNew(Instance* instance, void* exceptionArg);
   static int32_t throwException(Instance* instance, void* exceptionArg);
   template <bool ZeroFields>
@@ -625,7 +640,6 @@ class alignas(16) Instance {
   static int32_t intrI8VecMul(Instance* instance, uint32_t dest, uint32_t src1,
                               uint32_t src2, uint32_t len, uint8_t* memBase);
 
-#ifdef ENABLE_WASM_JS_STRING_BUILTINS
   static int32_t stringTest(Instance* instance, void* stringArg);
   static void* stringCast(Instance* instance, void* stringArg);
   static void* stringFromCharCodeArray(Instance* instance, void* arrayArg,
@@ -647,7 +661,6 @@ class alignas(16) Instance {
                               void* secondStringArg);
   static int32_t stringCompare(Instance* instance, void* firstStringArg,
                                void* secondStringArg);
-#endif  // ENABLE_WASM_JS_STRING_BUILTINS
 };
 
 bool ResultsToJSValue(JSContext* cx, ResultType type, void* registerResultLoc,
