@@ -100,6 +100,12 @@ using namespace js::wasm;
 #      define RLR_sig(p) ((p)->sc_lr)
 #      define R31_sig(p) ((p)->sc_sp)
 #    endif
+#    if defined(__riscv)
+#      define RPC_sig(p) ((p)->sc_sepc)
+#      define RRA_sig(p) ((p)->sc_ra)
+#      define RFP_sig(p) ((p)->sc_s[0])
+#      define R02_sig(p) ((p)->sc_sp)
+#    endif
 #    if defined(__mips__)
 #      define EPC_sig(p) ((p)->sc_pc)
 #      define RFP_sig(p) ((p)->sc_regs[30])
@@ -522,8 +528,8 @@ struct AutoHandlingTrap {
   }
 
   Trap trap;
-  TrapSiteDesc trapDesc;
-  if (!codeBlock->lookupTrap(pc, &trap, &trapDesc)) {
+  TrapSite trapSite;
+  if (!codeBlock->lookupTrap(pc, &trap, &trapSite)) {
     return false;
   }
 
@@ -546,7 +552,7 @@ struct AutoHandlingTrap {
   // point of the trap to allow stack unwinding or resumption, both of which
   // will call finishWasmTrap().
   jit::JitActivation* activation = cx->activation()->asJit();
-  activation->startWasmTrap(trap, trapDesc, ToRegisterState(context));
+  activation->startWasmTrap(trap, trapSite, ToRegisterState(context));
   SetContextPC(context, codeBlock->code->trapCode());
   return true;
 }
@@ -985,8 +991,8 @@ bool wasm::MemoryAccessTraps(const RegisterState& regs, uint8_t* addr,
   }
 
   Trap trap;
-  TrapSiteDesc trapDesc;
-  if (!codeBlock->code->lookupTrap(regs.pc, &trap, &trapDesc)) {
+  TrapSite trapSite;
+  if (!codeBlock->code->lookupTrap(regs.pc, &trap, &trapSite)) {
     return false;
   }
   switch (trap) {
@@ -1034,7 +1040,7 @@ bool wasm::MemoryAccessTraps(const RegisterState& regs, uint8_t* addr,
 
   JSContext* cx = TlsContext.get();  // Cold simulator helper function
   jit::JitActivation* activation = cx->activation()->asJit();
-  activation->startWasmTrap(trap, trapDesc, regs);
+  activation->startWasmTrap(trap, trapSite, regs);
   *newPC = codeBlock->code->trapCode();
   return true;
 #endif
@@ -1051,14 +1057,14 @@ bool wasm::HandleIllegalInstruction(const RegisterState& regs,
   }
 
   Trap trap;
-  TrapSiteDesc trapDesc;
-  if (!codeBlock->code->lookupTrap(regs.pc, &trap, &trapDesc)) {
+  TrapSite trapSite;
+  if (!codeBlock->code->lookupTrap(regs.pc, &trap, &trapSite)) {
     return false;
   }
 
   JSContext* cx = TlsContext.get();  // Cold simulator helper function
   jit::JitActivation* activation = cx->activation()->asJit();
-  activation->startWasmTrap(trap, trapDesc, regs);
+  activation->startWasmTrap(trap, trapSite, regs);
   *newPC = codeBlock->code->trapCode();
   return true;
 #endif
