@@ -9,7 +9,10 @@
 #ifndef nsPresContext_h___
 #define nsPresContext_h___
 
-#include "mozilla/intl/Bidi.h"
+#include "FontVisibilityProvider.h"
+#include "Units.h"
+#include "gfxRect.h"
+#include "gfxTypes.h"
 #include "mozilla/AppUnits.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DepthOrderedFrameList.h"
@@ -23,27 +26,25 @@
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/WeakPtr.h"
+#include "mozilla/intl/Bidi.h"
 #include "mozilla/widget/ThemeChangeKind.h"
+#include "nsAtom.h"
+#include "nsCOMPtr.h"
+#include "nsChangeHint.h"
 #include "nsColor.h"
 #include "nsCompatibility.h"
 #include "nsCoord.h"
-#include "nsCOMPtr.h"
+#include "nsCycleCollectionParticipant.h"
 #include "nsFontMetrics.h"
+#include "nsGkAtoms.h"
 #include "nsHashKeys.h"
+#include "nsIWidgetListener.h"  // for nsSizeMode
 #include "nsRect.h"
 #include "nsStringFwd.h"
+#include "nsTArray.h"
 #include "nsTHashSet.h"
 #include "nsTHashtable.h"
-#include "nsAtom.h"
-#include "nsIWidgetListener.h"  // for nsSizeMode
-#include "nsGkAtoms.h"
-#include "nsCycleCollectionParticipant.h"
-#include "nsChangeHint.h"
-#include "gfxTypes.h"
-#include "gfxRect.h"
-#include "nsTArray.h"
 #include "nsThreadUtils.h"
-#include "Units.h"
 
 class nsIPrintSettings;
 class nsDocShell;
@@ -128,7 +129,9 @@ class nsRootPresContext;
 // An interface for presentation contexts. Presentation contexts are
 // objects that provide an outer context for a presentation shell.
 
-class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
+class nsPresContext : public nsISupports,
+                      public mozilla::SupportsWeakPtr,
+                      public FontVisibilityProvider {
  public:
   using Encoding = mozilla::Encoding;
   template <typename T>
@@ -143,6 +146,8 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS_FINAL
   NS_DECL_CYCLE_COLLECTION_CLASS(nsPresContext)
 
+  FONT_VISIBILITY_PROVIDER_IMPL
+
   enum nsPresContextType : uint8_t {
     eContext_Galley,        // unpaginated screen presentation
     eContext_PrintPreview,  // paginated screen presentation
@@ -165,21 +170,6 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
 
   void UpdateFontCacheUserFonts(gfxUserFontSet* aUserFontSet);
 
-  /**
-   * Return the font visibility level to be applied to this context,
-   * potentially blocking user-installed or non-standard fonts from being
-   * used by web content.
-   * Note that depending on ResistFingerprinting options, the caller may
-   * override this value when resolving CSS <generic-family> keywords.
-   */
-  FontVisibility GetFontVisibility() const { return mFontVisibility; }
-
-  /**
-   * Log a message to the console about a font request being blocked.
-   */
-  void ReportBlockedFontFamily(const mozilla::fontlist::Family& aFamily);
-  void ReportBlockedFontFamily(const gfxFontFamily& aFamily);
-
   /**
    * Get the nsFontMetrics that describe the properties of
    * an nsFont.
@@ -480,6 +470,12 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
    */
   bool HasPaginatedScrolling() const { return mCanPaginatedScroll; }
 
+  uint32_t LastScrollGeneration() { return mLastScrollGeneration; }
+  void UpdateLastScrollGeneration() { mLastScrollGeneration += 1; }
+  bool HasBeenScrolledSince(const uint32_t& mPreviousScrollGeneration) const {
+    return mPreviousScrollGeneration < mLastScrollGeneration;
+  }
+
   /**
    * Get/set the size of a page
    */
@@ -917,16 +913,8 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
 
   bool IsPrintPreview() const { return mType == eContext_PrintPreview; }
 
-  // Is this presentation in a chrome docshell?
-  bool IsChrome() const;
-
   gfxUserFontSet* GetUserFontSet();
 
-  // Should be called whenever the set of fonts available in the user
-  // font set changes (e.g., because a new font loads, or because the
-  // user font set is changed and fonts become unavailable).
-  void UserFontSetUpdated(gfxUserFontEntry* aUpdatedFont = nullptr);
-
   gfxMissingFontRecorder* MissingFontRecorder() { return mMissingFonts.get(); }
 
   void NotifyMissingFonts();
@@ -952,8 +940,6 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
   // be dispatched to MozAfterPaint events when NotifyDidPaintForSubtree is
   // called for the transaction id (or any higher id).
   void NotifyInvalidation(TransactionId aTransactionId, const nsRect& aRect);
-  // aRect is in device pixels
-  void NotifyInvalidation(TransactionId aTransactionId, const nsIntRect& aRect);
   void NotifyDidPaintForSubtree(
       TransactionId aTransactionId = TransactionId{0},
       const mozilla::TimeStamp& aTimeStamp = mozilla::TimeStamp());
@@ -1082,7 +1068,7 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
 
   void FinishedContainerQueryUpdate();
 
-  bool UpdateContainerQueryStyles();
+  void UpdateContainerQueryStylesAndAnchorPosLayout();
 
   mozilla::intl::Bidi& BidiEngine();
 
@@ -1152,6 +1138,18 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
     return mMarkPaintTimingStart;
   }
 
+  // Are we using normalized ruby-positioning metrics? This will fetch and
+  // cache the pref if not already initialized.
+  bool NormalizeRubyMetrics();
+
+  // Get the scaling factor to apply to em-normalized font ascent/descent. This
+  // should only be used if NormalizeRubyMetrics() has returned true, otherwise
+  // its return value may be meaningless.
+  float RubyPositioningFactor() const {
+    MOZ_ASSERT(mRubyPositioningFactor > 0.0f);
+    return mRubyPositioningFactor;
+  }
+
  protected:
   // May be called multiple times (unlink, destructor)
   void Destroy();
@@ -1163,7 +1161,8 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
   // Creates a one-shot timer with the given aCallback & aDelay.
   // Returns a refcounted pointer to the timer (or nullptr on failure).
   already_AddRefed<nsITimer> CreateTimer(nsTimerCallbackFunc aCallback,
-                                         const char* aName, uint32_t aDelay);
+                                         const nsACString& aName,
+                                         uint32_t aDelay);
 
   struct TransactionInvalidations {
     TransactionId mTransactionId;
@@ -1180,8 +1179,6 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
   // visibile to CSS. Returns whether the current visibility value actually
   // changed (in which case content should be reflowed).
   bool UpdateFontVisibility();
-  void ReportBlockedFontFamilyName(const nsCString& aFamily,
-                                   FontVisibility aVisibility);
 
   // IMPORTANT: The ownership implicit in the following member variables
   // has been explicitly checked.  If you add any members to this class,
@@ -1218,11 +1215,14 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
 
   float mTextZoom;  // Text zoom, defaults to 1.0
   float mFullZoom;  // Page zoom, defaults to 1.0
+
   gfxSize mLastFontInflationScreenSize;
 
   int32_t mCurAppUnitsPerDevPixel;
   int32_t mAutoQualityMinFontSizePixelsPref;
 
+  float mRubyPositioningFactor = -1.0f;  // negative indicates uninitialized
+
   nsCOMPtr<nsITheme> mTheme;
   nsCOMPtr<nsIPrintSettings> mPrintSettings;
 
@@ -1293,6 +1293,11 @@ class nsPresContext : public nsISupports, public mozilla::SupportsWeakPtr {
   mozilla::TimeStamp mFirstMouseMoveTime;
   mozilla::TimeStamp mFirstScrollTime;
 
+  // incremented each time the root scroller scrolls, helpful to determine if
+  // it has scrolled between the start and end of some deferred work (such as a
+  // navigation intercept).
+  uint32_t mLastScrollGeneration;
+
   // last time we did a full style flush
   mozilla::TimeStamp mLastStyleUpdateForAllAnimations;
 
