@@ -8,26 +8,26 @@
 #include "SVGGeometryFrame.h"
 
 // Keep others in (case-insensitive) order:
+#include "SVGAnimatedTransformList.h"
+#include "SVGMarkerFrame.h"
 #include "gfx2DGlue.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
 #include "gfxUtils.h"
+#include "mozilla/ArrayUtils.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/SVGContentUtils.h"
+#include "mozilla/SVGContextPaint.h"
+#include "mozilla/SVGObserverUtils.h"
+#include "mozilla/SVGUtils.h"
+#include "mozilla/StaticPrefs_svg.h"
 #include "mozilla/dom/SVGGeometryElement.h"
 #include "mozilla/dom/SVGGraphicsElement.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/gfx/Helpers.h"
-#include "mozilla/ArrayUtils.h"
-#include "mozilla/PresShell.h"
-#include "mozilla/RefPtr.h"
-#include "mozilla/StaticPrefs_svg.h"
-#include "mozilla/SVGContextPaint.h"
-#include "mozilla/SVGContentUtils.h"
-#include "mozilla/SVGObserverUtils.h"
-#include "mozilla/SVGUtils.h"
 #include "nsGkAtoms.h"
 #include "nsLayoutUtils.h"
-#include "SVGAnimatedTransformList.h"
-#include "SVGMarkerFrame.h"
 
 using namespace mozilla::dom;
 using namespace mozilla::gfx;
@@ -64,8 +64,7 @@ void SVGGeometryFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
 }
 
 nsresult SVGGeometryFrame::AttributeChanged(int32_t aNameSpaceID,
-                                            nsAtom* aAttribute,
-                                            int32_t aModType) {
+                                            nsAtom* aAttribute, AttrModType) {
   // We don't invalidate for transform changes (the layers code does that).
   // Also note that SVGTransformableElement::GetAttributeChangeHint will
   // return nsChangeHint_UpdateOverflow for "transform" attribute changes
@@ -121,7 +120,8 @@ void SVGGeometryFrame::DidSetComputedStyle(ComputedStyle* aOldComputedStyle) {
     NotifySVGChanged(TRANSFORM_CHANGED);
   }
 
-  if (element->IsGeometryChangedViaCSS(*Style(), *aOldComputedStyle)) {
+  if (element->IsGeometryChangedViaCSS(*Style(), *aOldComputedStyle) ||
+      aOldComputedStyle->EffectiveZoom() != Style()->EffectiveZoom()) {
     element->ClearAnyCachedPath();
     SVGObserverUtils::InvalidateRenderingObservers(this);
   }
