@@ -11,19 +11,18 @@
 #ifndef nsMenuPopupFrame_h__
 #define nsMenuPopupFrame_h__
 
+#include "Units.h"
 #include "mozilla/Attributes.h"
-#include "mozilla/gfx/Types.h"
 #include "mozilla/StaticPrefs_ui.h"
 #include "mozilla/TimeStamp.h"
+#include "mozilla/gfx/Types.h"
 #include "nsAtom.h"
+#include "nsBlockFrame.h"
 #include "nsCOMPtr.h"
+#include "nsExpirationState.h"
 #include "nsIDOMEventListener.h"
 #include "nsXULPopupManager.h"
 
-#include "nsBlockFrame.h"
-
-#include "Units.h"
-
 class nsIWidget;
 
 namespace mozilla {
@@ -196,15 +195,18 @@ class nsMenuPopupFrame final : public nsBlockFrame {
             nsIFrame* aPrevInFlow) override;
 
   nsresult AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
-                            int32_t aModType) override;
+                            AttrModType aModType) override;
 
   // FIXME: This shouldn't run script (this can end up calling HidePopup).
   MOZ_CAN_RUN_SCRIPT_BOUNDARY void Destroy(DestroyContext&) override;
 
   bool HasRemoteContent() const;
 
-  // Whether we should create a widget on Init().
-  bool ShouldCreateWidgetUpfront() const;
+  // Whether this is a drag popup to show drag feedback.
+  bool IsDragPopup() const;
+
+  // Whether we should have a widget even when we're not shown.
+  bool ShouldHaveWidgetWhenHidden() const;
 
   // Whether we should expand the menu to take the size of the parent menulist.
   bool ShouldExpandToInflowParentOrAnchor() const;
@@ -264,14 +266,15 @@ class nsMenuPopupFrame final : public nsBlockFrame {
     return mPopupState == ePopupOpening || mPopupState == ePopupVisible ||
            mPopupState == ePopupShown;
   }
-  bool IsVisible() {
+  bool IsVisible() const {
     return mPopupState == ePopupVisible || mPopupState == ePopupShown;
   }
-  bool IsVisibleOrShowing() {
+  bool IsVisibleOrShowing() const {
     return IsOpen() || mPopupState == ePopupPositioning ||
            mPopupState == ePopupShowing;
   }
   bool IsNativeMenu() const { return mIsNativeMenu; }
+  bool CanSkipLayout() const;
   bool IsMouseTransparent() const;
 
   // Return true if the popup is for a menulist.
@@ -550,14 +553,15 @@ class nsMenuPopupFrame final : public nsBlockFrame {
   int8_t GetPopupAnchor() const { return mPopupAnchor; }
   FlipType GetFlipType() const { return mFlip; }
 
-  void WidgetPositionOrSizeDidChange();
-
   uint64_t GetAPZFocusSequenceNumber() const { return mAPZFocusSequenceNumber; }
 
   void UpdateAPZFocusSequenceNumber(uint64_t aNewNumber) {
     mAPZFocusSequenceNumber = aNewNumber;
   }
 
+  void DestroyWidgetIfNeeded();
+  nsExpirationState* GetExpirationState() { return &mExpirationState; }
+
  protected:
   nsString mIncrementalString;  // for incremental typing navigation
 
@@ -661,8 +665,9 @@ class nsMenuPopupFrame final : public nsBlockFrame {
 
   nsRect mOverrideConstraintRect;
 
-  static mozilla::TimeStamp sLastKeyTime;
+  nsExpirationState mExpirationState;
 
+  static mozilla::TimeStamp sLastKeyTime;
 };  // class nsMenuPopupFrame
 
 #endif
