@@ -266,6 +266,12 @@ nsresult CacheEntry::HashingKey(const nsACString& aStorageID,
   return HashingKey(aStorageID, aEnhanceID, spec, aResult);
 }
 
+// The hash key (which is also the filename) is:
+// A[~B]:C
+// Where A is the storage ID ([O<oa>,][a,][p,]), B is the optional 'id',
+// and C is the URI  'oa' are the OriginAttributes in suffix form
+// (i.e. |^key=value&key2=value2|)
+
 // static
 nsresult CacheEntry::HashingKey(const nsACString& aStorageID,
                                 const nsACString& aEnhanceID,
@@ -289,6 +295,12 @@ nsresult CacheEntry::HashingKey(const nsACString& aStorageID,
   return NS_OK;
 }
 
+nsresult CacheEntry::SetDictionary(DictionaryCacheEntry* aDict) {
+  mDict = aDict;
+  mFile->SetDictionary(aDict);
+  return NS_OK;
+}
+
 void CacheEntry::AsyncOpen(nsICacheEntryOpenCallback* aCallback,
                            uint32_t aFlags) {
   bool readonly = aFlags & nsICacheStorage::OPEN_READONLY;
@@ -521,7 +533,7 @@ already_AddRefed<CacheEntryHandle> CacheEntry::ReopenTruncated(
   mLock.AssertCurrentThreadOwns();
 
   // Hold callbacks invocation, AddStorageEntry would invoke from doom
-  // prematurly
+  // prematurely
   mPreventCallbacks = true;
 
   RefPtr<CacheEntryHandle> handle;
@@ -1083,6 +1095,12 @@ nsresult CacheEntry::GetOnStopTime(uint64_t* aTime) {
   return mFile->GetOnStopTime(aTime);
 }
 
+nsresult CacheEntry::GetReadyOrRevalidating(bool* aReady) {
+  mozilla::MutexAutoLock lock(mLock);
+  *aReady = (mState == READY || mState == REVALIDATING);
+  return NS_OK;
+}
+
 nsresult CacheEntry::SetNetworkTimes(uint64_t aOnStartTime,
                                      uint64_t aOnStopTime) {
   if (NS_SUCCEEDED(mFileStatus)) {
