@@ -5,9 +5,9 @@
 
 #include "CookieParser.h"
 #include "CookieLogging.h"
+#include "CookieValidation.h"
 
 #include "mozilla/CheckedInt.h"
-#include "mozilla/dom/nsMixedContentBlocker.h"
 #include "mozilla/glean/NetwerkMetrics.h"
 #include "mozilla/net/Cookie.h"
 #include "mozilla/StaticPrefs_network.h"
@@ -21,14 +21,7 @@
 constexpr char ATTRIBUTE_PATH[] = "path";
 constexpr uint64_t ATTRIBUTE_MAX_LENGTH = 1024;
 
-constexpr auto CONSOLE_CHIPS_CATEGORY = "cookiesCHIPS"_ns;
-constexpr auto CONSOLE_OVERSIZE_CATEGORY = "cookiesOversize"_ns;
-constexpr auto CONSOLE_REJECTION_CATEGORY = "cookiesRejection"_ns;
-constexpr auto CONSOLE_SAMESITE_CATEGORY = "cookieSameSite"_ns;
 constexpr auto CONSOLE_INVALID_ATTRIBUTE_CATEGORY = "cookieInvalidAttribute"_ns;
-constexpr auto SAMESITE_MDN_URL =
-    "https://developer.mozilla.org/docs/Web/HTTP/Headers/Set-Cookie/"
-    u"SameSite"_ns;
 
 namespace mozilla {
 namespace net {
@@ -43,6 +36,10 @@ CookieParser::CookieParser(nsIConsoleReportCollector* aCRC, nsIURI* aHostURI)
 CookieParser::~CookieParser() {
   MOZ_COUNT_DTOR(CookieParser);
 
+  if (mValidation) {
+    mValidation->ReportErrorsAndWarnings(mCRC, mHostURI);
+  }
+
 #define COOKIE_LOGGING_WITH_NAME(category, x)                 \
   CookieLogging::LogMessageToConsole(                         \
       mCRC, mHostURI, nsIScriptError::errorFlag, category, x, \
@@ -57,77 +54,11 @@ CookieParser::~CookieParser() {
                                "CookieRejectedInvalidCharAttributes"_ns);
       break;
 
-    case RejectedNoneRequiresSecure:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_SAMESITE_CATEGORY,
-                               "CookieRejectedNonRequiresSecure2"_ns);
-      break;
-
-    case RejectedPartitionedRequiresSecure:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedPartitionedRequiresSecure"_ns);
-      break;
-
-    case RejectedEmptyNameAndValue:
-      CookieLogging::LogMessageToConsole(
-          mCRC, mHostURI, nsIScriptError::warningFlag,
-          CONSOLE_REJECTION_CATEGORY, "CookieRejectedEmptyNameAndValue"_ns,
-          nsTArray<nsString>());
-
-      break;
-
-    case RejectedNameValueOversize: {
-      AutoTArray<nsString, 2> params = {
-          NS_ConvertUTF8toUTF16(mCookieData.name())};
-
-      nsString size;
-      size.AppendInt(kMaxBytesPerCookie);
-      params.AppendElement(size);
-
-      CookieLogging::LogMessageToConsole(
-          mCRC, mHostURI, nsIScriptError::warningFlag,
-          CONSOLE_OVERSIZE_CATEGORY, "CookieOversize"_ns, params);
-      break;
-    }
-
-    case RejectedInvalidCharName:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedInvalidCharName"_ns);
-      break;
-
-    case RejectedInvalidDomain:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedInvalidDomain"_ns);
-      break;
-
-    case RejectedInvalidPrefix:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedInvalidPrefix"_ns);
-      break;
-    case RejectedInvalidPath:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedInvalidPath"_ns);
-      break;
-
-    case RejectedInvalidCharValue:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedInvalidCharValue"_ns);
-      break;
-
     case RejectedHttpOnlyButFromScript:
       COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
                                "CookieRejectedHttpOnlyButFromScript"_ns);
       break;
 
-    case RejectedSecureButNonHttps:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_REJECTION_CATEGORY,
-                               "CookieRejectedSecureButNonHttps"_ns);
-      break;
-
-    case RejectedForNonSameSiteness:
-      COOKIE_LOGGING_WITH_NAME(CONSOLE_SAMESITE_CATEGORY,
-                               "CookieRejectedForNonSameSiteness"_ns);
-      break;
-
     case RejectedForeignNoPartitionedError:
       COOKIE_LOGGING_WITH_NAME(CONSOLE_CHIPS_CATEGORY,
                                "CookieForeignNoPartitionedError"_ns);
@@ -146,7 +77,8 @@ CookieParser::~CookieParser() {
 
 #undef COOKIE_LOGGING_WITH_NAME
 
-  if (mRejection != NoRejection || !mContainsCookie) {
+  if (mRejection != NoRejection || !mValidation ||
+      mValidation->Result() != nsICookieValidation::eOK) {
     return;
   }
 
@@ -185,29 +117,6 @@ CookieParser::~CookieParser() {
         AutoTArray<nsString, 1>{NS_ConvertUTF8toUTF16(mCookieData.name())});
   }
 
-  if (mWarnings.mSameSiteNoneRequiresSecureForBeta) {
-    CookieLogging::LogMessageToConsole(
-        mCRC, mHostURI, nsIScriptError::warningFlag, CONSOLE_SAMESITE_CATEGORY,
-        "CookieRejectedNonRequiresSecureForBeta3"_ns,
-        AutoTArray<nsString, 2>{NS_ConvertUTF8toUTF16(mCookieData.name()),
-                                SAMESITE_MDN_URL});
-  }
-
-  if (mWarnings.mSameSiteLaxForced) {
-    CookieLogging::LogMessageToConsole(
-        mCRC, mHostURI, nsIScriptError::infoFlag, CONSOLE_SAMESITE_CATEGORY,
-        "CookieLaxForced2"_ns,
-        AutoTArray<nsString, 1>{NS_ConvertUTF8toUTF16(mCookieData.name())});
-  }
-
-  if (mWarnings.mSameSiteLaxForcedForBeta) {
-    CookieLogging::LogMessageToConsole(
-        mCRC, mHostURI, nsIScriptError::warningFlag, CONSOLE_SAMESITE_CATEGORY,
-        "CookieLaxForcedForBeta2"_ns,
-        AutoTArray<nsString, 2>{NS_ConvertUTF8toUTF16(mCookieData.name()),
-                                SAMESITE_MDN_URL});
-  }
-
   if (mWarnings.mForeignNoPartitionedWarning) {
     CookieLogging::LogMessageToConsole(
         mCRC, mHostURI, nsIScriptError::warningFlag, CONSOLE_CHIPS_CATEGORY,
@@ -355,12 +264,6 @@ void CookieParser::GetTokenValue(nsACString::const_char_iterator& aIter,
   }
 }
 
-static inline void SetSameSiteAttribute(CookieStruct& aCookieData,
-                                        int32_t aValue) {
-  aCookieData.sameSite() = aValue;
-  aCookieData.rawSameSite() = aValue;
-}
-
 // Tests for control characters, defined by RFC 5234 to be %x00-1F / %x7F.
 // An exception is made for HTAB as the cookie spec treats that as whitespace.
 static bool ContainsControlChars(const nsACString& aString) {
@@ -372,13 +275,6 @@ static bool ContainsControlChars(const nsACString& aString) {
          }) != end;
 }
 
-static inline void SetSameSiteAttributeDefault(CookieStruct& aCookieData) {
-  // Set cookie with SameSite attribute that is treated as Default
-  // and doesn't requires changing the DB schema.
-  aCookieData.sameSite() = nsICookie::SAMESITE_LAX;
-  aCookieData.rawSameSite() = nsICookie::SAMESITE_NONE;
-}
-
 // static
 bool CookieParser::CheckAttributeSize(const nsACString& currentValue,
                                       const char* aAttribute,
@@ -428,8 +324,7 @@ void CookieParser::ParseAttributes(nsCString& aCookieHeader,
   mCookieData.isSecure() = false;
   mCookieData.isHttpOnly() = false;
   mCookieData.isPartitioned() = false;
-
-  SetSameSiteAttributeDefault(mCookieData);
+  mCookieData.sameSite() = nsICookie::SAMESITE_UNSET;
 
   nsDependentCSubstring tokenString(cookieStart, cookieStart);
   nsDependentCSubstring tokenValue(cookieStart, cookieStart);
@@ -493,61 +388,21 @@ void CookieParser::ParseAttributes(nsCString& aCookieHeader,
 
     } else if (tokenString.LowerCaseEqualsLiteral(kSameSite)) {
       if (tokenValue.LowerCaseEqualsLiteral(kSameSiteLax)) {
-        SetSameSiteAttribute(mCookieData, nsICookie::SAMESITE_LAX);
+        mCookieData.sameSite() = nsICookie::SAMESITE_LAX;
       } else if (tokenValue.LowerCaseEqualsLiteral(kSameSiteStrict)) {
-        SetSameSiteAttribute(mCookieData, nsICookie::SAMESITE_STRICT);
+        mCookieData.sameSite() = nsICookie::SAMESITE_STRICT;
       } else if (tokenValue.LowerCaseEqualsLiteral(kSameSiteNone)) {
-        SetSameSiteAttribute(mCookieData, nsICookie::SAMESITE_NONE);
+        mCookieData.sameSite() = nsICookie::SAMESITE_NONE;
       } else {
         // Reset to Default if unknown token value (see Bug 1682450)
-        SetSameSiteAttributeDefault(mCookieData);
+        mCookieData.sameSite() = nsICookie::SAMESITE_UNSET;
         mWarnings.mInvalidSameSiteAttribute = true;
       }
     }
   }
 
-  // re-assign aCookieHeader, in case we need to process another cookie
-  aCookieHeader.Assign(Substring(cookieStart, cookieEnd));
-
-  // If same-site is explicitly set to 'none' but this is not a secure context,
-  // let's abort the parsing.
-  if (!mCookieData.isSecure() &&
-      mCookieData.sameSite() == nsICookie::SAMESITE_NONE) {
-    if (StaticPrefs::network_cookie_sameSite_noneRequiresSecure()) {
-      RejectCookie(RejectedNoneRequiresSecure);
-      return;
-    }
-
-    // Still warn about the missing Secure attribute when not enforcing.
-    mWarnings.mSameSiteNoneRequiresSecureForBeta = true;
-  }
-
-  // Ensure the partitioned cookie is set with the secure attribute if CHIPS
-  // is enabled.
-  if (StaticPrefs::network_cookie_CHIPS_enabled() &&
-      mCookieData.isPartitioned() && !mCookieData.isSecure()) {
-    RejectCookie(RejectedPartitionedRequiresSecure);
-    return;
-  }
-
-  if (mCookieData.rawSameSite() == nsICookie::SAMESITE_NONE &&
-      mCookieData.sameSite() == nsICookie::SAMESITE_LAX) {
-    bool laxByDefault =
-        StaticPrefs::network_cookie_sameSite_laxByDefault() &&
-        !nsContentUtils::IsURIInPrefList(
-            mHostURI, "network.cookie.sameSite.laxByDefault.disabledHosts");
-    if (laxByDefault) {
-      mWarnings.mSameSiteLaxForced = true;
-    } else if (StaticPrefs::
-                   network_cookie_sameSite_laxByDefaultWarningsForBeta()) {
-      mWarnings.mSameSiteLaxForcedForBeta = true;
-    }
-  }
-
   // Cookie accepted.
   aAcceptedByParser = true;
-
-  MOZ_ASSERT(Cookie::ValidateSameSite(mCookieData));
 }
 
 namespace {
@@ -584,73 +439,14 @@ nsAutoCString GetPathFromURI(nsIURI* aHostURI) {
 }  // namespace
 
 // static
-bool CookieParser::CheckPath(CookieStruct& aCookieData, nsIURI* aHostURI,
-                             CookieParser* aParser) {
+void CookieParser::FixPath(CookieStruct& aCookieData, nsIURI* aHostURI) {
   // if a path is given, check the host has permission
   if (aCookieData.path().IsEmpty() || aCookieData.path().First() != '/') {
     nsAutoCString path = GetPathFromURI(aHostURI);
-    if (CheckAttributeSize(aCookieData.path(), ATTRIBUTE_PATH, path, aParser)) {
+    if (CheckAttributeSize(aCookieData.path(), ATTRIBUTE_PATH, path)) {
       aCookieData.path() = path;
     }
   }
-
-  MOZ_ASSERT(CookieCommons::CheckPathSize(aCookieData));
-
-  return !aCookieData.path().Contains('\t');
-}
-
-// static
-bool CookieParser::HasSecurePrefix(const nsACString& aString) {
-  return StringBeginsWith(aString, "__Secure-"_ns,
-                          nsCaseInsensitiveCStringComparator);
-}
-
-// static
-bool CookieParser::HasHostPrefix(const nsACString& aString) {
-  return StringBeginsWith(aString, "__Host-"_ns,
-                          nsCaseInsensitiveCStringComparator);
-}
-
-// CheckPrefixes
-//
-// Reject cookies whose name starts with the magic prefixes from
-// https://datatracker.ietf.org/doc/html/draft-ietf-httpbis-rfc6265bis
-// if they do not meet the criteria required by the prefix.
-//
-// Must not be called until after CheckDomain() and CheckPath() have
-// regularized and validated the CookieStruct values!
-
-// static
-bool CookieParser::CheckPrefixes(CookieStruct& aCookieData,
-                                 bool aSecureRequest) {
-  bool hasSecurePrefix = HasSecurePrefix(aCookieData.name());
-  bool hasHostPrefix = HasHostPrefix(aCookieData.name());
-
-  if (!hasSecurePrefix && !hasHostPrefix) {
-    // not one of the magic prefixes: carry on
-    return true;
-  }
-
-  if (!aSecureRequest || !aCookieData.isSecure()) {
-    // the magic prefixes may only be used from a secure request and
-    // the secure attribute must be set on the cookie
-    return false;
-  }
-
-  if (hasHostPrefix) {
-    // The host prefix requires that the path is "/" and that the cookie
-    // had no domain attribute. CheckDomain() and CheckPath() MUST be run
-    // first to make sure invalid attributes are rejected and to regularlize
-    // them. In particular all explicit domain attributes result in a host
-    // that starts with a dot, and if the host doesn't start with a dot it
-    // correctly matches the true host.
-    if (aCookieData.host()[0] == '.' ||
-        !aCookieData.path().EqualsLiteral("/")) {
-      return false;
-    }
-  }
-
-  return true;
 }
 
 bool CookieParser::ParseMaxAgeAttribute(const nsACString& aMaxage,
@@ -701,7 +497,8 @@ bool CookieParser::ParseMaxAgeAttribute(const nsACString& aMaxage,
 
 bool CookieParser::GetExpiry(CookieStruct& aCookieData,
                              const nsACString& aExpires,
-                             const nsACString& aMaxage, int64_t aCurrentTime,
+                             const nsACString& aMaxage,
+                             int64_t aCurrentTimeInMSec,
                              const nsACString& aDateHeader, bool aFromHttp) {
   int64_t maxageCap = StaticPrefs::network_cookie_maxageCap();
 
@@ -716,11 +513,11 @@ bool CookieParser::GetExpiry(CookieStruct& aCookieData,
   // check for max-age attribute first; this overrides expires attribute
   int64_t maxage = 0;
   if (ParseMaxAgeAttribute(aMaxage, &maxage)) {
-    if (maxage == INT64_MIN || maxage == INT64_MAX) {
+    if (maxage == INT64_MIN) {
       aCookieData.expiry() = maxage;
     } else {
-      CheckedInt<int64_t> value(aCurrentTime);
-      value += maxageCap ? std::min(maxage, maxageCap) : maxage;
+      CheckedInt<int64_t> value(aCurrentTimeInMSec);
+      value += (maxageCap ? std::min(maxage, maxageCap) : maxage) * 1000;
 
       aCookieData.expiry() = value.isValid() ? value.value() : INT64_MAX;
     }
@@ -731,13 +528,13 @@ bool CookieParser::GetExpiry(CookieStruct& aCookieData,
   // check for expires attribute
   if (!aExpires.IsEmpty()) {
     // parse expiry time
-    PRTime expiresTime;
-    if (PR_ParseTimeString(aExpires.BeginReading(), true, &expiresTime) !=
+    PRTime expiresTimeInUSec;
+    if (PR_ParseTimeString(aExpires.BeginReading(), true, &expiresTimeInUSec) !=
         PR_SUCCESS) {
       return true;
     }
 
-    int64_t expires = expiresTime / int64_t(PR_USEC_PER_SEC);
+    int64_t expiresInMSec = expiresTimeInUSec / int64_t(PR_USEC_PER_MSEC);
 
     // If we have the server time, we can adjust the "expire" attribute value
     // by adding the delta between the server and the local times.  If the
@@ -746,13 +543,14 @@ bool CookieParser::GetExpiry(CookieStruct& aCookieData,
     if (!aDateHeader.IsEmpty()) {
       MOZ_ASSERT(aFromHttp);
 
-      PRTime dateHeaderTime;
+      PRTime dateHeaderTimeInUSec;
       if (PR_ParseTimeString(aDateHeader.BeginReading(), true,
-                             &dateHeaderTime) == PR_SUCCESS &&
+                             &dateHeaderTimeInUSec) == PR_SUCCESS &&
           StaticPrefs::network_cookie_useServerTime()) {
-        int64_t serverTime = dateHeaderTime / int64_t(PR_USEC_PER_SEC);
-        int64_t delta = aCurrentTime - serverTime;
-        expires += delta;
+        int64_t serverTimeInMSec =
+            dateHeaderTimeInUSec / int64_t(PR_USEC_PER_MSEC);
+        int64_t delta = aCurrentTimeInMSec - serverTimeInMSec;
+        expiresInMSec += delta;
       }
     }
 
@@ -761,12 +559,9 @@ bool CookieParser::GetExpiry(CookieStruct& aCookieData,
     // Because if current time be set in the future, but the cookie expire
     // time be set less than current time and more than server time.
     // The cookie item have to be used to the expired cookie.
-    if (maxageCap) {
-      aCookieData.expiry() = std::min(expires, aCurrentTime + maxageCap);
-    } else {
-      aCookieData.expiry() = expires;
-    }
 
+    aCookieData.expiry() =
+        CookieCommons::MaybeCapExpiry(aCurrentTimeInMSec, expiresInMSec);
     return false;
   }
 
@@ -776,24 +571,10 @@ bool CookieParser::GetExpiry(CookieStruct& aCookieData,
   return true;
 }
 
-// returns true if 'a' is equal to or a subdomain of 'b',
-// assuming no leading dots are present.
-static inline bool IsSubdomainOf(const nsACString& a, const nsACString& b) {
-  if (a == b) {
-    return true;
-  }
-  if (a.Length() > b.Length()) {
-    return a[a.Length() - b.Length() - 1] == '.' && StringEndsWith(a, b);
-  }
-  return false;
-}
-
-// processes domain attribute, and returns true if host has permission to set
-// for this domain.
 // static
-bool CookieParser::CheckDomain(CookieStruct& aCookieData, nsIURI* aHostURI,
-                               const nsACString& aBaseDomain,
-                               bool aRequireHostMatch) {
+void CookieParser::FixDomain(CookieStruct& aCookieData, nsIURI* aHostURI,
+                             const nsACString& aBaseDomain,
+                             bool aRequireHostMatch) {
   // Note: The logic in this function is mirrored in
   // toolkit/components/extensions/ext-cookies.js:checkSetCookiePermissions().
   // If it changes, please update that function, or file a bug for someone
@@ -803,47 +584,53 @@ bool CookieParser::CheckDomain(CookieStruct& aCookieData, nsIURI* aHostURI,
   nsAutoCString hostFromURI;
   nsContentUtils::GetHostOrIPv6WithBrackets(aHostURI, hostFromURI);
 
-  // if a domain is given, check the host has permission
-  if (!aCookieData.host().IsEmpty()) {
-    // Tolerate leading '.' characters, but not if it's otherwise an empty host.
-    if (aCookieData.host().Length() > 1 && aCookieData.host().First() == '.') {
-      aCookieData.host().Cut(0, 1);
-    }
+  // no domain specified, use hostFromURI
+  if (aCookieData.host().IsEmpty()) {
+    aCookieData.host() = hostFromURI;
+    return;
+  }
 
-    // switch to lowercase now, to avoid case-insensitive compares everywhere
-    ToLowerCase(aCookieData.host());
+  nsCString cookieHost = aCookieData.host();
 
+  // Tolerate leading '.' characters, but not if it's otherwise an empty host.
+  if (cookieHost.Length() > 1 && cookieHost.First() == '.') {
+    cookieHost.Cut(0, 1);
+  }
+
+  // switch to lowercase now, to avoid case-insensitive compares everywhere
+  ToLowerCase(cookieHost);
+
+  if (aRequireHostMatch) {
     // check whether the host is either an IP address, an alias such as
     // 'localhost', an eTLD such as 'co.uk', or the empty string. in these
     // cases, require an exact string match for the domain, and leave the cookie
     // as a non-domain one. bug 105917 originally noted the requirement to deal
     // with IP addresses.
-    if (aRequireHostMatch) {
-      return hostFromURI.Equals(aCookieData.host());
+    if (hostFromURI.Equals(cookieHost)) {
+      aCookieData.host() = cookieHost;
     }
 
-    // ensure the proposed domain is derived from the base domain; and also
-    // that the host domain is derived from the proposed domain (per RFC2109).
-    if (IsSubdomainOf(aCookieData.host(), aBaseDomain) &&
-        IsSubdomainOf(hostFromURI, aCookieData.host())) {
-      // prepend a dot to indicate a domain cookie
-      aCookieData.host().InsertLiteral(".", 0);
-      return true;
-    }
-
-    /*
-     * note: RFC2109 section 4.3.2 requires that we check the following:
-     * that the portion of host not in domain does not contain a dot.
-     * this prevents hosts of the form x.y.co.nz from setting cookies in the
-     * entire .co.nz domain. however, it's only a only a partial solution and
-     * it breaks sites (IE doesn't enforce it), so we don't perform this check.
-     */
-    return false;
+    // If the match fails, we keep the aCookieData.Host() as it was. The
+    // Validator will reject the cookie with the correct reason.
+    return;
   }
 
-  // no domain specified, use hostFromURI
-  aCookieData.host() = hostFromURI;
-  return true;
+  // ensure the proposed domain is derived from the base domain; and also
+  // that the host domain is derived from the proposed domain (per RFC2109).
+  if (CookieCommons::IsSubdomainOf(cookieHost, aBaseDomain) &&
+      CookieCommons::IsSubdomainOf(hostFromURI, cookieHost)) {
+    // prepend a dot to indicate a domain cookie
+    cookieHost.InsertLiteral(".", 0);
+    aCookieData.host() = cookieHost;
+  }
+
+  /*
+   * note: RFC2109 section 4.3.2 requires that we check the following:
+   * that the portion of host not in domain does not contain a dot.
+   * this prevents hosts of the form x.y.co.nz from setting cookies in the
+   * entire .co.nz domain. however, it's only a only a partial solution and
+   * it breaks sites (IE doesn't enforce it), so we don't perform this check.
+   */
 }
 
 static void RecordPartitionedTelemetry(const CookieStruct& aCookieData,
@@ -861,98 +648,16 @@ static void RecordPartitionedTelemetry(const CookieStruct& aCookieData,
   }
 }
 
-// static
-CookieParser::Rejection CookieParser::CheckCookieStruct(
-    CookieStruct& aCookieStruct, nsIURI* aHostURI,
-    const nsCString& aCookieString, const nsACString& aBaseDomain,
-    bool aRequireHostMatch, bool aFromHttp, CookieParser* aParser) {
-  // reject cookie if name and value are empty, per RFC6265bis
-  if (aCookieStruct.name().IsEmpty() && aCookieStruct.value().IsEmpty()) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "cookie name and value are empty");
-
-    return RejectedEmptyNameAndValue;
-  }
-
-  // reject cookie if it's over the size limit, per RFC2109
-  if (!CookieCommons::CheckNameAndValueSize(aCookieStruct)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "cookie too big (> 4kb)");
-    return RejectedNameValueOversize;
-  }
-
-  if (!CookieCommons::CheckName(aCookieStruct)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "invalid name character");
-    return RejectedInvalidCharName;
-  }
-
-  // domain & path checks
-  if (!CheckDomain(aCookieStruct, aHostURI, aBaseDomain, aRequireHostMatch)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "failed the domain tests");
-    return RejectedInvalidDomain;
-  }
-
-  if (!CheckPath(aCookieStruct, aHostURI, aParser)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "failed the path tests");
-    return RejectedInvalidPath;
-  }
-
-  // If a cookie is nameless, then its value must not start with
-  // `__Host-` or `__Secure-`
-  if (aCookieStruct.name().IsEmpty() &&
-      (HasSecurePrefix(aCookieStruct.value()) ||
-       HasHostPrefix(aCookieStruct.value()))) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "failed hidden prefix tests");
-    return RejectedInvalidPrefix;
-  }
-
-  bool potentiallyTrustworthy =
-      nsMixedContentBlocker::IsPotentiallyTrustworthyOrigin(aHostURI);
-
-  // magic prefix checks. MUST be run after CheckDomain() and CheckPath()
-  if (!CheckPrefixes(aCookieStruct, potentiallyTrustworthy)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "failed the prefix tests");
-    return RejectedInvalidPrefix;
-  }
-
-  if (!CookieCommons::CheckValue(aCookieStruct)) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "invalid value character");
-    return RejectedInvalidCharValue;
-  }
-
-  // if the new cookie is httponly, make sure we're not coming from script
-  if (!aFromHttp && aCookieStruct.isHttpOnly()) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "cookie is httponly; coming from script");
-    return RejectedHttpOnlyButFromScript;
-  }
-
-  // If the new cookie is non-https and wants to set secure flag,
-  // browser have to ignore this new cookie.
-  // (draft-ietf-httpbis-cookie-alone section 3.1)
-  if (aCookieStruct.isSecure() && !potentiallyTrustworthy) {
-    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                      "non-https cookie can't set secure flag");
-    return RejectedSecureButNonHttps;
-  }
-
-  return NoRejection;
-}
-
-// processes a single cookie, and returns true if there are more cookies
-// to be processed
+// Main entry point for cookie parsing. Parses a single cookie string
+// (from either document.cookie or a Set-Cookie header) and populates
+// the internal CookieStruct data.
 void CookieParser::Parse(const nsACString& aBaseDomain, bool aRequireHostMatch,
                          CookieStatus aStatus, nsCString& aCookieHeader,
                          const nsACString& aDateHeader, bool aFromHttp,
                          bool aIsForeignAndNotAddon, bool aPartitionedOnly,
-                         bool aIsInPrivateBrowsing, bool aOn3pcbException) {
-  MOZ_ASSERT(!mContainsCookie);
+                         bool aIsInPrivateBrowsing, bool aOn3pcbException,
+                         int64_t aCurrentTimeInMSec) {
+  MOZ_ASSERT(!mValidation);
 
   // init expiryTime such that session cookies won't prematurely expire
   mCookieData.expiry() = INT64_MAX;
@@ -973,47 +678,17 @@ void CookieParser::Parse(const nsACString& aBaseDomain, bool aRequireHostMatch,
     return;
   }
 
-  int64_t currentTimeInUsec = PR_Now();
-
   // calculate expiry time of cookie.
-  mCookieData.isSession() =
-      GetExpiry(mCookieData, expires, maxage,
-                currentTimeInUsec / PR_USEC_PER_SEC, aDateHeader, aFromHttp);
+  mCookieData.isSession() = GetExpiry(
+      mCookieData, expires, maxage, aCurrentTimeInMSec, aDateHeader, aFromHttp);
   if (aStatus == STATUS_ACCEPT_SESSION) {
     // force lifetime to session. note that the expiration time, if set above,
     // will still apply.
     mCookieData.isSession() = true;
   }
 
-  // We count SetCookie operations in the parent process only for HTTP set
-  // cookies to prevent double counting.
-  if (XRE_IsParentProcess() || !aFromHttp) {
-    RecordPartitionedTelemetry(mCookieData, aIsForeignAndNotAddon);
-  }
-
-  auto result =
-      CheckCookieStruct(mCookieData, mHostURI, mCookieString, aBaseDomain,
-                        aRequireHostMatch, aFromHttp, this);
-  if (result != NoRejection) {
-    RejectCookie(result);
-    return;
-  }
-
-  // If the new cookie is same-site but in a cross site context,
-  // browser must ignore the cookie.
-  bool laxByDefault =
-      StaticPrefs::network_cookie_sameSite_laxByDefault() &&
-      !nsContentUtils::IsURIInPrefList(
-          mHostURI, "network.cookie.sameSite.laxByDefault.disabledHosts");
-  auto effectiveSameSite =
-      laxByDefault ? mCookieData.sameSite() : mCookieData.rawSameSite();
-  if ((effectiveSameSite != nsICookie::SAMESITE_NONE) &&
-      aIsForeignAndNotAddon) {
-    COOKIE_LOGFAILURE(SET_COOKIE, mHostURI, mCookieString,
-                      "failed the samesite tests");
-    RejectCookie(RejectedForNonSameSiteness);
-    return;
-  }
+  FixDomain(mCookieData, mHostURI, aBaseDomain, aRequireHostMatch);
+  FixPath(mCookieData, mHostURI);
 
   // If the cookie is on the 3pcd exception list, we apply partitioned
   // attribute to the cookie.
@@ -1038,8 +713,6 @@ void CookieParser::Parse(const nsACString& aBaseDomain, bool aRequireHostMatch,
         (aIsInPrivateBrowsing &&
          StaticPrefs::
              network_cookie_cookieBehavior_optInPartitioning_pbmode())) {
-      COOKIE_LOGFAILURE(SET_COOKIE, mHostURI, mCookieString,
-                        "foreign cookies must be partitioned");
       RejectCookie(RejectedForeignNoPartitionedError);
       return;
     }
@@ -1047,7 +720,20 @@ void CookieParser::Parse(const nsACString& aBaseDomain, bool aRequireHostMatch,
     mWarnings.mForeignNoPartitionedWarning = true;
   }
 
-  mContainsCookie = true;
+  mValidation = CookieValidation::ValidateInContext(
+      mCookieData, mHostURI, aBaseDomain, aRequireHostMatch, aFromHttp,
+      aIsForeignAndNotAddon, aPartitionedOnly, aIsInPrivateBrowsing);
+  MOZ_ASSERT(mValidation);
+
+  if (mValidation->Result() != nsICookieValidation::eOK) {
+    return;
+  }
+
+  // We count SetCookie operations in the parent process only for HTTP set
+  // cookies to prevent double counting.
+  if (XRE_IsParentProcess() || !aFromHttp) {
+    RecordPartitionedTelemetry(mCookieData, aIsForeignAndNotAddon);
+  }
 }
 
 void CookieParser::RejectCookie(Rejection aRejection) {
