@@ -370,6 +370,16 @@ ObliviousHttpChannel::SetRequestContextID(uint64_t rcID) {
   return mInnerChannel->SetRequestContextID(rcID);
 }
 
+NS_IMETHODIMP
+ObliviousHttpChannel::GetIsUserAgentHeaderOutdated(bool* aValue) {
+  return mInnerChannel->GetIsUserAgentHeaderOutdated(aValue);
+}
+
+NS_IMETHODIMP
+ObliviousHttpChannel::SetIsUserAgentHeaderOutdated(bool aValue) {
+  return mInnerChannel->SetIsUserAgentHeaderOutdated(aValue);
+}
+
 NS_IMETHODIMP
 ObliviousHttpChannel::GetProtocolVersion(nsACString& aProtocolVersion) {
   return mInnerChannel->GetProtocolVersion(aProtocolVersion);
@@ -802,7 +812,7 @@ NS_IMETHODIMP ObliviousHttpChannel::ExplicitSetUploadStream(
   if (mStreamListener) {
     return NS_ERROR_IN_PROGRESS;
   }
-  if (aMethod != "POST"_ns && aMethod != "PUT") {
+  if (aMethod != "POST"_ns && aMethod != "PUT" && aMethod != "DELETE") {
     return NS_ERROR_INVALID_ARG;
   }
   if (aStreamHasHeaders) {
@@ -870,4 +880,15 @@ NS_IMETHODIMP ObliviousHttpChannel::GetDocumentCharacterSet(
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP ObliviousHttpChannel::GetDecompressDictionary(
+    DictionaryCacheEntry** aDictionary) {
+  *aDictionary = nullptr;
+  return NS_OK;
+}
+
+NS_IMETHODIMP ObliviousHttpChannel::SetDecompressDictionary(
+    DictionaryCacheEntry* aDictionary) {
+  return NS_OK;
+}
+
 }  // namespace mozilla::net
