@@ -5,15 +5,19 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "Classifier.h"
+#include "HttpBaseChannel.h"
 #include "mozilla/Components.h"
 #include "mozilla/ErrorNames.h"
 #include "mozilla/net/AsyncUrlChannelClassifier.h"
+#include "mozilla/dom/BrowsingContext.h"
+#include "mozilla/dom/CanonicalBrowsingContext.h"
 #include "mozilla/net/UrlClassifierCommon.h"
 #include "mozilla/net/UrlClassifierFeatureFactory.h"
 #include "mozilla/net/UrlClassifierFeatureResult.h"
 #include "nsContentUtils.h"
 #include "nsIChannel.h"
 #include "nsIHttpChannel.h"
+#include "nsIUrlClassifierExceptionList.h"
 #include "nsNetCID.h"
 #include "nsNetUtil.h"
 #include "nsPrintfCString.h"
@@ -21,6 +25,7 @@
 #include "nsServiceManagerUtils.h"
 #include "nsUrlClassifierDBService.h"
 #include "nsUrlClassifierUtils.h"
+#include "mozilla/net/UrlClassifierCommon.h"
 
 namespace mozilla {
 namespace net {
@@ -287,7 +292,6 @@ class FeatureData {
 
   State mState{eUnclassified};
   nsCOMPtr<nsIUrlClassifierFeature> mFeature;
-  nsCOMPtr<nsIChannel> mChannel;
 
   nsTArray<RefPtr<TableData>> mBlocklistTables;
   nsTArray<RefPtr<TableData>> mEntitylistTables;
@@ -317,7 +321,6 @@ nsresult FeatureData::Initialize(FeatureTask* aTask, nsIChannel* aChannel,
   }
 
   mFeature = aFeature;
-  mChannel = aChannel;
 
   nsresult rv = InitializeList(
       aTask, aChannel, nsIUrlClassifierFeature::blocklist, mBlocklistTables);
@@ -453,26 +456,40 @@ bool FeatureData::MaybeCompleteClassification(nsIChannel* aChannel) {
   MOZ_ASSERT(mState == eMatchBlocklist);
 
   // Maybe we have to ignore this host
-  nsAutoCString exceptionList;
-  nsresult rv = mFeature->GetExceptionHostList(exceptionList);
+  nsCOMPtr<nsIUrlClassifierExceptionList> exceptionList;
+  nsresult rv = mFeature->GetExceptionList(getter_AddRefs(exceptionList));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     UC_LOG_WARN(
-        ("AsyncChannelClassifier::FeatureData::MayebeCompleteClassification - "
-         "error. Let's move on [this=%p channel=%p]",
-         this, aChannel));
+        ("AsyncChannelClassifier::FeatureData::MaybeCompleteClassification - "
+         "error while getting exception list. Let's move on [exceptionList=%p "
+         "this=%p channel=%p]",
+         exceptionList.get(), this, aChannel));
     return true;
   }
 
-  if (!mBlocklistTables.IsEmpty() &&
-      nsContentUtils::IsURIInList(mBlocklistTables[0]->URI(), exceptionList)) {
-    nsCString spec = mBlocklistTables[0]->URI()->GetSpecOrDefault();
-    spec.Truncate(std::min(spec.Length(), UrlClassifierCommon::sMaxSpecLength));
-    UC_LOG(
-        ("AsyncChannelClassifier::FeatureData::MaybeCompleteClassification - "
-         "uri %s found in "
-         "exceptionlist of feature %s [this=%p channel=%p]",
-         spec.get(), name.get(), this, aChannel));
-    return true;
+  // Check if current load is allow-listed by the exception list.
+  if (!mBlocklistTables.IsEmpty() && exceptionList) {
+    // Get top level URI from channel.
+    nsCOMPtr<nsIURI> topLevelURI;
+    rv = UrlClassifierCommon::GetTopWindowURI(aChannel,
+                                              getter_AddRefs(topLevelURI));
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Failed to get top level URI");
+
+    bool isPrivateBrowsing = NS_UsePrivateBrowsing(aChannel);
+    bool isAllowListed = false;
+    rv = exceptionList->Matches(mBlocklistTables[0]->URI(), topLevelURI,
+                                isPrivateBrowsing, &isAllowListed);
+    if (NS_SUCCEEDED(rv) && isAllowListed) {
+      nsCString spec = mBlocklistTables[0]->URI()->GetSpecOrDefault();
+      spec.Truncate(
+          std::min(spec.Length(), UrlClassifierCommon::sMaxSpecLength));
+      UC_LOG(
+          ("AsyncChannelClassifier::FeatureData::MaybeCompleteClassification - "
+           "uri %s found in "
+           "exceptionlist of feature %s [this=%p channel=%p]",
+           spec.get(), name.get(), this, aChannel));
+      return true;
+    }
   }
 
   nsTArray<nsCString> list;
@@ -895,19 +912,43 @@ nsresult AsyncUrlChannelClassifier::CheckChannel(
     return NS_ERROR_FAILURE;
   }
 
+  // raise the priority of URLClassifier's return dispatch to the MainThread if
+  // the channel is considered important
+  EventQueuePriority eventPriority = EventQueuePriority::Normal;
+  if (nsCOMPtr<HttpBaseChannel> baseChannel = do_QueryInterface(aChannel)) {
+    uint32_t classOfServiceFlags = 0;
+    baseChannel->GetClassFlags(&classOfServiceFlags);
+    if (classOfServiceFlags &
+        (nsIClassOfService::Leader | nsIClassOfService::UrgentStart |
+         nsIClassOfService::Unblocked)) {
+      eventPriority = EventQueuePriority::MediumHigh;
+    }
+  }
+  if (nsCOMPtr<nsISupportsPriority> supportsPriority =
+          do_QueryInterface(aChannel)) {
+    int32_t priority = nsISupportsPriority::PRIORITY_NORMAL;
+    supportsPriority->GetPriority(&priority);
+    // note that higher priorities have lower numeric values
+    if (priority <= nsISupportsPriority::PRIORITY_HIGH) {
+      eventPriority = EventQueuePriority::MediumHigh;
+    }
+  }
+
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
       "AsyncUrlChannelClassifier::CheckChannel",
-      [task, workerClassifier]() -> void {
+      [task, workerClassifier, eventPriority]() -> void {
         MOZ_ASSERT(!NS_IsMainThread());
         task->DoLookup(workerClassifier);
 
-        nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
-            "AsyncUrlChannelClassifier::CheckChannel - return",
-            [task]() -> void { task->CompleteClassification(); });
-
-        NS_DispatchToMainThread(r);
+        NS_DispatchToMainThreadQueue(
+            NS_NewRunnableFunction(
+                "AsyncUrlChannelClassifier::CheckChannel - return",
+                [task]() -> void { task->CompleteClassification(); }),
+            eventPriority);
       });
 
+  // no need to prioritize the dispatch to the URLClassifier thread
+  // since overriding prioritization is ignored if we aren't on the MainThread
   return nsUrlClassifierDBService::BackgroundThread()->Dispatch(
       r, NS_DISPATCH_NORMAL);
 }
