@@ -5,6 +5,7 @@
 #include "mozilla/StaticPrefs_extensions.h"
 #include "mozilla/StaticPrefs_geo.h"
 #include "mozilla/StaticPrefs_layout.h"
+#include "mozilla/StaticPrefs_mathml.h"
 #include "mozilla/StaticPrefs_media.h"
 #include "mozilla/StaticPrefs_network.h"
 #include "mozilla/StaticPrefs_privacy.h"
@@ -112,6 +113,8 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::dom_select_showPicker_enabled,
   // dom.serviceWorkers.navigationPreload.enabled
   StaticPrefs::dom_serviceWorkers_navigationPreload_enabled,
+  // dom.shadowdom.referenceTarget.enabled
+  StaticPrefs::dom_shadowdom_referenceTarget_enabled,
   // dom.shadowdom.selection_across_boundary_enabled
   StaticPrefs::dom_shadowdom_selection_across_boundary_enabled,
   // dom.storage.testing
@@ -186,8 +189,6 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::layout_css_moz_control_character_visibility_enabled,
   // layout.css.osx-font-smoothing.enabled
   StaticPrefs::layout_css_osx_font_smoothing_enabled,
-  // layout.css.overflow-clip-box.enabled
-  StaticPrefs::layout_css_overflow_clip_box_enabled,
   // layout.css.prefixes.animations
   StaticPrefs::layout_css_prefixes_animations,
   // layout.css.prefixes.border-image
@@ -214,6 +215,8 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::layout_css_typed_om_enabled,
   // layout.css.zoom.enabled
   StaticPrefs::layout_css_zoom_enabled,
+  // mathml.math_shift.enabled
+  StaticPrefs::mathml_math_shift_enabled,
   // media.allowed-to-play.enabled
   StaticPrefs::media_allowed_to_play_enabled,
   // media.eme.hdcp-policy-check.enabled
@@ -242,6 +245,8 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::media_webspeech_synth_enabled,
   // network.fetchpriority.enabled
   StaticPrefs::network_fetchpriority_enabled,
+  // privacy.fingerprintingProtection.testing
+  StaticPrefs::privacy_fingerprintingProtection_testing,
   // privacy.globalprivacycontrol.functionality.enabled
   StaticPrefs::privacy_globalprivacycontrol_functionality_enabled,
   // security.webauth.webauthn
