@@ -10,7 +10,6 @@ use euclid::num::Zero;
 use std::cmp::{max, min};
 use std::fmt::{self, Debug, Error, Formatter};
 use std::ops::{Add, Sub};
-use unicode_bidi as bidi;
 
 pub enum BlockFlowDirection {
     TopToBottom,
@@ -41,6 +40,7 @@ pub enum InlineBaseDirection {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(u8)]
 pub enum WritingModeProperty {
@@ -307,6 +307,25 @@ impl WritingMode {
         }
     }
 
+    /// Given a physical side, flips the start on that axis, and returns the corresponding
+    /// physical side.
+    #[inline]
+    pub fn flipped_start_side(&self, side: PhysicalSide) -> PhysicalSide {
+        let bs = self.block_start_physical_side();
+        if side == bs {
+            return self.inline_start_physical_side();
+        }
+        let be = self.block_end_physical_side();
+        if side == be {
+            return self.inline_end_physical_side();
+        }
+        if side == self.inline_start_physical_side() {
+            return bs;
+        }
+        debug_assert_eq!(side, self.inline_end_physical_side());
+        be
+    }
+
     #[inline]
     pub fn start_start_physical_corner(&self) -> PhysicalCorner {
         PhysicalCorner::from_sides(
@@ -357,18 +376,6 @@ impl WritingMode {
         }
     }
 
-    #[inline]
-    /// The default bidirectional embedding level for this writing mode.
-    ///
-    /// Returns bidi level 0 if the mode is LTR, or 1 otherwise.
-    pub fn to_bidi_level(&self) -> bidi::Level {
-        if self.is_bidi_ltr() {
-            bidi::Level::ltr()
-        } else {
-            bidi::Level::rtl()
-        }
-    }
-
     #[inline]
     /// Is the text layout vertical?
     pub fn is_text_vertical(&self) -> bool {
@@ -1135,10 +1142,10 @@ impl<T: Clone> LogicalMargin<T> {
 impl<T: PartialEq + Zero> LogicalMargin<T> {
     #[inline]
     pub fn is_zero(&self) -> bool {
-        self.block_start == Zero::zero() &&
-            self.inline_end == Zero::zero() &&
-            self.block_end == Zero::zero() &&
-            self.inline_start == Zero::zero()
+        self.block_start == Zero::zero()
+            && self.inline_end == Zero::zero()
+            && self.block_end == Zero::zero()
+            && self.inline_start == Zero::zero()
     }
 }
 
@@ -1399,8 +1406,8 @@ impl<T: Copy + Add<T, Output = T> + Sub<T, Output = T>> LogicalRect<T> {
 
     pub fn translate(&self, offset: &LogicalPoint<T>) -> LogicalRect<T> {
         LogicalRect {
-            start: self.start +
-                LogicalSize {
+            start: self.start
+                + LogicalSize {
                     inline: offset.i,
                     block: offset.b,
                     debug_writing_mode: offset.debug_writing_mode,
@@ -1568,10 +1575,10 @@ impl LogicalSide {
         ];
 
         debug_assert!(
-            WritingMode::VERTICAL.bits() == 0x01 &&
-                WritingMode::INLINE_REVERSED.bits() == 0x02 &&
-                WritingMode::VERTICAL_LR.bits() == 0x04 &&
-                WritingMode::LINE_INVERTED.bits() == 0x08
+            WritingMode::VERTICAL.bits() == 0x01
+                && WritingMode::INLINE_REVERSED.bits() == 0x02
+                && WritingMode::VERTICAL_LR.bits() == 0x04
+                && WritingMode::LINE_INVERTED.bits() == 0x08
         );
         let index = (wm.bits() & 0xF) as usize;
         INLINE_MAPPING[index][edge]
