@@ -29,8 +29,9 @@ use crate::stylesheets::{layer_rule::LayerOrder, Origin};
 use crate::stylist::Stylist;
 #[cfg(feature = "gecko")]
 use crate::values::specified::length::FontBaseSize;
+use crate::values::specified::position::PositionTryFallbacksTryTactic;
 use crate::values::{computed, specified};
-use fxhash::FxHashMap;
+use rustc_hash::FxHashMap;
 use servo_arc::Arc;
 use smallvec::SmallVec;
 use std::borrow::Cow;
@@ -70,6 +71,7 @@ pub fn cascade<E>(
     parent_style: Option<&ComputedValues>,
     layout_parent_style: Option<&ComputedValues>,
     first_line_reparenting: FirstLineReparenting,
+    try_tactic: PositionTryFallbacksTryTactic,
     visited_rules: Option<&StrongRuleNode>,
     cascade_input_flags: ComputedValueFlags,
     rule_cache: Option<&RuleCache>,
@@ -87,6 +89,7 @@ where
         parent_style,
         layout_parent_style,
         first_line_reparenting,
+        try_tactic,
         CascadeMode::Unvisited { visited_rules },
         cascade_input_flags,
         rule_cache,
@@ -186,6 +189,7 @@ fn cascade_rules<E>(
     parent_style: Option<&ComputedValues>,
     layout_parent_style: Option<&ComputedValues>,
     first_line_reparenting: FirstLineReparenting,
+    try_tactic: PositionTryFallbacksTryTactic,
     cascade_mode: CascadeMode,
     cascade_input_flags: ComputedValueFlags,
     rule_cache: Option<&RuleCache>,
@@ -204,6 +208,7 @@ where
         parent_style,
         layout_parent_style,
         first_line_reparenting,
+        try_tactic,
         cascade_mode,
         cascade_input_flags,
         rule_cache,
@@ -260,6 +265,7 @@ pub fn apply_declarations<'a, E, I>(
     parent_style: Option<&'a ComputedValues>,
     layout_parent_style: Option<&ComputedValues>,
     first_line_reparenting: FirstLineReparenting<'a>,
+    try_tactic: PositionTryFallbacksTryTactic,
     cascade_mode: CascadeMode,
     cascade_input_flags: ComputedValueFlags,
     rule_cache: Option<&'a RuleCache>,
@@ -299,7 +305,7 @@ where
 
     let using_cached_reset_properties;
     let ignore_colors = context.builder.device.forced_colors().is_active();
-    let mut cascade = Cascade::new(first_line_reparenting, ignore_colors);
+    let mut cascade = Cascade::new(first_line_reparenting, try_tactic, ignore_colors);
     let mut declarations = Default::default();
     let mut shorthand_cache = ShorthandsWithPropertyReferencesCache::default();
     let properties_to_apply = match cascade_mode {
@@ -437,8 +443,8 @@ fn tweak_when_ignoring_colors(
     if context
         .builder
         .pseudo
-        .map_or(false, |p| p.is_color_swatch()) &&
-        longhand_id == LonghandId::BackgroundColor
+        .map_or(false, |p| p.is_color_swatch())
+        && longhand_id == LonghandId::BackgroundColor
     {
         return;
     }
@@ -495,8 +501,8 @@ fn tweak_when_ignoring_colors(
                 .builder
                 .get_parent_inherited_text()
                 .clone_color()
-                .alpha ==
-                0.0
+                .alpha
+                == 0.0
             {
                 let color = context.builder.device.default_color();
                 declarations_to_apply_unless_overridden.push(PropertyDeclaration::Color(
@@ -626,6 +632,7 @@ impl<'a> Declarations<'a> {
 
 struct Cascade<'b> {
     first_line_reparenting: FirstLineReparenting<'b>,
+    try_tactic: PositionTryFallbacksTryTactic,
     ignore_colors: bool,
     seen: LonghandIdSet,
     author_specified: LonghandIdSet,
@@ -635,9 +642,10 @@ struct Cascade<'b> {
 }
 
 impl<'b> Cascade<'b> {
-    fn new(first_line_reparenting: FirstLineReparenting<'b>, ignore_colors: bool) -> Self {
+    fn new(first_line_reparenting: FirstLineReparenting<'b>, try_tactic: PositionTryFallbacksTryTactic, ignore_colors: bool) -> Self {
         Self {
             first_line_reparenting,
+            try_tactic,
             ignore_colors,
             seen: LonghandIdSet::default(),
             author_specified: LonghandIdSet::default(),
@@ -770,15 +778,15 @@ impl<'b> Cascade<'b> {
         }
 
         if apply!(Zoom) {
-            context.builder.effective_zoom = context
-                .builder
-                .inherited_effective_zoom()
-                .compute_effective(context.builder.specified_zoom());
-            // NOTE(emilio): This is a bit of a hack, but matches the shipped WebKit and Blink
-            // behavior for now. Ideally, in the future, we have a pass over all
-            // implicitly-or-explicitly-inherited properties that can contain lengths and
-            // re-compute them properly, see https://github.com/w3c/csswg-drafts/issues/9397.
-            self.recompute_font_size_for_zoom_change(&mut context.builder);
+            context.builder.recompute_effective_zooms();
+            if !context.builder.effective_zoom_for_inheritance.is_one() {
+                // NOTE(emilio): This is a bit of a hack, but matches the shipped WebKit and Blink
+                // behavior for now. Ideally, in the future, we have a pass over all
+                // implicitly-or-explicitly-inherited properties that can contain lengths and
+                // re-compute them properly, see https://github.com/w3c/csswg-drafts/issues/9397.
+                // TODO(emilio): we need to eagerly do this for line-height as well, probably.
+                self.recompute_font_size_for_zoom_change(&mut context.builder);
+            }
         }
 
         // Compute font-family.
@@ -861,6 +869,9 @@ impl<'b> Cascade<'b> {
                     .set_writing_mode_dependency(wm);
                 longhand_id = longhand_id.to_physical(wm);
             }
+            if !self.try_tactic.is_empty() {
+                longhand_id = self.try_tactic.apply_to_property(longhand_id, context.builder.writing_mode);
+            }
             self.apply_one_longhand(
                 context,
                 longhand_id,
@@ -881,6 +892,24 @@ impl<'b> Cascade<'b> {
                 }
             }
         }
+
+        if !context.builder.effective_zoom_for_inheritance.is_one() {
+            self.recompute_zoom_dependent_inherited_lengths(context);
+        }
+    }
+
+    #[cold]
+    fn recompute_zoom_dependent_inherited_lengths(&self, context: &mut computed::Context) {
+        debug_assert!(self.seen.contains(LonghandId::Zoom));
+        for prop in LonghandIdSet::zoom_dependent_inherited_properties().iter() {
+            if self.seen.contains(prop) {
+                continue;
+            }
+            let declaration = PropertyDeclaration::css_wide_keyword(prop, CSSWideKeyword::Inherit);
+            unsafe {
+                self.do_apply_declaration(context, prop, &declaration);
+            }
+        }
     }
 
     fn apply_one_longhand(
@@ -918,20 +947,29 @@ impl<'b> Cascade<'b> {
                 &mut self.declarations_to_apply_unless_overridden,
             );
         }
-
-        let is_unset = match declaration.get_css_wide_keyword() {
-            Some(keyword) => match keyword {
-                CSSWideKeyword::RevertLayer | CSSWideKeyword::Revert => {
+        let can_skip_apply = match declaration.get_css_wide_keyword() {
+            Some(keyword) => {
+                if matches!(
+                    keyword,
+                    CSSWideKeyword::RevertLayer | CSSWideKeyword::Revert
+                ) {
                     let origin_revert = keyword == CSSWideKeyword::Revert;
                     // We intentionally don't want to insert it into `self.seen`, `reverted` takes
                     // care of rejecting other declarations as needed.
                     self.reverted_set.insert(longhand_id);
                     self.reverted.insert(longhand_id, (priority, origin_revert));
                     return;
-                },
-                CSSWideKeyword::Unset => true,
-                CSSWideKeyword::Inherit => longhand_id.inherited(),
-                CSSWideKeyword::Initial => !longhand_id.inherited(),
+                }
+
+                let inherited = longhand_id.inherited();
+                let zoomed = !context.builder.effective_zoom_for_inheritance.is_one()
+                    && longhand_id.zoom_dependent();
+                match keyword {
+                    CSSWideKeyword::Revert | CSSWideKeyword::RevertLayer => unreachable!(),
+                    CSSWideKeyword::Unset => !zoomed || !inherited,
+                    CSSWideKeyword::Inherit => inherited && !zoomed,
+                    CSSWideKeyword::Initial => !inherited,
+                }
             },
             None => false,
         };
@@ -941,11 +979,9 @@ impl<'b> Cascade<'b> {
             self.author_specified.insert(longhand_id);
         }
 
-        if is_unset {
-            return;
+        if !can_skip_apply {
+            unsafe { self.do_apply_declaration(context, longhand_id, &declaration) }
         }
-
-        unsafe { self.do_apply_declaration(context, longhand_id, &declaration) }
     }
 
     #[inline]
@@ -997,6 +1033,7 @@ impl<'b> Cascade<'b> {
             visited_parent!(parent_style),
             visited_parent!(layout_parent_style),
             self.first_line_reparenting,
+            self.try_tactic,
             CascadeMode::Visited {
                 unvisited_context: &*context,
             },
@@ -1037,12 +1074,6 @@ impl<'b> Cascade<'b> {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_FONT_FAMILY);
         }
 
-        if self.author_specified.contains_any(LonghandIdSet::margin_properties()) &&
-           self.author_specified.contains(LonghandId::FontSize)
-        {
-            builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_MARGIN_AND_FONT_SIZE);
-        }
-
         if self.author_specified.contains(LonghandId::Color) {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_TEXT_COLOR);
         }
@@ -1062,6 +1093,7 @@ impl<'b> Cascade<'b> {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_FONT_SYNTHESIS_WEIGHT);
         }
 
+        #[cfg(feature = "gecko")]
         if self
             .author_specified
             .contains(LonghandId::FontSynthesisStyle)
@@ -1107,11 +1139,11 @@ impl<'b> Cascade<'b> {
         // style specified viewport units / used font-relative lengths, this one
         // would as well.  It matches the same rules, so it is the right thing
         // to do anyways, even if it's only used on inherited properties.
-        let bits_to_copy = ComputedValueFlags::HAS_AUTHOR_SPECIFIED_BORDER_BACKGROUND |
-            ComputedValueFlags::DEPENDS_ON_SELF_FONT_METRICS |
-            ComputedValueFlags::DEPENDS_ON_INHERITED_FONT_METRICS |
-            ComputedValueFlags::USES_CONTAINER_UNITS |
-            ComputedValueFlags::USES_VIEWPORT_UNITS;
+        let bits_to_copy = ComputedValueFlags::HAS_AUTHOR_SPECIFIED_BORDER_BACKGROUND
+            | ComputedValueFlags::DEPENDS_ON_SELF_FONT_METRICS
+            | ComputedValueFlags::DEPENDS_ON_INHERITED_FONT_METRICS
+            | ComputedValueFlags::USES_CONTAINER_UNITS
+            | ComputedValueFlags::USES_VIEWPORT_UNITS;
         builder.add_flags(style.flags & bits_to_copy);
 
         true
@@ -1164,8 +1196,8 @@ impl<'b> Cascade<'b> {
 
         // Check the use_document_fonts setting for content, but for chrome
         // documents they're treated as always enabled.
-        if static_prefs::pref!("browser.display.use_document_fonts") != 0 ||
-            builder.device.chrome_rules_enabled_for_document()
+        if static_prefs::pref!("browser.display.use_document_fonts") != 0
+            || builder.device.chrome_rules_enabled_for_document()
         {
             return;
         }
@@ -1197,7 +1229,6 @@ impl<'b> Cascade<'b> {
     }
 
     /// Some keyword sizes depend on the font family and language.
-    #[cfg(feature = "gecko")]
     fn recompute_keyword_font_size_if_needed(&self, context: &mut computed::Context) {
         use crate::values::computed::ToComputedValue;
 
@@ -1216,6 +1247,7 @@ impl<'b> Cascade<'b> {
                 },
             };
 
+            #[cfg(feature = "gecko")]
             if font.mScriptUnconstrainedSize == new_size.computed_size {
                 return;
             }
@@ -1279,7 +1311,7 @@ impl<'b> Cascade<'b> {
         // NOTE(emilio): Intentionally not using the effective zoom here, since all the inherited
         // zooms are already applied.
         let old_size = builder.get_font().clone_font_size();
-        let new_size = old_size.zoom(builder.resolved_specified_zoom());
+        let new_size = old_size.zoom(builder.effective_zoom_for_inheritance);
         if old_size == new_size {
             return;
         }
@@ -1295,8 +1327,8 @@ impl<'b> Cascade<'b> {
         use crate::values::generics::NonNegative;
 
         // Do not do anything if font-size: math or math-depth is not set.
-        if context.builder.get_font().clone_font_size().keyword_info.kw !=
-            specified::FontSizeKeyword::Math
+        if context.builder.get_font().clone_font_size().keyword_info.kw
+            != specified::FontSizeKeyword::Math
         {
             return;
         }
@@ -1353,6 +1385,8 @@ impl<'b> Cascade<'b> {
         }
 
         let (new_size, new_unconstrained_size) = {
+            use crate::values::specified::font::QueryFontMetricsFlags;
+
             let builder = &context.builder;
             let font = builder.get_font();
             let parent_font = builder.get_parent_font();
@@ -1374,7 +1408,7 @@ impl<'b> Cascade<'b> {
                 let font_metrics = context.query_font_metrics(
                     FontBaseSize::InheritedStyle,
                     FontMetricsOrientation::Horizontal,
-                    /* retrieve_math_scales = */ true,
+                    QueryFontMetricsFlags::NEEDS_MATH_SCALES,
                 );
                 scale_factor_for_math_depth_change(
                     parent_font.mMathDepth as i32,
