@@ -52,7 +52,9 @@ impl Parse for Spacing {
 }
 
 /// A specified value for the `letter-spacing` property.
-#[derive(Clone, Debug, MallocSizeOf, Parse, PartialEq, SpecifiedValueInfo, ToCss, ToShmem)]
+#[derive(
+    Clone, Debug, MallocSizeOf, Parse, PartialEq, SpecifiedValueInfo, ToCss, ToShmem, ToTyped,
+)]
 pub struct LetterSpacing(pub Spacing);
 
 impl ToComputedValue for LetterSpacing {
@@ -77,7 +79,9 @@ impl ToComputedValue for LetterSpacing {
 }
 
 /// A specified value for the `word-spacing` property.
-#[derive(Clone, Debug, MallocSizeOf, Parse, PartialEq, SpecifiedValueInfo, ToCss, ToShmem)]
+#[derive(
+    Clone, Debug, MallocSizeOf, Parse, PartialEq, SpecifiedValueInfo, ToCss, ToShmem, ToTyped,
+)]
 pub struct WordSpacing(pub Spacing);
 
 impl ToComputedValue for WordSpacing {
@@ -109,6 +113,7 @@ impl ToComputedValue for WordSpacing {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C, u8)]
 pub enum HyphenateCharacter {
@@ -196,6 +201,7 @@ pub enum TextOverflowSide {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C)]
 /// text-overflow.
@@ -281,6 +287,7 @@ impl ToCss for TextOverflow {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[cfg_attr(
     feature = "gecko",
@@ -381,6 +388,7 @@ pub enum TextTransformCase {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[cfg_attr(
     feature = "gecko",
@@ -445,6 +453,18 @@ impl TextTransform {
         // Case bits are exclusive with each other.
         case.is_empty() || case.bits().is_power_of_two()
     }
+
+    /// Returns the corresponding TextTransformCase.
+    pub fn case(&self) -> TextTransformCase {
+        match *self & Self::CASE_TRANSFORMS {
+            Self::NONE => TextTransformCase::None,
+            Self::UPPERCASE => TextTransformCase::Uppercase,
+            Self::LOWERCASE => TextTransformCase::Lowercase,
+            Self::CAPITALIZE => TextTransformCase::Capitalize,
+            Self::MATH_AUTO => TextTransformCase::MathAuto,
+            _ => unreachable!("Case bits are exclusive with each other"),
+        }
+    }
 }
 
 /// Specified and computed value of text-align-last.
@@ -463,6 +483,7 @@ impl TextTransform {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 #[repr(u8)]
@@ -492,6 +513,7 @@ pub enum TextAlignLast {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 #[repr(u8)]
@@ -512,7 +534,18 @@ pub enum TextAlignKeyword {
 
 /// Specified value of text-align property.
 #[derive(
-    Clone, Copy, Debug, Eq, Hash, MallocSizeOf, Parse, PartialEq, SpecifiedValueInfo, ToCss, ToShmem,
+    Clone,
+    Copy,
+    Debug,
+    Eq,
+    Hash,
+    MallocSizeOf,
+    Parse,
+    PartialEq,
+    SpecifiedValueInfo,
+    ToCss,
+    ToShmem,
+    ToTyped,
 )]
 pub enum TextAlign {
     /// Keyword value of text-align property.
@@ -598,7 +631,7 @@ fn fill_mode_is_default_and_shape_exists(
 /// Specified value of text-emphasis-style property.
 ///
 /// https://drafts.csswg.org/css-text-decor/#propdef-text-emphasis-style
-#[derive(Clone, Debug, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToCss, ToShmem)]
+#[derive(Clone, Debug, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToCss, ToShmem, ToTyped)]
 #[allow(missing_docs)]
 pub enum TextEmphasisStyle {
     /// [ <fill> || <shape> ]
@@ -685,8 +718,8 @@ impl ToComputedValue for TextEmphasisStyle {
                     //
                     // Also should probably use WritingMode::is_vertical rather
                     // than the computed value of the `writing-mode` property.
-                    if context.style().get_inherited_box().clone_writing_mode() ==
-                        SpecifiedWritingMode::HorizontalTb
+                    if context.style().get_inherited_box().clone_writing_mode()
+                        == SpecifiedWritingMode::HorizontalTb
                     {
                         TextEmphasisShapeKeyword::Circle
                     } else {
@@ -778,6 +811,7 @@ impl Parse for TextEmphasisStyle {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C)]
 #[css(bitflags(
@@ -835,6 +869,7 @@ impl TextEmphasisPosition {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum WordBreak {
@@ -864,6 +899,7 @@ pub enum WordBreak {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum TextJustify {
@@ -891,6 +927,7 @@ pub enum TextJustify {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum MozControlCharacterVisibility {
@@ -924,6 +961,7 @@ impl Default for MozControlCharacterVisibility {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum LineBreak {
@@ -949,6 +987,7 @@ pub enum LineBreak {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum OverflowWrap {
@@ -1027,6 +1066,7 @@ impl Parse for TextIndent {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum TextDecorationSkipInk {
@@ -1077,7 +1117,7 @@ impl Parse for TextDecorationTrim {
         if let Ok(start) = input.try_parse(|i| Length::parse(ctx, i)) {
             let end = input.try_parse(|i| Length::parse(ctx, i));
             let end = end.unwrap_or_else(|_| start.clone());
-            return Ok(TextDecorationTrim::Length{start, end});
+            return Ok(TextDecorationTrim::Length { start, end });
         }
         input.expect_ident_matching("auto")?;
         Ok(TextDecorationTrim::Auto)
@@ -1096,6 +1136,7 @@ impl Parse for TextDecorationTrim {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[css(bitflags(
     single = "auto",
@@ -1172,7 +1213,16 @@ impl ToCss for TextUnderlinePosition {
 /// Values for `ruby-position` property
 #[repr(u8)]
 #[derive(
-    Clone, Copy, Debug, Eq, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem,
+    Clone,
+    Copy,
+    Debug,
+    Eq,
+    MallocSizeOf,
+    PartialEq,
+    ToComputedValue,
+    ToResolvedValue,
+    ToShmem,
+    ToTyped,
 )]
 #[allow(missing_docs)]
 pub enum RubyPosition {
@@ -1200,8 +1250,8 @@ impl Parse for RubyPosition {
             "under" => false,
         };
         // Parse alternate after
-        let alternate = alternate ||
-            input
+        let alternate = alternate
+            || input
                 .try_parse(|i| i.expect_ident_matching("alternate"))
                 .is_ok();
 
@@ -1259,11 +1309,13 @@ impl SpecifiedValueInfo for RubyPosition {
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[css(bitflags(
     single = "normal,auto,no-autospace",
     // Bug 1980111: add 'replace' to 'mixed' in the future so that it parses correctly.
-    mixed = "ideograph-alpha,ideograph-numeric,punctuation,insert",
+    // Bug 1986500: add 'punctuation' to 'mixed' in the future so that it parses correctly.
+    mixed = "ideograph-alpha,ideograph-numeric,insert",
     // Bug 1980111: Uncomment 'validate_mixed' to support 'replace' value.
     // validate_mixed = "Self::validate_mixed_flags",
 ))]
@@ -1271,14 +1323,14 @@ impl SpecifiedValueInfo for RubyPosition {
 pub struct TextAutospace(u8);
 bitflags! {
     impl TextAutospace: u8 {
-        /// Same behavior as ideograph-alpha ideograph-numeric.
-        const NORMAL = 0;
+        /// No automatic space is inserted.
+        const NO_AUTOSPACE = 0;
 
         /// The user agent chooses a set of typographically high quality spacing values.
         const AUTO = 1 << 0;
 
-        /// No automatic space is inserted.
-        const NO_AUTOSPACE = 1 << 1;
+        /// Same behavior as ideograph-alpha ideograph-numeric.
+        const NORMAL = 1 << 1;
 
         /// 1/8ic space between ideographic characters and non-ideographic letters.
         const IDEOGRAPH_ALPHA = 1 << 2;
@@ -1286,8 +1338,10 @@ bitflags! {
         /// 1/8ic space between ideographic characters and non-ideographic decimal numerals.
         const IDEOGRAPH_NUMERIC = 1 << 3;
 
+        /* Bug 1986500: Uncomment the following to support the 'punctuation' value.
         /// Apply special spacing between letters and punctuation (French).
         const PUNCTUATION = 1 << 4;
+        */
 
         /// Auto-spacing is only inserted if no space character is present in the text.
         const INSERT = 1 << 5;
