@@ -7,6 +7,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/ErrorResult.h"
 #include "mozilla/ScopeExit.h"
+#include "mozilla/Services.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/UniquePtrExtensions.h"
 #include "mozilla/WidgetUtils.h"
@@ -45,6 +46,7 @@
 #include "nsXULAppAPI.h"
 #include "nsThreadUtils.h"
 
+#include "nsIObserverService.h"
 #include "nsIRunnable.h"
 #include "nsXREDirProvider.h"
 #include "nsAppRunner.h"
@@ -362,13 +364,6 @@ nsToolkitProfile::SetStoreID(const nsACString& aStoreID) {
     rv = nsToolkitProfileService::gService->mProfileDB.SetString(
         mSection.get(), "ShowSelector", mShowProfileSelector ? "1" : "0");
     NS_ENSURE_SUCCESS(rv, rv);
-
-    if (nsToolkitProfileService::gService->mCurrent == this) {
-      rv = prefs->SetCharPref(STORE_ID_PREF, aStoreID);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      nsToolkitProfileService::gService->mGroupProfile = this;
-    }
   } else {
     // If the string was not present in the ini file, just ignore the error.
     nsToolkitProfileService::gService->mProfileDB.DeleteString(mSection.get(),
@@ -381,13 +376,6 @@ nsToolkitProfile::SetStoreID(const nsACString& aStoreID) {
     // If the string was not present in the ini file, just ignore the error.
     nsToolkitProfileService::gService->mProfileDB.DeleteString(mSection.get(),
                                                                "ShowSelector");
-
-    if (nsToolkitProfileService::gService->mCurrent == this) {
-      rv = prefs->ClearUserPref(STORE_ID_PREF);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      nsToolkitProfileService::gService->mGroupProfile = nullptr;
-    }
   }
   mStoreID = aStoreID;
 
@@ -469,6 +457,10 @@ nsToolkitProfile::SetShowProfileSelector(bool aShowProfileSelector) {
   NS_ENSURE_SUCCESS(rv, rv);
 
   mShowProfileSelector = aShowProfileSelector;
+#  ifdef XP_WIN
+  nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+  obs->NotifyObservers(nullptr, "profile-show-selector-changed", nullptr);
+#  endif
   return NS_OK;
 #else
   return NS_ERROR_FAILURE;
@@ -674,6 +666,24 @@ nsToolkitProfileService::~nsToolkitProfileService() {
   mProfiles.clear();
 }
 
+void nsToolkitProfileService::UpdateCurrentProfile() {
+  nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  NS_ENSURE_TRUE_VOID(prefs);
+
+  nsCString storeID;
+  nsresult rv = prefs->GetCharPref(STORE_ID_PREF, storeID);
+  bool hasStoreIdPref = NS_SUCCEEDED(rv) && !storeID.IsEmpty();
+
+  if (!mCurrent && hasStoreIdPref) {
+    mCurrent = GetProfileByStoreID(storeID);
+    return;
+  }
+
+  if (mCurrent && !hasStoreIdPref && !mCurrent->mStoreID.IsVoid()) {
+    prefs->SetCharPref(STORE_ID_PREF, mCurrent->mStoreID);
+  }
+}
+
 void nsToolkitProfileService::CompleteStartup() {
   if (!mStartupProfileSelected) {
     return;
@@ -683,62 +693,24 @@ void nsToolkitProfileService::CompleteStartup() {
   glean::startup::profile_database_version.Set(mStartupFileVersion);
   glean::startup::profile_count.Set(static_cast<uint32_t>(mProfiles.length()));
 
-  nsresult rv;
-  bool needsFlush = false;
-
-  // If we started into an unmanaged profile in a profile group, set the group
-  // profile to be the managed profile belonging to the group.
-  nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (!mCurrent) {
-    nsCString storeID;
-    rv = prefs->GetCharPref(STORE_ID_PREF, storeID);
-    if (NS_SUCCEEDED(rv) && !storeID.IsEmpty()) {
-      mGroupProfile = GetProfileByStoreID(storeID);
-    }
-  } else {
-    // Otherwise, if the current profile has a storeID, then it must be the
-    // profile for some group.
-    if (!mCurrent->mStoreID.IsVoid()) {
-      mGroupProfile = mCurrent;
-      rv = prefs->SetCharPref(STORE_ID_PREF, mCurrent->mStoreID);
-      NS_ENSURE_SUCCESS_VOID(rv);
-    } else {
-      // Otherwise if the current profile has a store ID set in prefs but not in
-      // the database then restore it. This can happen if a version of Firefox
-      // prior to 67 has overwritten the database.
-      nsCString storeID;
-      rv = prefs->GetCharPref(STORE_ID_PREF, storeID);
-      if (NS_SUCCEEDED(rv) && !storeID.IsEmpty()) {
-        rv = mCurrent->SetStoreID(storeID);
-        if (NS_SUCCEEDED(rv)) {
-          needsFlush = true;
-        }
-      }
-    }
-  }
-
   if (mMaybeLockProfile) {
     nsCOMPtr<nsIToolkitShellService> shell =
         do_GetService(NS_TOOLKITSHELLSERVICE_CONTRACTID);
     if (shell) {
       bool isDefaultApp;
-      rv = shell->IsDefaultApplication(&isDefaultApp);
+      nsresult rv = shell->IsDefaultApplication(&isDefaultApp);
       if (NS_SUCCEEDED(rv) && isDefaultApp) {
         mProfileDB.SetString(mInstallSection.get(), "Locked", "1");
 
-        needsFlush = true;
+        // There is a very small chance that this could fail if something else
+        // overwrote the profiles database since we started up, probably less
+        // than a second ago. There isn't really a sane response here, all the
+        // other profile changes are already flushed so whether we fail to flush
+        // here or force quit the app makes no difference.
+        NS_ENSURE_SUCCESS_VOID(Flush());
       }
     }
   }
-
-  if (needsFlush) {
-    // There is a very small chance that this could fail if something else
-    // overwrote the profiles database since we started up, probably less than
-    // a second ago. There isn't really a sane response here, all the other
-    // profile changes are already flushed so whether we fail to flush here or
-    // force quit the app makes no difference.
-    NS_ENSURE_SUCCESS_VOID(Flush());
-  }
 }
 
 // Tests whether the passed profile was last used by this install.
@@ -1265,7 +1237,7 @@ nsToolkitProfileService::GetProfiles(nsISimpleEnumerator** aResult) {
 
 NS_IMETHODIMP
 nsToolkitProfileService::ProfileEnumerator::HasMoreElements(bool* aResult) {
-  *aResult = mCurrent ? true : false;
+  *aResult = static_cast<bool>(mCurrent);
   return NS_OK;
 }
 
@@ -1285,12 +1257,6 @@ nsToolkitProfileService::GetCurrentProfile(nsIToolkitProfile** aResult) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsToolkitProfileService::GetGroupProfile(nsIToolkitProfile** aResult) {
-  NS_IF_ADDREF(*aResult = mGroupProfile);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsToolkitProfileService::GetDefaultProfile(nsIToolkitProfile** aResult) {
   RefPtr<nsToolkitProfile> profile = GetDefaultProfile();
@@ -1461,6 +1427,7 @@ nsToolkitProfileService::SelectStartupProfile(
   // Since we were called outside of the normal startup path complete any
   // startup tasks.
   if (NS_SUCCEEDED(rv)) {
+    UpdateCurrentProfile();
     CompleteStartup();
   }
 
@@ -2340,10 +2307,10 @@ nsToolkitProfileService::GetProfileCount(uint32_t* aResult) {
 }
 
 // Attempts to merge the given profile data into the on-disk versions which may
-// have changed since rthey were loaded.
+// have changed since they were loaded.
 nsresult WriteProfileInfo(nsIFile* profilesDBFile, nsIFile* installDBFile,
                           const nsCString& installSection,
-                          const GroupProfileData* profileInfo) {
+                          const CurrentProfileData* profileInfo) {
   nsINIParser profilesIni;
   nsresult rv = profilesIni.Init(profilesDBFile);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2450,12 +2417,12 @@ nsISerialEventTarget* nsToolkitProfileService::AsyncQueue() {
 }
 
 NS_IMETHODIMP
-nsToolkitProfileService::AsyncFlushGroupProfile(JSContext* aCx,
-                                                dom::Promise** aPromise) {
+nsToolkitProfileService::AsyncFlushCurrentProfile(JSContext* aCx,
+                                                  dom::Promise** aPromise) {
 #ifndef MOZ_HAS_REMOTE
   return NS_ERROR_FAILURE;
 #else
-  if (!mGroupProfile) {
+  if (!mCurrent) {
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
@@ -2472,12 +2439,12 @@ nsToolkitProfileService::AsyncFlushGroupProfile(JSContext* aCx,
     return result.StealNSResult();
   }
 
-  UniquePtr<GroupProfileData> profileData = MakeUnique<GroupProfileData>();
-  profileData->mStoreID = mGroupProfile->mStoreID;
-  profileData->mShowSelector = mGroupProfile->mShowProfileSelector;
+  UniquePtr<CurrentProfileData> profileData = MakeUnique<CurrentProfileData>();
+  profileData->mStoreID = mCurrent->mStoreID;
+  profileData->mShowSelector = mCurrent->mShowProfileSelector;
 
   bool isRelative;
-  GetProfileDescriptor(mGroupProfile, profileData->mPath, &isRelative);
+  GetProfileDescriptor(mCurrent, profileData->mPath, &isRelative);
 
   nsCOMPtr<nsIRemoteService> rs = GetRemoteService();
   RefPtr<nsRemoteService> remoteService =
@@ -2512,7 +2479,7 @@ nsToolkitProfileService::AsyncFlushGroupProfile(JSContext* aCx,
   // This keeps the promise alive after this method returns.
   nsMainThreadPtrHandle<dom::Promise> promiseHolder(
       new nsMainThreadPtrHolder<dom::Promise>(
-          "nsToolkitProfileService::AsyncFlushGroupProfile", promise));
+          "nsToolkitProfileService::AsyncFlushCurrentProfile", promise));
 
   p->Then(GetCurrentSerialEventTarget(), __func__,
           [requestHolder, promiseHolder](
@@ -2586,7 +2553,7 @@ nsToolkitProfileService::AsyncFlush(JSContext* aCx, dom::Promise** aPromise) {
   // This keeps the promise alive after this method returns.
   nsMainThreadPtrHandle<dom::Promise> promiseHolder(
       new nsMainThreadPtrHolder<dom::Promise>(
-          "nsToolkitProfileService::AsyncFlushGroupProfile", promise));
+          "nsToolkitProfileService::AsyncFlush", promise));
 
   p->Then(GetCurrentSerialEventTarget(), __func__,
           [requestHolder, promiseHolder](
@@ -2794,6 +2761,15 @@ nsresult nsToolkitProfileService::GetLocalDirFromRootDir(nsIFile* aRootDir,
   return NS_OK;
 }
 
+bool nsToolkitProfileService::HasShowProfileSelector() {
+  for (RefPtr<nsToolkitProfile> profile : mProfiles) {
+    if (profile->GetShowProfileSelector()) {
+      return true;
+    }
+  }
+  return false;
+}
+
 already_AddRefed<nsToolkitProfileService> NS_GetToolkitProfileService() {
   if (!nsToolkitProfileService::gService) {
     nsToolkitProfileService::gService = new nsToolkitProfileService();
