@@ -11,17 +11,35 @@
 #include "base/task.h"            // for NewRunnableMethod, etc
 #include "mozilla/gfx/Logging.h"  // for gfxCriticalNote
 #include "mozilla/StaticMutex.h"
-#include "mozilla/Array.h"
 #include "mozilla/StaticPtr.h"
 #include "mozilla/ThreadLocal.h"
 #include "mozilla/StaticPrefs_widget.h"
 #include "mozilla/StaticPrefs_general.h"
 #include "mozilla/Sprintf.h"
 #include "WidgetUtilsGtk.h"
+#include "mozilla/widget/xx-pip-v1-client-protocol.h"
 #include "nsGtkKeyUtils.h"
+#include "nsGtkUtils.h"
+#include "nsLayoutUtils.h"
 #include "nsWindow.h"
 #include "wayland-proxy.h"
 
+#undef LOG
+#undef LOG_VERBOSE
+#ifdef MOZ_LOGGING
+#  include "mozilla/Logging.h"
+#  include "nsTArray.h"
+#  include "Units.h"
+extern mozilla::LazyLogModule gWidgetWaylandLog;
+#  define LOG(...) \
+    MOZ_LOG(gWidgetWaylandLog, mozilla::LogLevel::Debug, (__VA_ARGS__))
+#  define LOG_VERBOSE(...) \
+    MOZ_LOG(gWidgetWaylandLog, mozilla::LogLevel::Verbose, (__VA_ARGS__))
+#else
+#  define LOG(...)
+#  define LOG_VERBOSE(...)
+#endif /* MOZ_LOGGING */
+
 namespace mozilla::widget {
 
 static nsWaylandDisplay* gWaylandDisplay;
@@ -32,6 +50,7 @@ void WaylandDisplayRelease() {
   if (!gWaylandDisplay) {
     return;
   }
+  LOG("WaylandDisplayRelease()");
   delete gWaylandDisplay;
   gWaylandDisplay = nullptr;
 }
@@ -66,6 +85,8 @@ void nsWaylandDisplay::SetShm(wl_shm* aShm) { mShm = aShm; }
 
 class WaylandPointerEvent {
  public:
+  constexpr WaylandPointerEvent() = default;
+
   RefPtr<nsWindow> TakeWindow(wl_surface* aSurface) {
     if (!aSurface) {
       mWindow = nullptr;
@@ -100,7 +121,7 @@ class WaylandPointerEvent {
   void SetTime(uint32_t aTime) { mTime = aTime; }
 
   void SendScrollEvent() {
-    if (!mWindow || !StaticPrefs::general_smoothScroll()) {
+    if (!mWindow || !nsLayoutUtils::IsSmoothScrollingEnabled()) {
       return;
     }
 
@@ -125,8 +146,6 @@ class WaylandPointerEvent {
 
   void Clear() { mWindow = nullptr; }
 
-  WaylandPointerEvent() { Clear(); }
-
  private:
   StaticRefPtr<nsWindow> mWindow;
   uint32_t mTime = 0;
@@ -135,7 +154,7 @@ class WaylandPointerEvent {
   float mDeltaY = 0;
 };
 
-MOZ_RUNINIT static WaylandPointerEvent sHoldGesture;
+static WaylandPointerEvent sHoldGesture;
 
 static void gesture_hold_begin(void* data,
                                struct zwp_pointer_gesture_hold_v1* hold,
@@ -164,7 +183,10 @@ static void gesture_hold_end(void* data,
 static const struct zwp_pointer_gesture_hold_v1_listener gesture_hold_listener =
     {gesture_hold_begin, gesture_hold_end};
 
-MOZ_RUNINIT static WaylandPointerEvent sScrollEvent;
+static WaylandPointerEvent sScrollEvent;
+
+uint32_t gLastSerial = 0;
+uint32_t nsWaylandDisplay::GetLastEventSerial() { return gLastSerial; }
 
 static void pointer_handle_enter(void* data, struct wl_pointer* pointer,
                                  uint32_t serial, struct wl_surface* surface,
@@ -183,7 +205,11 @@ static void pointer_handle_motion(void* data, struct wl_pointer* pointer,
 
 static void pointer_handle_button(void* data, struct wl_pointer* pointer,
                                   uint32_t serial, uint32_t time,
-                                  uint32_t button, uint32_t state) {}
+                                  uint32_t button, uint32_t state) {
+  if (state == WL_POINTER_BUTTON_STATE_PRESSED) {
+    gLastSerial = serial;
+  }
+}
 
 static void pointer_handle_axis(void* data, struct wl_pointer* pointer,
                                 uint32_t time, uint32_t axis,
@@ -297,6 +323,13 @@ static void seat_handle_capabilities(void* data, struct wl_seat* seat,
   } else if (!(caps & WL_SEAT_CAPABILITY_KEYBOARD) && keyboard) {
     display->ClearKeyboard();
   }
+
+  wl_touch* touch = display->GetTouch();
+  if ((caps & WL_SEAT_CAPABILITY_TOUCH) && !touch) {
+    display->SetTouch(wl_seat_get_touch(seat));
+  } else if (!(caps & WL_SEAT_CAPABILITY_TOUCH) && touch) {
+    display->ClearTouch();
+  }
 }
 
 static void seat_handle_name(void* data, struct wl_seat* seat,
@@ -332,17 +365,20 @@ static void keyboard_handle_keymap(void* data, struct wl_keyboard* wl_keyboard,
 static void keyboard_handle_enter(void* data, struct wl_keyboard* keyboard,
                                   uint32_t serial, struct wl_surface* surface,
                                   struct wl_array* keys) {
-  KeymapWrapper::SetFocusIn(surface, serial);
+  gLastSerial = serial;
 }
 
 static void keyboard_handle_leave(void* data, struct wl_keyboard* keyboard,
                                   uint32_t serial, struct wl_surface* surface) {
-  KeymapWrapper::SetFocusOut(surface);
+  KeymapWrapper::ResetRepeatState();
 }
 
 static void keyboard_handle_key(void* data, struct wl_keyboard* keyboard,
                                 uint32_t serial, uint32_t time, uint32_t key,
                                 uint32_t state) {
+  if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
+    gLastSerial = serial;
+  }
   // hardware key code is +8.
   // https://gitlab.gnome.org/GNOME/gtk/-/blob/3.24.41/gdk/wayland/gdkdevice-wayland.c#L2341
   KeymapWrapper::KeyboardHandlerForWayland(serial, key + 8, state);
@@ -370,12 +406,46 @@ void nsWaylandDisplay::SetKeyboard(wl_keyboard* aKeyboard) {
 
 void nsWaylandDisplay::ClearKeyboard() {
   if (mKeyboard) {
-    wl_keyboard_destroy(mKeyboard);
+    wl_keyboard_release(mKeyboard);
     mKeyboard = nullptr;
     KeymapWrapper::ClearKeymap();
   }
 }
 
+static void touch_handle_down(void* data, struct wl_touch* touch,
+                              uint32_t serial, uint32_t time,
+                              struct wl_surface* surface, int32_t id,
+                              wl_fixed_t x, wl_fixed_t y) {
+  gLastSerial = serial;
+}
+
+static void touch_handle_up(void* data, struct wl_touch* touch, uint32_t serial,
+                            uint32_t time, int32_t id) {}
+
+static void touch_handle_motion(void* data, struct wl_touch* touch,
+                                uint32_t time, int32_t id, wl_fixed_t x,
+                                wl_fixed_t y) {}
+
+static void touch_handle_frame(void* data, struct wl_touch* touch) {}
+
+static void touch_handle_cancel(void* data, struct wl_touch* touch) {}
+
+static const struct wl_touch_listener touch_listener = {
+    touch_handle_down,  touch_handle_up,     touch_handle_motion,
+    touch_handle_frame, touch_handle_cancel,
+};
+
+void nsWaylandDisplay::SetTouch(wl_touch* aTouch) {
+  MOZ_ASSERT(aTouch);
+  MOZ_DIAGNOSTIC_ASSERT(!mTouch);
+  mTouch = aTouch;
+  wl_touch_add_listener(mTouch, &touch_listener, nullptr);
+}
+
+void nsWaylandDisplay::ClearTouch() {
+  MozClearPointer(mTouch, wl_touch_release);
+}
+
 void nsWaylandDisplay::SetCompositor(wl_compositor* aCompositor) {
   mCompositor = aCompositor;
 }
@@ -439,6 +509,7 @@ void nsWaylandDisplay::SetAppMenuManager(
 }
 
 void nsWaylandDisplay::SetCMSupportedFeature(uint32_t aFeature) {
+  LOG("nsWaylandDisplay::SetCMSupportedFeature() [%d]", aFeature);
   switch (aFeature) {
     case WP_COLOR_MANAGER_V1_FEATURE_ICC_V2_V4:
       mColorManagerSupportedFeature.mICC = true;
@@ -463,6 +534,7 @@ void nsWaylandDisplay::SetCMSupportedFeature(uint32_t aFeature) {
 
 void nsWaylandDisplay::SetCMSupportedTFNamed(uint32_t aTF) {
   if (aTF < sColorTransfersNum) {
+    LOG("nsWaylandDisplay::SetCMSupportedTFNamed() [%d]", aTF);
     mSupportedTransfer[aTF] = aTF;
   } else {
     NS_WARNING("Unknow color transfer function!");
@@ -471,6 +543,7 @@ void nsWaylandDisplay::SetCMSupportedTFNamed(uint32_t aTF) {
 
 void nsWaylandDisplay::SetCMSupportedPrimariesNamed(uint32_t aPrimaries) {
   if (aPrimaries < sColorPrimariesNum) {
+    LOG("nsWaylandDisplay::SetCMSupportedPrimariesNamed() [%u]", aPrimaries);
     mSupportedPrimaries[aPrimaries] = aPrimaries;
   } else {
     NS_WARNING("Unknown color primaries!");
@@ -513,11 +586,145 @@ static const struct wp_color_manager_v1_listener color_manager_listener = {
 void nsWaylandDisplay::SetColorManager(wp_color_manager_v1* aColorManager) {
   mColorManager = aColorManager;
   if (mColorManager) {
+    LOG("nsWaylandDisplay::SetColorManager()");
     wp_color_manager_v1_add_listener(mColorManager, &color_manager_listener,
                                      this);
   }
 }
 
+void nsWaylandDisplay::SetSupportedCoefficientsAndRanges(uint32_t aCoefficients,
+                                                         uint32_t aRange) {
+  if (aCoefficients < sSupportedRangesNum) {
+    LOG("nsWaylandDisplay::SetSupportedCoefficientsAndRanges(): coefficients "
+        "%d range %d",
+        aCoefficients, aRange);
+    mSupportedRanges[aCoefficients] += aRange;
+  }
+}
+
+uint32_t nsWaylandDisplay::GetColorRange(uint32_t aCoefficients,
+                                         bool aFullRange) {
+  if (aCoefficients >= sSupportedRangesNum) {
+    return 0;
+  }
+  auto range = mSupportedRanges[aCoefficients];
+  if (aFullRange) {
+    return range == sSupportedRangeBoth || range == sSupportedRangeFull
+               ? WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_FULL
+               : 0;
+  } else {
+    return range == sSupportedRangeBoth || range == sSupportedRangeLimited
+               ? WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_LIMITED
+               : 0;
+  }
+}
+
+static void supported_alpha_mode(
+    void* data,
+    struct wp_color_representation_manager_v1* color_representation_manager,
+    uint32_t alpha_mode) {}
+
+static void supported_coefficients_and_ranges(
+    void* data,
+    struct wp_color_representation_manager_v1* color_representation_manager,
+    uint32_t coefficients, uint32_t range) {
+  auto* display = static_cast<nsWaylandDisplay*>(data);
+  display->SetSupportedCoefficientsAndRanges(coefficients, range);
+}
+
+static void color_representation_done(
+    void* data,
+    struct wp_color_representation_manager_v1* color_representation_manager) {}
+
+static const struct wp_color_representation_manager_v1_listener
+    color_representation_listener = {
+        supported_alpha_mode,
+        supported_coefficients_and_ranges,
+        color_representation_done,
+};
+
+void nsWaylandDisplay::SetColorRepresentationManager(
+    wp_color_representation_manager_v1* aColorRepresentationManager) {
+  mColorRepresentationManager = aColorRepresentationManager;
+  if (mColorRepresentationManager) {
+    LOG("nsWaylandDisplay::SetColorRepresentationManager()");
+    wp_color_representation_manager_v1_add_listener(
+        mColorRepresentationManager, &color_representation_listener, this);
+  }
+}
+
+static void output_handle_geometry(void* data, struct wl_output* wl_output,
+                                   int x, int y, int physical_width,
+                                   int physical_height, int subpixel,
+                                   const char* make, const char* model,
+                                   int32_t transform) {
+  auto* monitor = static_cast<nsWaylandDisplay::MonitorConfig*>(data);
+  LOG("nsWaylandDisplay ID %d geometry position %d %d physical size %d %d "
+      "subpixel %d transform %d",
+      monitor->id, x, y, physical_width, physical_height, subpixel, transform);
+  monitor->x = x;
+  monitor->y = y;
+}
+
+static void output_handle_done(void* data, struct wl_output* wl_output) {}
+
+static void output_handle_scale(void* data, struct wl_output* wl_output,
+                                int32_t scale) {}
+
+static void output_handle_mode(void* data, struct wl_output* wl_output,
+                               uint32_t flags, int width, int height,
+                               int refresh) {
+  auto* monitor = static_cast<nsWaylandDisplay::MonitorConfig*>(data);
+  LOG("nsWaylandDisplay ID %d mode output size %d x %d", monitor->id, width,
+      height);
+  if ((flags & WL_OUTPUT_MODE_CURRENT) == 0) {
+    return;
+  }
+  monitor->pixelWidth = width;
+  monitor->pixelHeight = height;
+}
+
+static const struct wl_output_listener output_listener = {
+    output_handle_geometry,
+    output_handle_mode,
+    output_handle_done,
+    output_handle_scale,
+};
+
+void nsWaylandDisplay::AddWlOutput(wl_output* aWlOutput, int aId) {
+  wl_output_add_listener(aWlOutput, &output_listener, AddMonitorConfig(aId));
+}
+
+nsWaylandDisplay::MonitorConfig* nsWaylandDisplay::AddMonitorConfig(int aId) {
+  LOG("nsWaylandDisplay add monitor ID %d num %zu", aId, mMonitors.Length());
+  UniquePtr<MonitorConfig> monitor = MakeUnique<MonitorConfig>(aId);
+  mMonitors.AppendElement(std::move(monitor));
+  return mMonitors.LastElement().get();
+}
+
+bool nsWaylandDisplay::RemoveMonitorConfig(int aId) {
+  for (unsigned int i = 0; i < mMonitors.Length(); i++) {
+    if (mMonitors[i]->id == aId) {
+      LOG("nsWaylandDisplay remove Monitor ID %d num %d", aId, i);
+      mMonitors.RemoveElementAt(i);
+      return true;
+    }
+  }
+  return false;
+}
+
+nsWaylandDisplay::MonitorConfig* nsWaylandDisplay::GetMonitorConfig(int x,
+                                                                    int y) {
+  for (const auto& monitor : mMonitors) {
+    if (monitor->x == x && monitor->y == y) {
+      LOG("nsWaylandDisplay::GetMonitorConfig() %d, %d matches", x, y);
+      return monitor.get();
+    }
+  }
+  LOG("nsWaylandDisplay::GetMonitorConfig() %d, %d missing!", x, y);
+  return nullptr;
+}
+
 static void global_registry_handler(void* data, wl_registry* registry,
                                     uint32_t id, const char* interface,
                                     uint32_t version) {
@@ -599,6 +806,23 @@ static void global_registry_handler(void* data, wl_registry* registry,
     auto* colorManager = WaylandRegistryBind<wp_color_manager_v1>(
         registry, id, &wp_color_manager_v1_interface, version);
     display->SetColorManager(colorManager);
+  } else if (iface.EqualsLiteral("wp_color_representation_manager_v1")) {
+    auto* colorRepresentationManager =
+        WaylandRegistryBind<wp_color_representation_manager_v1>(
+            registry, id, &wp_color_representation_manager_v1_interface, 1);
+    display->SetColorRepresentationManager(colorRepresentationManager);
+  } else if (iface.EqualsLiteral("xx_pip_shell_v1")) {
+    auto* pipShell = WaylandRegistryBind<xx_pip_shell_v1>(
+        registry, id, &xx_pip_shell_v1_interface, version);
+    display->SetPipShell(pipShell);
+  } else if (iface.EqualsLiteral("xdg_wm_base")) {
+    auto* xdgWm = WaylandRegistryBind<xdg_wm_base>(
+        registry, id, &xdg_wm_base_interface, version);
+    display->SetXdgWm(xdgWm);
+  } else if (iface.EqualsLiteral("wl_output") && version > 1) {
+    auto* output =
+        WaylandRegistryBind<wl_output>(registry, id, &wl_output_interface, 2);
+    display->AddWlOutput(output, id);
   }
 }
 
@@ -608,13 +832,50 @@ static void global_registry_remover(void* data, wl_registry* registry,
   if (!display) {
     return;
   }
+  if (display->RemoveMonitorConfig(id)) {
+    return;
+  }
   display->RemoveSeat(id);
 }
 
 static const struct wl_registry_listener registry_listener = {
     global_registry_handler, global_registry_remover};
 
-nsWaylandDisplay::~nsWaylandDisplay() = default;
+nsWaylandDisplay::~nsWaylandDisplay() {
+  g_list_free_full(mAsyncRoundtrips, (GDestroyNotify)wl_callback_destroy);
+  MozClearPointer(mColorManager, wp_color_manager_v1_destroy);
+  MozClearPointer(mColorRepresentationManager,
+                  wp_color_representation_manager_v1_destroy);
+}
+
+void nsWaylandDisplay::AsyncRoundtripCallback(void* aData,
+                                              wl_callback* aCallback,
+                                              uint32_t aTime) {
+  auto* display = static_cast<nsWaylandDisplay*>(aData);
+  display->mAsyncRoundtrips =
+      g_list_remove(display->mAsyncRoundtrips, aCallback);
+  wl_callback_destroy(aCallback);
+}
+
+static const struct wl_callback_listener async_roundtrip_listener = {
+    nsWaylandDisplay::AsyncRoundtripCallback};
+
+void nsWaylandDisplay::RequestAsyncRoundtrip() {
+  LOG("nsWaylandDisplay::RequestAsyncRoundtrip()");
+  wl_callback* callback = wl_display_sync(mDisplay);
+  wl_callback_add_listener(callback, &async_roundtrip_listener, this);
+  mAsyncRoundtrips = g_list_append(mAsyncRoundtrips, callback);
+}
+
+void nsWaylandDisplay::WaitForAsyncRoundtrips() {
+  LOG("nsWaylandDisplay::WaitForAsyncRoundtrips()");
+  while (g_list_length(mAsyncRoundtrips) > 0) {
+    if (wl_display_dispatch(mDisplay) < 0) {
+      NS_WARNING("Failed to get events from Wayland display!");
+      return;
+    }
+  }
+}
 
 static void WlLogHandler(const char* format, va_list args) {
   char error[1000];
@@ -647,17 +908,24 @@ void WlCompositorCrashHandler() {
 
 nsWaylandDisplay::nsWaylandDisplay(wl_display* aDisplay)
     : mThreadId(PR_GetCurrentThread()), mDisplay(aDisplay) {
+  MOZ_DIAGNOSTIC_ASSERT(XRE_IsParentProcess());
+
   // GTK sets the log handler on display creation, thus we overwrite it here
   // in a similar fashion
   wl_log_set_handler_client(WlLogHandler);
 
+  LOG("nsWaylandDisplay::nsWaylandDisplay()");
+
   mFormats = new DMABufFormats();
   mRegistry = wl_display_get_registry(mDisplay);
   wl_registry_add_listener(mRegistry, &registry_listener, this);
   wl_display_roundtrip(mDisplay);
-  wl_display_roundtrip(mDisplay);
+  RequestAsyncRoundtrip();
+  WaitForAsyncRoundtrips();
   EnsureDMABufFormats();
 
+  LOG("nsWaylandDisplay::nsWaylandDisplay() init finished");
+
   for (auto& e : mSupportedTransfer) {
     e = -1;
   };
