@@ -101,7 +101,7 @@ struct TaskMarker : BaseMarkerType<TaskMarker> {
                                                MS::Location::MarkerTable};
   static constexpr const char* ChartLabel = "{marker.data.name}";
   static constexpr const char* TableLabel =
-      "{marker.name} - {marker.data.name} - priority: "
+      "{marker.data.name} - priority: "
       "{marker.data.priorityName} ({marker.data.priority})"
       " task: {marker.data.task}";
 
@@ -156,7 +156,7 @@ struct IncompleteTaskMarker : BaseMarkerType<IncompleteTaskMarker> {
                                                MS::Location::MarkerTable};
   static constexpr const char* ChartLabel = "{marker.data.name}";
   static constexpr const char* TableLabel =
-      "{marker.name} - {marker.data.name} - priority: "
+      "{marker.data.name} - priority: "
       "{marker.data.priorityName} ({marker.data.priority})"
       " task: {marker.data.task}";
 
@@ -517,8 +517,10 @@ void TaskController::AddTask(already_AddRefed<Task>&& aTask) {
 #endif
 
   LogTask::LogDispatch(task);
-  PROFILER_MARKER("TaskController::AddTask", OTHER, {}, FlowMarker,
-                  Flow::FromPointer(task.get()));
+  PROFILER_MARKER("TaskController::AddTask", OTHER,
+                  {MarkerStack::MaybeCapture(
+                      profiler_feature_active(ProfilerFeature::Flows))},
+                  FlowMarker, Flow::FromPointer(task.get()));
 
   std::pair<std::set<RefPtr<Task>, Task::PriorityCompare>::iterator, bool>
       insertion;
@@ -888,24 +890,37 @@ void ScheduleWantsLaterTimer(uint32_t aWantsLaterDelay) {
     sIdleMemoryCleanupRunner->Cancel();
     sIdleMemoryCleanupRunner = nullptr;
   }
+  nsresult timerInitOK = NS_OK;
   if (!sIdleMemoryCleanupWantsLater) {
     auto res = NS_NewTimerWithFuncCallback(
         CheckIdleMemoryCleanupNeeded, (void*)"IdleMemoryCleanupWantsLaterCheck",
         aWantsLaterDelay, nsITimer::TYPE_ONE_SHOT_LOW_PRIORITY,
-        "IdleMemoryCleanupWantsLaterCheck");
+        "IdleMemoryCleanupWantsLaterCheck"_ns);
     if (res.isOk()) {
       sIdleMemoryCleanupWantsLater = res.unwrap().forget();
+    } else {
+      timerInitOK = res.unwrapErr();
     }
   } else {
     if (sIdleMemoryCleanupWantsLaterScheduled) {
       sIdleMemoryCleanupWantsLater->Cancel();
     }
-    sIdleMemoryCleanupWantsLater->InitWithNamedFuncCallback(
+    timerInitOK = sIdleMemoryCleanupWantsLater->InitWithNamedFuncCallback(
         CheckIdleMemoryCleanupNeeded, (void*)"IdleMemoryCleanupWantsLaterCheck",
         aWantsLaterDelay, nsITimer::TYPE_ONE_SHOT_LOW_PRIORITY,
-        "IdleMemoryCleanupWantsLaterCheck");
+        "IdleMemoryCleanupWantsLaterCheck"_ns);
+  }
+  if (NS_SUCCEEDED(timerInitOK)) {
+    sIdleMemoryCleanupWantsLaterScheduled = true;
+  } else {
+    // Under normal conditions, we would never expect this to fail.
+    MOZ_ASSERT_UNREACHABLE(
+        "ScheduleWantsLaterTimer could not create the timer.");
+    // If we were not able to create/init the timer, we will retry the next
+    // time the main thread is about to fall idle. But if we were to stay
+    // idle, we would never purge without this emergency purge.
+    jemalloc_free_dirty_pages();
   }
-  sIdleMemoryCleanupWantsLaterScheduled = true;
 }
 
 void ScheduleIdleMemoryCleanup(uint32_t aWantsLaterDelay) {
@@ -919,10 +934,34 @@ void ScheduleIdleMemoryCleanup(uint32_t aWantsLaterDelay) {
       [aWantsLaterDelay](TimeStamp aDeadline) {
         return RunIdleMemoryCleanup(aDeadline, aWantsLaterDelay);
       },
-      "TaskController::IdlePurgeRunner", TimeDuration(), maxPurgeDelay,
+      "TaskController::IdlePurgeRunner"_ns, TimeDuration(), maxPurgeDelay,
       minPurgeBudget, true, nullptr, nullptr);
 }
+}  // namespace mozilla
 
+namespace geckoprofiler::markers {
+struct IdlePurgePeekMarker : mozilla::BaseMarkerType<IdlePurgePeekMarker> {
+  static constexpr const char* Name = "IdlePurgePeek";
+  static constexpr const char* Description = "Check if we should purge memory";
+
+  using MS = mozilla::MarkerSchema;
+  using String8View = mozilla::ProfilerString8View;
+
+  static constexpr MS::PayloadField PayloadFields[] = {
+      {"status", MS::InputType::CString, "Status", MS::Format::String}};
+
+  static void StreamJSONMarkerData(
+      mozilla::baseprofiler::SpliceableJSONWriter& aWriter,
+      const String8View& aStatus) {
+    aWriter.StringProperty("status", aStatus);
+  }
+
+  static constexpr MS::Location Locations[] = {MS::Location::MarkerChart,
+                                               MS::Location::MarkerTable};
+};
+}  // namespace geckoprofiler::markers
+
+namespace mozilla {
 // Check if a purge needs to be scheduled now or later.
 // Both used as timer callback and directly from MayScheduleIdleMemoryCleanup.
 //
@@ -934,12 +973,8 @@ void ScheduleIdleMemoryCleanup(uint32_t aWantsLaterDelay) {
 // (very cheap) check actually runs.
 //
 // aTimer:   Not used
-// aClosure: Supposed to point to a name literal to be used for profile
-//           markers.
+// aClosure: Not used
 void CheckIdleMemoryCleanupNeeded(nsITimer* aTimer, void* aClosure) {
-  MOZ_ASSERT(aClosure);
-  const char* name = (const char*)aClosure;
-
   uint32_t reuseGracePeriod =
       StaticPrefs::memory_lazypurge_reuse_grace_period();
 
@@ -955,7 +990,7 @@ void CheckIdleMemoryCleanupNeeded(nsITimer* aTimer, void* aClosure) {
   auto result =
       moz_may_purge_now(/* aPeekOnly */ true, reuseGracePeriod, Nothing());
   switch (result) {
-    case purge_result_t::Done:
+    case may_purge_now_result_t::Done:
       // Currently we unqueue purge requests only:
       // if we run moz_may_purge_one_now with aPeekOnly==false and that happens
       // only in the IdleTaskRunner which cancels itself when done
@@ -964,29 +999,33 @@ void CheckIdleMemoryCleanupNeeded(nsITimer* aTimer, void* aClosure) {
       // jemalloc_free_(excess)_dirty_pages or moz_set_max_dirty_page_modifier)
       // which can happen anytime.
       if (sIdleMemoryCleanupRunner || sIdleMemoryCleanupWantsLaterScheduled) {
-        PROFILER_MARKER_TEXT(
-            ProfilerString8View::WrapNullTerminatedString(name), OTHER, {},
-            "Done (Cancel timer or runner)"_ns);
+        PROFILER_MARKER("IdlePurgePeek", GCCC, MarkerTiming::InstantNow(),
+                        IdlePurgePeekMarker,
+                        ProfilerString8View::WrapNullTerminatedString(
+                            "Done (Cancel timer or runner)"));
         CancelIdleMemoryCleanupTimerAndRunner();
       }
       break;
-    case purge_result_t::WantsLater:
+    case may_purge_now_result_t::WantsLater:
       if (!sIdleMemoryCleanupWantsLaterScheduled) {
-        PROFILER_MARKER_TEXT(
-            ProfilerString8View::WrapNullTerminatedString(name), OTHER, {},
-            "WantsLater (First schedule of low priority timer)"_ns);
+        PROFILER_MARKER(
+            "IdlePurgePeek", GCCC, MarkerTiming::InstantNow(),
+            IdlePurgePeekMarker,
+            ProfilerString8View::WrapNullTerminatedString(
+                "WantsLater (First schedule of low priority timer)"));
       }
       // We always want to (re-)schedule the timer to prevent it from firing
       // as much as possible.
       ScheduleWantsLaterTimer(wantsLaterDelay);
       break;
-    case purge_result_t::NeedsMore:
+    case may_purge_now_result_t::NeedsMore:
       // We can get here from the main thread going repeatedly idle after we
       // already scheduled a runner. Just keep it.
       if (!sIdleMemoryCleanupRunner) {
-        PROFILER_MARKER_TEXT(
-            ProfilerString8View::WrapNullTerminatedString(name), OTHER, {},
-            "NeedsMore (Schedule as-soon-as-idle cleanup)"_ns);
+        PROFILER_MARKER("IdlePurgePeek", GCCC, MarkerTiming::InstantNow(),
+                        IdlePurgePeekMarker,
+                        ProfilerString8View::WrapNullTerminatedString(
+                            "NeedsMore (Schedule as-soon-as-idle cleanup)"));
         ScheduleIdleMemoryCleanup(wantsLaterDelay);
       } else {
         MOZ_ASSERT(!sIdleMemoryCleanupWantsLaterScheduled);
@@ -994,6 +1033,35 @@ void CheckIdleMemoryCleanupNeeded(nsITimer* aTimer, void* aClosure) {
       break;
   }
 }
+}  // namespace mozilla
+
+namespace geckoprofiler::markers {
+struct IdlePurgeMarker : mozilla::BaseMarkerType<IdlePurgeMarker> {
+  static constexpr const char* Name = "IdlePurge";
+  static constexpr const char* Description =
+      "Purge memory from mozjemalloc in idle time";
+
+  using MS = mozilla::MarkerSchema;
+  using String8View = mozilla::ProfilerString8View;
+
+  static constexpr MS::PayloadField PayloadFields[] = {
+      {"num_calls", MS::InputType::Uint32, "Number of PurgeNow() calls",
+       MS::Format::Integer},
+      {"next", MS::InputType::CString, "Last result", MS::Format::String}};
+
+  static void StreamJSONMarkerData(
+      mozilla::baseprofiler::SpliceableJSONWriter& aWriter, uint32_t aNumCalls,
+      const String8View& aLastResult) {
+    aWriter.IntProperty("num_calls", aNumCalls);
+    aWriter.StringProperty("last_result", aLastResult);
+  }
+
+  static constexpr MS::Location Locations[] = {MS::Location::MarkerChart,
+                                               MS::Location::MarkerTable};
+};
+}  // namespace geckoprofiler::markers
+
+namespace mozilla {
 
 // Do some purging until our idle budget is used.
 //
@@ -1008,39 +1076,44 @@ void CheckIdleMemoryCleanupNeeded(nsITimer* aTimer, void* aClosure) {
 //                   allowed to consume time.
 // aWantsLaterDelay: (Minimum) delay to be used for the WantsLater timer.
 bool RunIdleMemoryCleanup(TimeStamp aDeadline, uint32_t aWantsLaterDelay) {
-  AUTO_PROFILER_MARKER_TEXT("RunIdleMemoryCleanup", OTHER, {}, ""_ns);
-
   MOZ_ASSERT(!sIdleMemoryCleanupWantsLaterScheduled);
 
+  TimeStamp start_time = TimeStamp::Now();
+  uint32_t num_calls = 0;
+
   uint32_t reuseGracePeriod =
       StaticPrefs::memory_lazypurge_reuse_grace_period();
 
-  purge_result_t result;
+  may_purge_now_result_t result;
   do {
+    num_calls++;
     result = moz_may_purge_now(
         /* aPeekOnly */ false, reuseGracePeriod, Some([aDeadline] {
           return aDeadline.IsNull() || TimeStamp::Now() <= aDeadline;
         }));
-  } while ((result == purge_result_t::NeedsMore) &&
+  } while ((result == may_purge_now_result_t::NeedsMore) &&
            (aDeadline.IsNull() || TimeStamp::Now() <= aDeadline));
 
+  const char* last_result;
   switch (result) {
-    case purge_result_t::Done:
-      PROFILER_MARKER_TEXT("RunIdleMemoryCleanup", OTHER, {},
-                           "Done (Cancel timer and runner)"_ns);
+    case may_purge_now_result_t::Done:
+      last_result = "Done (Cancel timer and runner)";
       CancelIdleMemoryCleanupTimerAndRunner();
       break;
-    case purge_result_t::WantsLater:
-      PROFILER_MARKER_TEXT(
-          "RunIdleMemoryCleanup", OTHER, {},
-          "WantsLater (First schedule of low priority timer)"_ns);
+    case may_purge_now_result_t::WantsLater:
+      last_result = "WantsLater (First schedule of low priority timer)";
       ScheduleWantsLaterTimer(aWantsLaterDelay);
       break;
-    case purge_result_t::NeedsMore:
-      PROFILER_MARKER_TEXT("RunIdleMemoryCleanup", OTHER, {},
-                           "NeedsMore (wait for next idle slice)."_ns);
+    case may_purge_now_result_t::NeedsMore:
+      last_result = "NeedsMore (wait for next idle slice)";
       break;
   }
+
+  PROFILER_MARKER("IdlePurge", GCCC,
+                  MarkerTiming::IntervalUntilNowFrom(start_time),
+                  IdlePurgeMarker, num_calls,
+                  ProfilerString8View::WrapNullTerminatedString(last_result));
+
   return true;
 };
 
